@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,6 @@
 """Provides helper async functions."""
 
 import asyncio
-import functools
 from typing import Any, Coroutine, TypeVar
 import nest_asyncio
 
@@ -23,19 +22,6 @@ import nest_asyncio
 _T = TypeVar('_T')
 
 
-def as_async_function(func):
-  """Wraps a function to make it async."""
-
-  @functools.wraps(func)
-  async def run(*args, loop=None, executor=None, **kwargs):
-    if loop is None:
-      loop = asyncio.get_event_loop()
-    partial_func = functools.partial(func, *args, **kwargs)
-    return await loop.run_in_executor(executor, partial_func)
-
-  return run
-
-
 def run_sync(
     coro: Coroutine[Any, Any, _T],
     enable_nest_asyncio: bool = True,  # For testing.
