@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,12 +31,12 @@ import numpy as np
 import optax
 from orbax.checkpoint import args
 from orbax.checkpoint import test_utils
-from orbax.checkpoint import utils
 from orbax.checkpoint._src import asyncio_utils
 from orbax.checkpoint._src.checkpointers import async_checkpointer
 from orbax.checkpoint._src.checkpointers import checkpointer as checkpointer_lib
 from orbax.checkpoint._src.handlers import async_checkpoint_handler
 from orbax.checkpoint._src.handlers import composite_checkpoint_handler
+from orbax.checkpoint._src.handlers import json_checkpoint_handler
 from orbax.checkpoint._src.handlers import pytree_checkpoint_handler
 from orbax.checkpoint._src.logging import step_statistics
 from orbax.checkpoint._src.metadata import array_metadata
@@ -46,11 +46,14 @@ from orbax.checkpoint._src.metadata import step_metadata_serialization
 from orbax.checkpoint._src.metadata import tree as tree_metadata
 from orbax.checkpoint._src.multihost import multihost
 from orbax.checkpoint._src.path import atomicity
+from orbax.checkpoint._src.path import gcs_utils
 from orbax.checkpoint._src.path import step
-from orbax.checkpoint._src.serialization import serialization
+from orbax.checkpoint._src.serialization import limits
+from orbax.checkpoint._src.serialization import type_handler_registry
 from orbax.checkpoint._src.serialization import type_handlers
 from orbax.checkpoint._src.testing import test_tree_utils
 
+from .privacy.minimization.access_event_logger.gdm import gdm_access_logger
 from .pyglib import gfile
 
 
@@ -61,6 +64,7 @@ ArrayRestoreArgs = pytree_checkpoint_handler.ArrayRestoreArgs
 TrainState = train_state.TrainState
 Mesh = jax.sharding.Mesh
 PyTree = Any
+PLACEHOLDER = type_handlers.PLACEHOLDER
 
 
 class CheckpointerTestBase:
@@ -109,7 +113,7 @@ class CheckpointerTestBase:
       if not gfile.Exists(path):
         return False
       mode = gfile.Stat(path, stat_proto=True).mode
-      return (mode & step.NON_WORLD_READABLE_MODE) != mode
+      return (mode & 0o770) != mode
 
     def test_save_restore(self):
       """Basic save and restore test."""
@@ -233,12 +237,14 @@ class CheckpointerTestBase:
         paths = list(self.directory.parent.iterdir())
         self.assertLen(paths, 1)
         tmp_dir = paths[0]
-        self.assertFalse(utils.is_checkpoint_finalized(tmp_dir))
-        self.assertTrue(utils.is_tmp_checkpoint(tmp_dir))
+        self.assertFalse(step.is_path_finalized(tmp_dir))
+        self.assertTrue(step.is_path_temporary(tmp_dir))
         with self.assertRaisesRegex(ValueError, 'Found incomplete checkpoint'):
           checkpointer.restore(tmp_dir)
 
-    @mock.patch.object(step, 'is_gcs_path', autospec=True, return_value=True)
+    @mock.patch.object(
+        gcs_utils, 'is_gcs_path', autospec=True, return_value=True
+    )
     def test_gcs(self, is_gcs_path):
       """Test normal operation in simulated GCS environment."""
       del is_gcs_path
@@ -250,9 +256,11 @@ class CheckpointerTestBase:
             path, restore_args=self.pytree_restore_args
         )
         test_utils.assert_tree_equal(self, self.pytree, restored)
-        self.assertTrue((path / step._COMMIT_SUCCESS_FILE).exists())
+        self.assertTrue((path / atomicity.COMMIT_SUCCESS_FILE).exists())
 
-    @mock.patch.object(step, 'is_gcs_path', autospec=True, return_value=True)
+    @mock.patch.object(
+        gcs_utils, 'is_gcs_path', autospec=True, return_value=True
+    )
     @mock.patch.object(atomicity.CommitFileTemporaryPath, 'finalize')
     def test_save_preempted_gcs(self, *mock_args):
       """Simulate effects of preemption."""
@@ -263,11 +271,11 @@ class CheckpointerTestBase:
         paths = list(self.directory.parent.iterdir())
         self.assertLen(paths, 1)
         tmp_dir = paths[0]
-        self.assertFalse(utils.is_checkpoint_finalized(tmp_dir))
-        self.assertTrue(utils.is_tmp_checkpoint(tmp_dir))
+        self.assertFalse(step.is_path_finalized(tmp_dir))
+        self.assertTrue(step.is_path_temporary(tmp_dir))
         with self.assertRaisesRegex(ValueError, 'Found incomplete checkpoint'):
           checkpointer.restore(tmp_dir)
-        self.assertFalse((tmp_dir / step._COMMIT_SUCCESS_FILE).exists())
+        self.assertFalse((tmp_dir / atomicity.COMMIT_SUCCESS_FILE).exists())
 
     def test_configure_atomicity(self):
       """Test case."""
@@ -282,7 +290,7 @@ class CheckpointerTestBase:
             path, restore_args=self.pytree_restore_args
         )
         test_utils.assert_tree_equal(self, self.pytree, restored)
-        self.assertTrue((path / step._COMMIT_SUCCESS_FILE).exists())  # pylint: disable=protected-access
+        self.assertTrue((path / atomicity.COMMIT_SUCCESS_FILE).exists())  # pylint: disable=protected-access
 
     def test_legacy_api(self):
       """Test case."""
@@ -342,7 +350,7 @@ class CheckpointerTestBase:
 
       byte_limiter = test_utils.get_byte_limiter(limit_bytes, sleep_time)
       with mock.patch.object(
-          serialization,
+          limits,
           'get_byte_limiter',
           new=lambda _: byte_limiter,
       ):
@@ -363,7 +371,7 @@ class CheckpointerTestBase:
 
       byte_limiter = test_utils.get_byte_limiter(limit_bytes, sleep_time)
       with mock.patch.object(
-          serialization,
+          limits,
           'get_byte_limiter',
           new=lambda _: byte_limiter,
       ):
@@ -523,11 +531,79 @@ class CheckpointerTestBase:
         self.assertGreater(step_metadata.commit_timestamp_nsecs, 0)
         self.assertEqual(step_metadata.custom_metadata, {'a': 1, 'b': 2})
 
+    def test_save_and_get_metadata(self):
+      """Basic save and restore test."""
+      with self.checkpointer(PyTreeCheckpointHandler()) as checkpointer:
+        directory = self.directory / 'step_metadata'
+        metadata = {'a': 1, 'b': 2}
+        checkpointer.save(
+            directory, args.PyTreeSave(self.pytree), custom_metadata=metadata
+        )
+        self.wait_if_async(checkpointer)
+        retrieved_metadata = checkpointer.metadata(directory)
+        self.assertEqual(retrieved_metadata.custom_metadata, metadata)
+
+    def test_composite_handler_metadata(self):
+      """Tests Checkpointer.metadata with CompositeCheckpointHandler."""
+      handler = composite_checkpoint_handler.CompositeCheckpointHandler()
+      with self.checkpointer(handler) as ckptr:
+        # Some test classes use a mock handler.
+        if not isinstance(
+            ckptr.handler,
+            composite_checkpoint_handler.CompositeCheckpointHandler,
+        ):
+          self.skipTest('Test only applies to CompositeCheckpointHandler.')
+
+        directory = self.directory / 'composite_metadata'
+        custom_metadata = {'user': 'test', 'version': 1}
+        item_args = args.Composite(
+            state=args.PyTreeSave(self.pytree),
+            other=args.JsonSave({'foo': 'bar'}),
+        )
+        ckptr.save(
+            directory,
+            item_args,
+            custom_metadata=custom_metadata,
+        )
+        self.wait_if_async(ckptr)
+
+        metadata = ckptr.metadata(directory)
+        self.assertIsInstance(metadata, checkpoint.StepMetadata)
+        self.assertEqual(metadata.custom_metadata, custom_metadata)
+
+        self.assertIsNotNone(metadata.item_handlers)
+        self.assertIn('state', metadata.item_handlers)
+        self.assertEqual(
+            metadata.item_handlers['state'],
+            PyTreeCheckpointHandler().typestr(),
+        )
+        self.assertIn('other', metadata.item_handlers)
+        self.assertEqual(
+            metadata.item_handlers['other'],
+            json_checkpoint_handler.JsonCheckpointHandler().typestr(),
+        )
+
+        self.assertIsNotNone(metadata.item_metadata)
+        self.assertIn('state', metadata.item_metadata)
+        self.assertIn('other', metadata.item_metadata)
+        self.assertIsInstance(
+            metadata.item_metadata['state'], tree_metadata.TreeMetadata
+        )
+        expected_pytree_metadata = PyTreeCheckpointHandler().metadata(
+            directory / 'state'
+        )
+        test_utils.assert_tree_equal(
+            self,
+            metadata.item_metadata['state'],
+            expected_pytree_metadata,
+        )
+        self.assertIsNone(metadata.item_metadata['other'])
+
     async def test_save_with_failing_array_metadata_store_in_finalize(self):
       """ArrayMetadata validation in CheckpointHandler.finalize()."""
 
-      class IncompleteArrayMetadataSerDeserializer(
-          array_metadata_store_lib.SerDeserializer
+      class IncompleteArrayMetadataSerializer(
+          array_metadata_store_lib.Serializer
       ):
         """process 0 metadata should be complete, others should be incomplete.
 
@@ -545,15 +621,15 @@ class CheckpointerTestBase:
             return super().serialize([array_metadatas[0]])  # incomplete
 
       array_metadata_store = array_metadata_store_lib.Store(
-          ser_deser=IncompleteArrayMetadataSerDeserializer()
+          serializer=IncompleteArrayMetadataSerializer()
       )
-      type_handler_registry = copy.deepcopy(
-          type_handlers.GLOBAL_TYPE_HANDLER_REGISTRY
+      registry = copy.deepcopy(
+          type_handler_registry.GLOBAL_TYPE_HANDLER_REGISTRY
       )
       array_handler = type_handlers.ArrayHandler(
           array_metadata_store=array_metadata_store
       )
-      type_handler_registry.add(jax.Array, array_handler, override=True)
+      registry.add(jax.Array, array_handler, override=True)
 
       class InterceptingValidator(array_metadata_store_lib.Validator):
         """Intercepts ArrayMetadata validation in CheckpointHandler.finalize()."""
@@ -574,7 +650,7 @@ class CheckpointerTestBase:
       array_metadata_validator = InterceptingValidator()
       with self.checkpointer(
           PyTreeCheckpointHandler(
-              type_handler_registry=type_handler_registry,
+              type_handler_registry=registry,
               array_metadata_validator=array_metadata_validator,
           )
       ) as checkpointer:
@@ -587,3 +663,129 @@ class CheckpointerTestBase:
         ):
           array_metadata_validator.maybe_raise_error()
       self.assertIsNotNone(await array_metadata_store.read(self.directory))
+
+    def test_partial_restore_with_placeholder(self):
+      """Basic save and restore test."""
+      with self.checkpointer(PyTreeCheckpointHandler()) as save_checkpointer:
+        directory = self.directory / 'partial_restore'
+        save_checkpointer.save(
+            directory,
+            args.PyTreeSave(self.pytree),
+        )
+
+      with self.subTest('success'):
+        reference_item = self.empty_pytree.copy()
+        reference_item['b'] = PLACEHOLDER
+        reference_item['c']['e'] = PLACEHOLDER
+
+        expected = self.pytree.copy()
+        expected['b'] = PLACEHOLDER
+        expected['c']['e'] = PLACEHOLDER
+
+        with self.checkpointer(
+            PyTreeCheckpointHandler()
+        ) as restore_checkpointer:
+          restored = restore_checkpointer.restore(
+              directory,
+              args=pytree_checkpoint_handler.PyTreeRestoreArgs(
+                  item=reference_item,
+                  restore_args=self.pytree_restore_args,
+              ),
+          )
+          test_utils.assert_tree_equal(self, expected, restored)
+
+      with self.subTest('missing_leaf'):
+        reference_item = self.empty_pytree.copy()
+        reference_item['b'] = PLACEHOLDER
+        reference_item['c']['e'] = PLACEHOLDER
+        del reference_item['c']['a']
+
+        with self.checkpointer(
+            PyTreeCheckpointHandler()
+        ) as restore_checkpointer:
+          with self.assertRaisesRegex(
+              ValueError, 'User-provided restore item and on-disk value'
+          ):
+            restore_checkpointer.restore(
+                directory,
+                args=pytree_checkpoint_handler.PyTreeRestoreArgs(
+                    item=reference_item,
+                    restore_args=self.pytree_restore_args,
+                ),
+            )
+
+      with self.subTest('non_leaf_placeholder'):
+        reference_item = self.empty_pytree.copy()
+        reference_item['c'] = PLACEHOLDER
+
+        with self.checkpointer(
+            PyTreeCheckpointHandler()
+        ) as restore_checkpointer:
+          with self.assertRaisesRegex(
+              ValueError, 'User-provided restore item and on-disk value'
+          ):
+            restore_checkpointer.restore(
+                directory,
+                args=pytree_checkpoint_handler.PyTreeRestoreArgs(
+                    item=reference_item,
+                    restore_args=self.pytree_restore_args,
+                ),
+            )
+
+    def test_partial_restore_with_omission(self):
+      """Basic save and restore test."""
+      with self.checkpointer(PyTreeCheckpointHandler()) as save_checkpointer:
+        directory = self.directory / 'partial_restore'
+        save_checkpointer.save(
+            directory,
+            args.PyTreeSave(self.pytree),
+        )
+
+      with self.subTest('success'):
+        with self.checkpointer(
+            PyTreeCheckpointHandler()
+        ) as restore_checkpointer:
+          reference_item = {
+              'a': 0,
+              'c': {
+                  'a': 0,
+              },
+          }
+          expected = {
+              'a': self.pytree['a'],
+              'c': {
+                  'a': self.pytree['c']['a'],
+              },
+          }
+          restored = restore_checkpointer.restore(
+              directory,
+              args=pytree_checkpoint_handler.PyTreeRestoreArgs(
+                  item=reference_item,
+                  restore_args=self.pytree_restore_args,
+                  partial_restore=True,
+              ),
+          )
+          test_utils.assert_tree_equal(self, expected, restored)
+
+    def test_restore_logs_read_event(self):
+      """Tests that restore logs a read event to DM Sawmill log."""
+      with self.checkpointer(PyTreeCheckpointHandler()) as checkpointer:
+        checkpointer.save(self.directory, self.pytree)
+        self.wait_if_async(checkpointer)
+
+        with mock.patch.object(
+            gdm_access_logger, 'GdmAccessLogger', autospec=True
+        ) as mock_gdm_access_logger:
+          # Create instance of GdmAccessLogger mock.
+          mock_instance = mock_gdm_access_logger()
+
+          restored = checkpointer.restore(
+              self.directory, restore_args=self.pytree_restore_args
+          )
+
+          mock_instance.read_event.assert_called_with(
+              checkpoint_path=str(self.directory),
+              log_provenance='orbax',
+          )
+
+        test_utils.assert_tree_equal(self, self.pytree, restored)
