@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,10 +35,13 @@ class PyTreeCheckpointer(checkpointer.Checkpointer):
       primary_host: Optional[int] = 0,
       use_ocdbt: bool = True,
       use_zarr3=False,
+      use_compression: bool = True,
   ):
     super().__init__(
         pytree_checkpoint_handler.PyTreeCheckpointHandler(
-            use_ocdbt=use_ocdbt, use_zarr3=use_zarr3
+            use_ocdbt=use_ocdbt,
+            use_zarr3=use_zarr3,
+            use_compression=use_compression,
         ),
         multiprocessing_options=options_lib.MultiprocessingOptions(
             primary_host=primary_host
