@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -48,7 +48,9 @@ from __future__ import annotations
 import asyncio
 import concurrent.futures
 import dataclasses
-from typing import Any, Coroutine, Dict, List, Mapping, MutableSet, Optional, Tuple, Type
+import threading
+import time
+from typing import Any, Dict, List, Mapping, MutableSet, Optional, Tuple, Type
 
 from absl import logging
 from etils import epath
@@ -63,13 +65,14 @@ from orbax.checkpoint._src.handlers import async_checkpoint_handler
 from orbax.checkpoint._src.handlers import checkpoint_handler
 from orbax.checkpoint._src.handlers import handler_registration
 from orbax.checkpoint._src.handlers import handler_type_registry
-from orbax.checkpoint._src.handlers import proto_checkpoint_handler
 from orbax.checkpoint._src.metadata import checkpoint
 from orbax.checkpoint._src.metadata import step_metadata_serialization
+from orbax.checkpoint._src.multihost import multihost
 from orbax.checkpoint._src.path import atomicity
 from orbax.checkpoint._src.path import atomicity_defaults
 from orbax.checkpoint._src.path import atomicity_types
 
+
 CheckpointArgs = checkpoint_args.CheckpointArgs
 Future = future.Future
 CheckpointArgs = checkpoint_args.CheckpointArgs
@@ -78,10 +81,6 @@ StepMetadata = checkpoint.StepMetadata
 CompositeItemMetadata = checkpoint.CompositeItemMetadata
 AsyncCheckpointHandler = async_checkpoint_handler.AsyncCheckpointHandler
 register_with_handler = checkpoint_args.register_with_handler
-ProtoCheckpointHandler = proto_checkpoint_handler.ProtoCheckpointHandler
-ProtoSaveArgs = proto_checkpoint_handler.ProtoSaveArgs
-ProtoRestoreArgs = proto_checkpoint_handler.ProtoRestoreArgs
-AsyncSaveCoroutine = Coroutine[Any, Any, Optional[List[Future]]]
 Composite = composite.Composite
 HandlerAwaitableSignal = synchronization.HandlerAwaitableSignal
 
@@ -89,7 +88,7 @@ HandlerAwaitableSignal = synchronization.HandlerAwaitableSignal
 _CONCURRENT_WORKERS = 3
 
 DESCRIPTOR_ITEM_NAME = 'descriptor'
-RESERVED_ITEM_NAMES = [DESCRIPTOR_ITEM_NAME]
+RESERVED_ITEM_NAMES = []
 _DIRECTORY_CREATION_SIGNALS = [HandlerAwaitableSignal.ITEM_DIRECTORY_CREATION]
 
 
@@ -255,24 +254,24 @@ def _add_to_handler_registry_from_global_registry(
 class CompositeCheckpointHandler(AsyncCheckpointHandler):
   """CheckpointHandler for saving multiple items.
 
-  As with all :py:class:`CheckpointHandler` implementations, use only in
-  conjunction with an instance of `AbstractCheckpointer`.
+  As with all :py:class:`.CheckpointHandler` implementations, use only in
+  conjunction with an instance of :py:class:`.AbstractCheckpointer`.
 
-  The :py:class:`CompositeCheckpointHandler` allows dealing with multiple items
+  The :py:class:`.CompositeCheckpointHandler` allows dealing with multiple items
   of different types or logical distinctness, such as training state (PyTree),
   dataset iterator, JSON metadata, or anything else. To specify the handler
   and args for each checkpointable item, use the `handler_registry` option at
-  initialization (see :py:class:`CheckpointHandlerRegistry`).
+  initialization (see :py:class:`.CheckpointHandlerRegistry`).
 
-  If a handler registry is provided, the :py:class:`CompositeCheckpointHandler`
-  will use this registry to determine the :py:class:`CheckpointHandler` for each
-  checkpointable item. If no registry is provided, an empty registry will be
-  constructed by default internally. Additional items not registered in the
+  If a handler registry is provided, the :py:class:`.CompositeCheckpointHandler`
+  will use this registry to determine the :py:class:`.CheckpointHandler` for
+  each checkpointable item. If no registry is provided, an empty registry will
+  be constructed by default internally. Additional items not registered in the
   handler registry can be specified when calling `save` or `restore`. The
-  handler will be determined from the provided :py:class:`CheckpointArgs` during
-  the first call to `save` or `restore`. Subsequent calls to `save` or `restore`
-  will use the same handler and will require the same :py:class:`CheckpointArgs`
-  to be provided.
+  handler will be determined from the provided :py:class:`.CheckpointArgs`
+  during the first call to `save` or `restore`. Subsequent calls to `save` or
+  `restore` will use the same handler and will require the same
+  :py:class:`.CheckpointArgs` to be provided.
 
   `item_names` and `items_and_handlers` are deprecated arguments for specifying
   the items and handlers. Please use `handler_registry` instead.
@@ -639,7 +638,6 @@ class CompositeCheckpointHandler(AsyncCheckpointHandler):
     )
     tmp_item_dir = temporary_path_class.from_final(
         self._get_item_directory(directory, item_name),
-        multiprocessing_options=self._multiprocessing_options,
         file_options=self._file_options,
     )
     return tmp_item_dir
@@ -671,6 +669,7 @@ class CompositeCheckpointHandler(AsyncCheckpointHandler):
       self, directory: epath.Path, args: CompositeArgs
   ) -> Optional[List[Future]]:
     """Saves multiple items to individual subdirectories."""
+    start_time = time.time()
     self._current_temporary_paths = self._get_item_temporary_paths(
         directory, args
     )
@@ -684,12 +683,17 @@ class CompositeCheckpointHandler(AsyncCheckpointHandler):
           )
       )
     else:
-      await atomicity.create_all(
-          list(self._current_temporary_paths.values()),
-          multiprocessing_options=self._multiprocessing_options,
-      )
+      future.CommitFutureAwaitingContractedSignals(
+          atomicity.create_all(
+              list(self._current_temporary_paths.values()),
+              multiprocessing_options=self._multiprocessing_options,
+          )
+      ).result()
     save_ops = []
+    time_taken_per_item = {}
+    time_taken_all_items = 0.0
     for item_name, item_directory in self._current_temporary_paths.items():
+      item_start_time = time.time()
       arg = args[item_name]
       _maybe_raise_reserved_item_error(item_name)
       handler = self._get_or_set_handler(item_name, arg)
@@ -697,11 +701,28 @@ class CompositeCheckpointHandler(AsyncCheckpointHandler):
         save_ops.append(handler.async_save(item_directory.get(), args=arg))
       else:
         # Blocking save.
-        handler.save(item_directory.get(), args=arg)
+        future.CommitFutureAwaitingContractedSignals(
+            asyncio.to_thread(handler.save, item_directory.get(), args=arg)
+        ).result()
+      item_end_time = time.time() - item_start_time
+      time_taken_per_item[item_name] = f'{item_end_time:.8f}'
+      time_taken_all_items += item_end_time
 
     commit_futures.extend(
         jax.tree.flatten(await asyncio.gather(*save_ops))[0] or []
     )
+    end_time = time.time()
+    logging.info(
+        '[process=%s][thread=%s] Initiated'
+        ' CompositeCheckpointHandler.async_save. Time taken: %fs'
+        ' (all_items=%fs, per_item=%s, temp_paths=%f)',
+        multihost.process_index(),
+        threading.current_thread().name,
+        end_time - start_time,
+        time_taken_all_items,
+        time_taken_per_item,
+        end_time - start_time - time_taken_all_items,
+    )
     return commit_futures
 
   def save(self, *args, **kwargs):
@@ -845,7 +866,12 @@ class CompositeCheckpointHandler(AsyncCheckpointHandler):
       items_to_handlers: dict[str, CheckpointHandler | None],
   ) -> dict[str, checkpoint.CheckpointHandlerTypeStr | None]:
     if saved_metadata.item_handlers is not None:
-      assert isinstance(saved_metadata.item_handlers, dict)
+      if not isinstance(saved_metadata.item_handlers, dict):
+        raise ValueError(
+            'StepMetadata.item_handlers must be a dict, but is a'
+            f' {type(saved_metadata.item_handlers)}. This likely indicates that'
+            ' the checkpoint was saved using a non-composite handler.'
+        )
       # Keep relevant non-None handler typestrs from on-disk metadata.
       item_handlers: dict[str, checkpoint.CheckpointHandlerTypeStr] = (
           saved_metadata.item_handlers
@@ -895,9 +921,18 @@ class CompositeCheckpointHandler(AsyncCheckpointHandler):
 
       handler = items_to_handlers[item_name]
       assert handler is not None
-      item_metadata[item_name] = handler.metadata(
-          self._get_item_directory(directory, item_name)
-      )
+      try:
+        item_metadata[item_name] = handler.metadata(
+            self._get_item_directory(directory, item_name)
+        )
+      except FileNotFoundError:
+        logging.warning(
+            'Item "%s" was found in the checkpoint, but could not'
+            ' be restored. Please provide a `CheckpointHandlerRegistry`, or'
+            ' call `restore` with an appropriate `CheckpointArgs` subclass.',
+            item_name,
+        )
+        item_metadata[item_name] = None
     return item_metadata
 
   def _get_metadata_base(
@@ -1009,7 +1044,9 @@ class CompositeCheckpointHandler(AsyncCheckpointHandler):
         # Not an error, as some items may not have been saved.
         continue
       handler.finalize(tmp_dir.get())
-      tmp_dir.finalize(
+      asyncio.run(
+          tmp_dir.finalize(
+          )
       )
 
       # Remove the temporary path once it has been finalized.
