@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,6 +31,8 @@ from orbax.checkpoint._src.metadata import checkpoint
 from orbax.checkpoint._src.metadata import step_metadata_serialization
 from orbax.checkpoint._src.metadata import value as value_metadata
 from orbax.checkpoint._src.multihost import multihost
+from orbax.checkpoint._src.path import atomicity
+from orbax.checkpoint._src.path import gcs_utils
 from orbax.checkpoint._src.path import step
 
 CompositeArgs = composite_checkpoint_handler.CompositeArgs
@@ -62,10 +64,10 @@ class CompositeCheckpointHandlerTest(parameterized.TestCase):
   def setUp(self):
     super().setUp()
     self.directory = epath.Path(self.create_tempdir(name='test_dir'))
-    synchronization.HandlerAwaitableSignalOperationIdGenerator.next_operation_id()
+    synchronization.OperationIdGenerator.next_operation_id()
 
   def save(self, handler, directory, *args, **kwargs):
-    synchronization.HandlerAwaitableSignalOperationIdGenerator.next_operation_id()
+    synchronization.OperationIdGenerator.next_operation_id()
     handler.save(directory, *args, **kwargs)
     if multihost.process_index() == 0:
       handler.finalize(directory)
@@ -816,6 +818,24 @@ class CompositeCheckpointHandlerTest(parameterized.TestCase):
     self.assertIsNone(step_metadata.commit_timestamp_nsecs)
     self.assertEmpty(step_metadata.custom_metadata)
 
+    metadata_handler = CompositeCheckpointHandler(handler_registry=registry)
+    restored_metadata = metadata_handler.metadata(self.directory)
+    self.assertDictEqual(
+        dict(restored_metadata.item_metadata.state.tree),
+        {
+            'a': value_metadata.ScalarMetadata(
+                name='a',
+                directory=self.directory / 'state',
+                dtype=jnp.int64,
+            ),
+            'b': value_metadata.ScalarMetadata(
+                name='b',
+                directory=self.directory / 'state',
+                dtype=jnp.int64,
+            ),
+        },
+    )
+
   def test_metadata_after_step_metadata_write(self):
     handler = CompositeCheckpointHandler(
         'extra',
@@ -1001,10 +1021,10 @@ class CompositeCheckpointHandlerTest(parameterized.TestCase):
       state_handler.finalize.assert_called_once()
       metadata_handler.finalize.assert_not_called()
       self.assertFalse(
-          (self.directory / 'state' / step._COMMIT_SUCCESS_FILE).exists()
+          (self.directory / 'state' / atomicity.COMMIT_SUCCESS_FILE).exists()
       )
 
-  @mock.patch.object(step, 'is_gcs_path', autospec=True, return_value=True)
+  @mock.patch.object(gcs_utils, 'is_gcs_path', autospec=True, return_value=True)
   def test_finalize_gcs(self, is_gcs_path):
     del is_gcs_path
     state_handler = mock.create_autospec(StandardCheckpointHandler)
@@ -1024,7 +1044,7 @@ class CompositeCheckpointHandlerTest(parameterized.TestCase):
       )
       state_handler.finalize.assert_called_once()
       self.assertTrue(
-          (self.directory / 'state' / step._COMMIT_SUCCESS_FILE).exists()
+          (self.directory / 'state' / atomicity.COMMIT_SUCCESS_FILE).exists()
       )
 
   def test_close(self):
@@ -1083,11 +1103,11 @@ class CompositeCheckpointHandlerTest(parameterized.TestCase):
     self.assertIn('state', tmp_dirs.keys())
     self.assertIn('metadata', tmp_dirs.keys())
     self.assertIn(
-        (self.directory / 'state.orbax-checkpoint-tmp-').as_posix(),
+        (self.directory / 'state.orbax-checkpoint-tmp').as_posix(),
         tmp_dirs['state'].get().as_posix(),
     )
     self.assertIn(
-        (self.directory / 'metadata.orbax-checkpoint-tmp-').as_posix(),
+        (self.directory / 'metadata.orbax-checkpoint-tmp').as_posix(),
         tmp_dirs['metadata'].get().as_posix(),
     )
 
@@ -1127,6 +1147,31 @@ class CompositeCheckpointHandlerTest(parameterized.TestCase):
     ):
       handler.metadata(self.directory)
 
+  def test_metadata_with_missing_metadata_file(self):
+    # Simulate a case where an item is not a PyTree, but the handler
+    # a PyTreeHandler.
+    handler = CompositeCheckpointHandler(
+        state=StandardCheckpointHandler(), datasets=StandardCheckpointHandler()
+    )
+    state = {'a': 1, 'b': 2}
+    self.save(
+        handler,
+        self.directory,
+        CompositeArgs(
+            state=args_lib.StandardSave(state),
+        ),
+    )
+
+    # Create dummy files in datasets to simulate a non-pytree item
+    (self.directory / 'datasets').mkdir()
+    (self.directory / 'datasets' / 'data.txt').write_text('some data')
+
+    step_metadata = handler.metadata(self.directory)
+    self.assertIn('state', step_metadata.item_metadata)
+    self.assertIsNotNone(step_metadata.item_metadata['state'])
+    self.assertIn('datasets', step_metadata.item_metadata)
+    self.assertIsNone(step_metadata.item_metadata['datasets'])
+
 
 if __name__ == '__main__':
   absltest.main()
