@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,9 +14,9 @@
 
 """PyTreeCheckpointHandler class.
 
-Implementation of `CheckpointHandler` interface dealing with JAX PyTrees. Much
-of the underlying reading/writing logic for individual leaf types can be
-customized, and is delegated to the `TypeHandler` class.
+Implementation of :py:class:`.CheckpointHandler` interface dealing with JAX
+PyTrees. Much of the underlying reading/writing logic for individual leaf types
+can be customized, and is delegated to the :py:class:`.TypeHandler` class.
 """
 
 from __future__ import annotations
@@ -44,9 +44,11 @@ from orbax.checkpoint._src.handlers import base_pytree_checkpoint_handler
 from orbax.checkpoint._src.metadata import array_metadata_store as array_metadata_store_lib
 from orbax.checkpoint._src.metadata import empty_values
 from orbax.checkpoint._src.metadata import tree as tree_metadata
-from orbax.checkpoint._src.serialization import serialization
+from orbax.checkpoint._src.serialization import limits
 from orbax.checkpoint._src.serialization import tensorstore_utils as ts_utils
+from orbax.checkpoint._src.serialization import type_handler_registry as handler_registry
 from orbax.checkpoint._src.serialization import type_handlers
+from orbax.checkpoint._src.serialization import types as serialization_types
 from orbax.checkpoint._src.tree import types as tree_types
 from orbax.checkpoint._src.tree import utils as tree_utils
 import tensorstore as ts
@@ -57,9 +59,9 @@ TupleKey = Tuple[str, ...]
 RestoreArgs = type_handlers.RestoreArgs
 ArrayRestoreArgs = type_handlers.ArrayRestoreArgs
 SaveArgs = type_handlers.SaveArgs
-ParamInfo = type_handlers.ParamInfo
-TypeHandler = type_handlers.TypeHandler
-TypeHandlerRegistry = type_handlers.TypeHandlerRegistry
+ParamInfo = serialization_types.ParamInfo
+TypeHandler = serialization_types.TypeHandler
+TypeHandlerRegistry = serialization_types.TypeHandlerRegistry
 AggregateHandler = aggregate_handlers.AggregateHandler
 MsgpackHandler = aggregate_handlers.MsgpackHandler
 LegacyTransformFn = Callable[[PyTree, PyTree, PyTree], Tuple[PyTree, PyTree]]
@@ -191,6 +193,7 @@ def _get_restore_parameters(
     byte_limiter: Optional[LimitInFlightBytes] = None,
     transforms_default_to_original: bool = True,
     use_zarr3: bool = False,
+    partial_restore: bool = False,
 ) -> Tuple[PyTree, PyTree]:
   """Construct parameters needed for restoration.
 
@@ -229,6 +232,7 @@ def _get_restore_parameters(
     byte_limiter: A LimitInFlightBytes object.
     transforms_default_to_original: See transform_utils.apply_transformations.
     use_zarr3: If True, use Zarr ver3 otherwise Zarr ver2
+    partial_restore: If True, only restore the parameters present in structure.
 
   Returns:
     Tuple of param_infos, and restore_args.
@@ -242,6 +246,7 @@ def _get_restore_parameters(
   flat_restore_args = tree_utils.to_flat_dict(
       restore_args, keep_empty_nodes=True
   )
+  flat_item = tree_utils.to_flat_dict(item, keep_empty_nodes=True)
   flat_param_infos = {}
   flat_input_restore_args = {}
   is_ocdbt_checkpoint = type_handlers.is_ocdbt_checkpoint(directory)
@@ -263,7 +268,6 @@ def _get_restore_parameters(
       skip_deserialize = meta_or_value.skip_deserialize
     return ParamInfo(
         name=name,
-        path=directory / name,
         parent_dir=directory,
         skip_deserialize=skip_deserialize,
         is_ocdbt_checkpoint=is_ocdbt_checkpoint,
@@ -275,7 +279,20 @@ def _get_restore_parameters(
         raise_array_data_missing_error=False,
     )
 
-  if transforms is None:
+  if partial_restore:
+    for key, meta in flat_structure.items():
+      if key not in flat_item:
+        flat_param_infos[key] = ParamInfo(
+            name='', parent_dir=directory, skip_deserialize=True
+        )
+        flat_input_restore_args[key] = RestoreArgs()
+      else:
+        flat_param_infos[key] = _get_param_info(flat_param_names[key], meta)
+        flat_input_restore_args[key] = flat_restore_args[key]
+    restore_args = tree_utils.from_flat_dict(
+        flat_input_restore_args, target=structure
+    )
+  elif transforms is None:
     for key, meta in flat_structure.items():
       flat_param_infos[key] = _get_param_info(flat_param_names[key], meta)
     restore_args = tree_metadata.serialize_tree(
@@ -287,7 +304,6 @@ def _get_restore_parameters(
           'If providing `transforms`, must provide `item` matching structure'
           ' of expected result.'
       )
-    flat_item = tree_utils.to_flat_dict(item, keep_empty_nodes=True)
     flat_transforms = tree_utils.to_flat_dict(transforms)
 
     for input_key, meta in flat_structure.items():
@@ -307,7 +323,9 @@ def _get_restore_parameters(
             # Specified `use_fallback`, but key was also present in the
             # checkpoint. This means we should skip loading, since it will be
             # overridden with a new value.
-            flat_param_infos[input_key] = ParamInfo(skip_deserialize=True)
+            flat_param_infos[input_key] = ParamInfo(
+                name='', parent_dir=directory, skip_deserialize=True
+            )
             flat_input_restore_args[input_key] = RestoreArgs()
           else:
             # Specified `use_fallback`, but `transforms_default_to_original`
@@ -328,12 +346,16 @@ def _get_restore_parameters(
           else:
             # Take the value from the user-provided `item`, ignoring any value
             # in the checkpoint.
-            flat_param_infos[input_key] = ParamInfo(skip_deserialize=True)
+            flat_param_infos[input_key] = ParamInfo(
+                name='', parent_dir=directory, skip_deserialize=True
+            )
             flat_input_restore_args[input_key] = RestoreArgs()
       else:
         # No match, restoration not required since it will be dropped from the
         # output.
-        flat_param_infos[input_key] = ParamInfo(skip_deserialize=True)
+        flat_param_infos[input_key] = ParamInfo(
+            name='', parent_dir=directory, skip_deserialize=True
+        )
         flat_input_restore_args[input_key] = RestoreArgs()
 
     restore_args = tree_utils.from_flat_dict(
@@ -432,14 +454,18 @@ def _get_impl_save_args(
       item=args.item,
       save_args=args.save_args,
       ocdbt_target_data_file_size=args.ocdbt_target_data_file_size,
-      enable_pinned_host_transfer=args.enable_pinned_host_transfer,
       custom_metadata=args.custom_metadata,
   )
 
 
-def _concurrent_bytes(concurrent_gb: Optional[int]) -> int:
+def _concurrent_bytes(
+    concurrent_gb: int | None, *, use_default_if_none: bool = True
+) -> int | None:
   if concurrent_gb is None:
-    return DEFAULT_CONCURRENT_GB * 10**9
+    if use_default_if_none:
+      return DEFAULT_CONCURRENT_GB * 10**9
+    else:
+      return None
   else:
     return concurrent_gb * 10**9
 
@@ -449,13 +475,14 @@ class PyTreeCheckpointHandler(async_checkpoint_handler.AsyncCheckpointHandler):
 
   See JAX documentation for more information on what consistutes a "PyTree".
   This handler is capable of saving and restoring any leaf object for which a
-  `TypeHandler` (see documentation) is registered. By default, `TypeHandler`s
-  for standard types like `np.ndarray`, `jax.Array`, Python scalars, and others
-  are registered.
+  :py:class:`.TypeHandler` (see documentation) is registered. By default,
+  `TypeHandler`s for standard types like `np.ndarray`, `jax.Array`, Python
+  scalars, and others are registered.
 
-  As with all `CheckpointHandler` subclasses, `PyTreeCheckpointHandler` should
-  only be used in conjunction with a `Checkpointer` (or subclass). By itself,
-  the `CheckpointHandler` is non-atomic.
+  As with all :py:class:`.CheckpointHandler` subclasses,
+  `PyTreeCheckpointHandler` should only be used in conjunction with a
+  :py:class:`.Checkpointer` (or subclass). By itself, the `CheckpointHandler` is
+  non-atomic.
 
   Example::
 
@@ -470,10 +497,14 @@ class PyTreeCheckpointHandler(async_checkpoint_handler.AsyncCheckpointHandler):
       *,
       save_concurrent_gb: Optional[int] = None,
       restore_concurrent_gb: Optional[int] = None,
+      save_device_host_concurrent_gb: Optional[int] = None,
       use_ocdbt: bool = True,
       use_zarr3: bool = False,
+      use_compression: bool = True,
       multiprocessing_options: options_lib.MultiprocessingOptions = options_lib.MultiprocessingOptions(),
-      type_handler_registry: TypeHandlerRegistry = type_handlers.GLOBAL_TYPE_HANDLER_REGISTRY,
+      type_handler_registry: TypeHandlerRegistry = (
+          handler_registry.GLOBAL_TYPE_HANDLER_REGISTRY
+      ),
       handler_impl: Optional[BasePyTreeCheckpointHandler] = None,
       pytree_metadata_options: tree_metadata.PyTreeMetadataOptions = (
           tree_metadata.PYTREE_METADATA_OPTIONS
@@ -481,6 +512,10 @@ class PyTreeCheckpointHandler(async_checkpoint_handler.AsyncCheckpointHandler):
       array_metadata_validator: array_metadata_store_lib.Validator = (
           array_metadata_store_lib.Validator()
       ),
+      enable_pinned_host_transfer: Optional[bool] = None,
+      is_prioritized_key_fn: Optional[
+          serialization_types.IsPrioritizedKeyFn
+      ] = None,
   ):
     """Creates PyTreeCheckpointHandler.
 
@@ -493,16 +528,42 @@ class PyTreeCheckpointHandler(async_checkpoint_handler.AsyncCheckpointHandler):
       restore_concurrent_gb: max concurrent GB that are allowed for writing. Can
         help to reduce the possibility of OOM's when large checkpoints are
         restored.
+      save_device_host_concurrent_gb: max concurrent GB allowed to be
+        transferred from device to host memory at once when saving, defined on a
+        per-worker basis. When the limit is reached, arrays must be finished
+        writing to the checkpoint before a new array can start being
+        transferred. This option is a stronger version of `save_concurrent_gb`,
+        which only attempts to limit memory usage after all shards have already
+        been transferred to host memory. Note that asynchronous saves may not be
+        truly asynchronous with this option enabled, as we have to block on some
+        array writes before beginning others.
       use_ocdbt: enables Tensorstore OCDBT driver. This option allows using a
         different checkpoint format which is faster to read and write, as well
         as more space efficient.
       use_zarr3: If True, use Zarr ver3 otherwise Zarr ver2
+      use_compression: If True and zarr2 is used, use zstd compression.
       multiprocessing_options: See orbax.checkpoint.options
       type_handler_registry: a type_handlers.TypeHandlerRegistry. If not
         specified, the global type handler registry will be used.
       handler_impl: Allows overriding the internal implementation.
       pytree_metadata_options: `PyTreeMetadataOptions` to manage metadata.
       array_metadata_validator: Validator for ArrayMetadata.
+      enable_pinned_host_transfer: Whether to use pinned_host memory for the
+        transfer from device to host memory. Passing None will enable
+        pinned_host memory depending on the platform used (currently only
+        enables it for the GPU backend).
+      is_prioritized_key_fn: A function that accepts a PyTree keypath (obtained
+        using jax.tree.map_with_path) that should be scheduled for D2H transfer
+        before other keys. The transfer is scheduled before returning to the
+        caller, so the values will never be corrupted by a concurrent update.
+        Keys that are not prioritized will not be scheduled for transfer until
+        all prioritized keys have been fully written to the checkpoint. This
+        means that these values may be altered if the values are updated
+        concurrently. Callers should take care to call `wait_until_finished`
+        before updating array values (e.g. `apply_gradients`) if some keys are
+        not prioritized. Note that any "prioritized" keys are assumed to be
+        lightweight, and `save_device_host_concurrent_gb` will be ignored for
+        them.
     """
     self._aggregate_handler = MsgpackHandler(
         primary_host=multiprocessing_options.primary_host,
@@ -517,15 +578,26 @@ class PyTreeCheckpointHandler(async_checkpoint_handler.AsyncCheckpointHandler):
     self._type_handler_registry = type_handler_registry
     self._save_concurrent_bytes = _concurrent_bytes(save_concurrent_gb)
     self._restore_concurrent_bytes = _concurrent_bytes(restore_concurrent_gb)
+    self._save_device_host_concurrent_bytes = _concurrent_bytes(
+        save_device_host_concurrent_gb, use_default_if_none=False
+    )
+    logging.info(
+        'save_device_host_concurrent_bytes=%s',
+        self._save_device_host_concurrent_bytes,
+    )
     self._handler_impl = handler_impl or BasePyTreeCheckpointHandler(
         save_concurrent_bytes=self._save_concurrent_bytes,
         restore_concurrent_bytes=self._restore_concurrent_bytes,
+        save_device_host_concurrent_bytes=self._save_device_host_concurrent_bytes,
         use_ocdbt=use_ocdbt,
         use_zarr3=use_zarr3,
+        use_compression=use_compression,
         multiprocessing_options=multiprocessing_options,
-        type_handler_registry=type_handler_registry,
+        type_handler_registry=self._type_handler_registry,
         pytree_metadata_options=pytree_metadata_options,
         array_metadata_validator=array_metadata_validator,
+        enable_pinned_host_transfer=enable_pinned_host_transfer,
+        is_prioritized_key_fn=is_prioritized_key_fn,
     )
     self._pytree_metadata_options = pytree_metadata_options
 
@@ -601,9 +673,7 @@ class PyTreeCheckpointHandler(async_checkpoint_handler.AsyncCheckpointHandler):
       restore_args: PyTree,
   ) -> PyTree:
     """Deserializes values or gets them from the aggregate file."""
-    byte_limiter = serialization.get_byte_limiter(
-        self._restore_concurrent_bytes
-    )
+    byte_limiter = limits.get_byte_limiter(self._restore_concurrent_bytes)
     param_infos = jax.tree.map(
         lambda info: dataclasses.replace(info, byte_limiter=byte_limiter),
         param_infos,
@@ -792,6 +862,7 @@ class PyTreeCheckpointHandler(async_checkpoint_handler.AsyncCheckpointHandler):
       args = BasePyTreeRestoreArgs(
           item,
           restore_args=restore_args,
+          partial_restore=args.partial_restore,
       )
       return self._handler_impl.restore(directory, args=args)
 
@@ -909,7 +980,9 @@ class PyTreeCheckpointHandler(async_checkpoint_handler.AsyncCheckpointHandler):
     """
     # Try reading metadata file.
     try:
-      internal_tree_metadata = self._handler_impl._read_metadata_file(directory)  # pylint: disable=protected-access
+      internal_tree_metadata = asyncio_utils.run_sync(
+          self._handler_impl._read_metadata_file(directory)  # pylint: disable=protected-access
+      )
       use_zarr3 = internal_tree_metadata.use_zarr3
       value_metadata_tree = internal_tree_metadata.as_nested_tree()
       flat_value_metadatas = tree_utils.to_flat_dict(
@@ -1057,9 +1130,6 @@ class PyTreeSaveArgs(CheckpointArgs):
       indicates no maximum file size limit.  For best results, ensure
       chunk_byte_size is smaller than this value.  For more details, refer to
       https://google.github.io/tensorstore/kvstore/ocdbt/index.html#json-kvstore/ocdbt.target_data_file_size
-    enable_pinned_host_transfer: If False, disables transfer to
-      pinned host when copying from device to host, regardless of the presence
-      of pinned host memory.
     custom_metadata: User-provided custom metadata. An arbitrary
       JSON-serializable dictionary the user can use to store additional
       information. The field is treated as opaque by Orbax.
@@ -1068,7 +1138,6 @@ class PyTreeSaveArgs(CheckpointArgs):
   item: PyTree
   save_args: Optional[PyTree] = None
   ocdbt_target_data_file_size: Optional[int] = None
-  enable_pinned_host_transfer: bool = False
   custom_metadata: tree_types.JsonType | None = None
 
   def __post_init__(self):
@@ -1103,12 +1172,15 @@ class PyTreeRestoreArgs(CheckpointArgs):
       regexes and implicit keys means that it does not need to match
       completely. See `transform_utils` for further information.
       `TreeMetadata` is also allowed as the `transforms` tree.
-    transforms_default_to_original: See transform_utils.apply_transformations.
+    transforms_default_to_original:
+      See transform_utils.apply_transformations.
     legacy_transform_fn: WARNING: NOT GENERALLY SUPPORTED. A function which
       accepts the `item` argument, a PyTree checkpoint structure and a PyTree
       of ParamInfos based on the checkpoint. Returns a transformed PyTree
       matching the desired return tree structure, and a matching ParamInfo
       tree.
+    partial_restore: If True, only restore the parameters that are specified
+      in PyTreeRestoreArgs.
   """
 
   item: Optional[PyTree] = None
@@ -1116,6 +1188,7 @@ class PyTreeRestoreArgs(CheckpointArgs):
   transforms: Optional[PyTree] = None
   transforms_default_to_original: bool = True
   legacy_transform_fn: Optional[LegacyTransformFn] = None
+  partial_restore: bool = False
 
   def __post_init__(self):
     if isinstance(self.item, tree_metadata.TreeMetadata):
