@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -49,7 +49,7 @@ class RandomKeyCheckpointHandlerTest(absltest.TestCase):
     self.directory = epath.Path(
         self.create_tempdir(name='checkpointing_test').full_path
     )
-    synchronization.HandlerAwaitableSignalOperationIdGenerator.next_operation_id()
+    synchronization.OperationIdGenerator.next_operation_id()
 
   def assert_dict_equal(self, left, right):
     """Asserts that two dicts are equal and allow np.ndarray as elements."""
