@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,11 +19,13 @@ import json
 import threading
 import time
 from typing import Any, Iterator, List, Sequence, Tuple
+
 from absl import logging
 from etils import epath
 import jax
 from orbax.checkpoint._src.metadata import array_metadata as array_metadata_lib
 from orbax.checkpoint._src.multihost import multihost
+from orbax.checkpoint._src.path import async_path
 from orbax.checkpoint._src.serialization import types
 
 
@@ -53,7 +55,7 @@ class PathResolver:
         checkpoint_dir / self._metadata_subdir / self._file_name(process_index)
     )
 
-  def get_read_file_paths(
+  async def get_read_file_paths(
       self, checkpoint_dir: epath.Path, process_index: int | None = None
   ) -> Iterator[epath.Path] | epath.Path | None:
     """Returns the file paths to read.
@@ -70,16 +72,16 @@ class PathResolver:
     """
     if process_index is None:
       file_name_pattern = self._file_name('*')
-      return checkpoint_dir.glob(f'{self._metadata_subdir}/{file_name_pattern}')
-    file_path = (
-        checkpoint_dir / self._metadata_subdir / self._file_name(process_index)
-    )
-    if file_path.exists():
+      return await async_path.glob(
+          checkpoint_dir, f'{self._metadata_subdir}/{file_name_pattern}'
+      )
+    file_path = self.get_write_file_path(checkpoint_dir, process_index)
+    if await async_path.exists(file_path):
       return file_path
     return None
 
 
-class SerDeserializer:
+class Serializer:
   """Serializes and deserializes `array_metadata.ArrayMetadata`."""
 
   def _to_dict(
@@ -123,7 +125,7 @@ class SerDeserializer:
   ) -> List[array_metadata_lib.SerializedArrayMetadata]:
     """Deserializes `serialized` to `array_metadata.ArrayMetadata`."""
     obj = json.loads(serialized, object_hook=self._from_dict)
-    return obj['array_metadatas']
+    return obj.get('array_metadatas', [])
 
 
 class Store:
@@ -132,12 +134,12 @@ class Store:
   def __init__(
       self,
       path_resolver: PathResolver = PathResolver(),
-      ser_deser: SerDeserializer = SerDeserializer(),
+      serializer: Serializer = Serializer(),
       primary_host: int | None = 0,  # None means all hosts are primary hosts.
       write_timeout_secs: int = 600,  # 10 minutes.
   ):
     self._path_resolver = path_resolver
-    self._ser_deser = ser_deser
+    self._serializer = serializer
     self._primary_host = primary_host
     self._write_timeout_secs = write_timeout_secs
 
@@ -149,13 +151,11 @@ class Store:
     """Primary host creates the base directory, rest of the hosts wait."""
     if multihost.is_primary_host(self._primary_host):
       # primary host creates, rest of the hosts wait.
-      return await asyncio.to_thread(
-          base_dir.mkdir, parents=True, exist_ok=True
-      )
+      return await async_path.mkdir(base_dir, parents=True, exist_ok=True)
 
     # non-primary host waits for primary host to create the base dir/folder.
     async def wait_for_base_dir_creation():
-      while not await asyncio.to_thread(base_dir.exists):
+      while not await async_path.exists(base_dir):
         await asyncio.sleep(0.25)
 
     try:
@@ -192,8 +192,13 @@ class Store:
         checkpoint_dir, process_index
     )
     await self._maybe_create_base_dir(file_path.parent)
-    await asyncio.to_thread(
-        file_path.write_text, self._ser_deser.serialize(array_metadatas)
+    if await async_path.exists(file_path):
+      _, all_array_metadatas = await self._get_array_metadatas(file_path)
+      all_array_metadatas.extend(array_metadatas)
+    else:
+      all_array_metadatas = array_metadatas
+    await async_path.write_text(
+        file_path, self._serializer.serialize(all_array_metadatas)
     )
     logging.info(
         '[process=%s][thread=%s] Wrote %d array_metadata.ArrayMetadata to %s',
@@ -207,8 +212,8 @@ class Store:
       self,
       array_metadatas_file_path: epath.Path,
   ) -> Tuple[epath.Path, list[array_metadata_lib.SerializedArrayMetadata]]:
-    serialized = await asyncio.to_thread(array_metadatas_file_path.read_text)
-    return array_metadatas_file_path, self._ser_deser.deserialize(serialized)
+    serialized = await async_path.read_text(array_metadatas_file_path)
+    return array_metadatas_file_path, self._serializer.deserialize(serialized)
 
   async def read(
       self,
@@ -231,16 +236,16 @@ class Store:
       is None. A list of metadata if `process_index` is not None. None if
       metadata does not exist.
     """
-    if not await asyncio.to_thread(checkpoint_dir.exists):
+    if not await async_path.exists(checkpoint_dir):
       raise ValueError(
           f'Checkpoint directory does not exist: {checkpoint_dir}.'
       )
     start_time = time.time()
-    file_paths = self._path_resolver.get_read_file_paths(
+    file_paths = await self._path_resolver.get_read_file_paths(
         checkpoint_dir, process_index
     )
     if file_paths is None:
-      logging.warning(
+      logging.info(
           '[process=%s][thread=%s] No metadata found for process_index=%s,'
           ' checkpoint_dir=%s. If the checkpoint does not contain jax.Array'
           ' then it is expected. If checkpoint contains jax.Array then it'
@@ -257,8 +262,8 @@ class Store:
       _, result = await self._get_array_metadatas(file_paths)
       logging.vlog(
           1,
-          '[process=%s][thread=%s] Read %s metadata from metadata path=%s'
-          ' in %s seconds.',
+          '[process=%s][thread=%s] Read %s metadata from metadata path=%s.'
+          ' Time taken: %fs.',
           multihost.process_index(),
           threading.current_thread().name,
           len(result),
@@ -276,9 +281,9 @@ class Store:
         for file_path, metadatas in path_metadatas_pairs
     }
     if not result:
-      logging.warning(
+      logging.info(
           '[process=%s][thread=%s] No metadata found for any process_index,'
-          ' checkpoint_dir=%s. time elapsed=%s seconds. If the checkpoint does'
+          ' checkpoint_dir=%s. Time taken: %fs. If the checkpoint does'
           ' not contain jax.Array then it is expected. If checkpoint contains'
           ' jax.Array then it should lead to an error eventually; if no error'
           ' is raised then it is a bug.',
@@ -291,8 +296,8 @@ class Store:
 
     logging.vlog(
         1,
-        '[process=%s][thread=%s] Read all metadata from checkpoint_dir=%s in %s'
-        ' seconds.',
+        '[process=%s][thread=%s] Read all metadata from checkpoint_dir=%s. Time'
+        ' taken: %fs.',
         multihost.process_index(),
         threading.current_thread().name,
         checkpoint_dir,
@@ -359,7 +364,7 @@ class Validator:
     if len(array_metadatas) == 1:
       # check if the number of processes are indeed just one.
       self._validate_process_count(ref_process_index=ref_process_index)
-      logging.warning(
+      logging.info(
           '[process=%s][thread=%s] Skipped cross-host ArrayMetadata validation'
           ' because only one process is found: process_index=%s.',
           multihost.process_index(),
@@ -404,8 +409,8 @@ class Validator:
           process_cache=process_cache,
       )
     logging.info(
-        '[process=%s][thread=%s] Validated ArrayMetadata from all %s hosts in'
-        ' %s seconds.',
+        '[process=%s][thread=%s] Validated ArrayMetadata from all %s hosts.'
+        ' Time taken: %fs.',
         multihost.process_index(),
         threading.current_thread().name,
         len(array_metadatas),
