@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ import numpy as np
 from orbax.checkpoint import test_utils
 from orbax.checkpoint._src.metadata import array_metadata as array_metadata_lib
 from orbax.checkpoint._src.metadata import array_metadata_store as array_metadata_store_lib
+from orbax.checkpoint._src.serialization import type_handler_registry
 from orbax.checkpoint._src.serialization import type_handlers
 
 
@@ -90,6 +91,58 @@ class StoreTest(parameterized.TestCase, unittest.IsolatedAsyncioTestCase):
         ],
     )
 
+  async def test_append_to_existing_metadata_file(self):
+    process_index = 0
+    initial_array_metadatas = [
+        array_metadata_lib.ArrayMetadata(
+            param_name='a',
+            shape=(10, 20, 30),
+            dtype=np.dtype(int),
+            write_shape=(10, 20, 30),
+            chunk_shape=(1, 2, 3),
+            use_ocdbt=False,
+            use_zarr3=False,
+        ),
+    ]
+    await self.store.write(
+        self.checkpoint_dir,
+        initial_array_metadatas,
+        process_index=process_index,
+    )
+
+    appended_array_metadatas = [
+        array_metadata_lib.ArrayMetadata(
+            param_name='b',
+            shape=(1, 1, 1),
+            dtype=np.dtype(int),
+            write_shape=(1, 1, 1),
+            chunk_shape=(1, 1, 1),
+            use_ocdbt=False,
+            use_zarr3=False,
+        ),
+    ]
+    await self.store.write(
+        self.checkpoint_dir,
+        appended_array_metadatas,
+        process_index=process_index,
+    )
+
+    self.assertEqual(
+        await self.store.read(self.checkpoint_dir, process_index=process_index),
+        [
+            array_metadata_lib.SerializedArrayMetadata(
+                param_name='a',
+                write_shape=(10, 20, 30),
+                chunk_shape=(1, 2, 3),
+            ),
+            array_metadata_lib.SerializedArrayMetadata(
+                param_name='b',
+                write_shape=(1, 1, 1),
+                chunk_shape=(1, 1, 1),
+            ),
+        ],
+    )
+
   async def test_write_and_read_multiple_process(self):
     for process_index in [0, 1, 2]:
       array_metadatas = [
@@ -150,17 +203,17 @@ class ResolveArrayMetadataStoreTest(parameterized.TestCase):
     fn = lambda ty: issubclass(ty, jax.Array)
     with test_utils.register_type_handler(ty, array_handler, fn):
       store = array_metadata_store_lib.resolve_array_metadata_store(
-          type_handlers.GLOBAL_TYPE_HANDLER_REGISTRY
+          type_handler_registry.GLOBAL_TYPE_HANDLER_REGISTRY
       )
       self.assertIs(store, array_metadata_store)
 
   def test_no_jax_array_type_handler(self):
-    type_handler_registry = type_handlers.create_type_handler_registry(
+    type_handler_reg = type_handler_registry.create_type_handler_registry(
         (int, type_handlers.ScalarHandler())
     )
 
     store = array_metadata_store_lib.resolve_array_metadata_store(
-        type_handler_registry
+        type_handler_reg
     )
 
     self.assertIsNone(store)
@@ -175,12 +228,12 @@ class ResolveArrayMetadataStoreTest(parameterized.TestCase):
     self.assertFalse(
         hasattr(WithoutArrayMetadataStore(), '_array_metadata_store')
     )
-    type_handler_registry = type_handlers.create_type_handler_registry(
+    type_handler_reg = type_handler_registry.create_type_handler_registry(
         (jax.Array, WithoutArrayMetadataStore())
     )
 
     store = array_metadata_store_lib.resolve_array_metadata_store(
-        type_handler_registry
+        type_handler_reg
     )
 
     self.assertIsNone(store)
