@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,12 +35,18 @@ PyTree = Any
 
 @dataclasses.dataclass
 class CheckpointInfo:
-  """Metadata about a checkpoint."""
+  """Basic information relevant to a checkpoint step.
+
+  Attributes:
+    step: Step number.
+    time: Time at which the checkpoint was saved. This should be treated as an
+      approximate start time of the save operation.
+    metrics: Metrics associated with the step, if provided.
+  """
 
   step: int
   time: datetime.datetime
   metrics: PyTree | None
-  is_locked: bool | None = None
 
   def __post_init__(self):
     # Users may provide step as a jax.Array.
@@ -48,10 +54,7 @@ class CheckpointInfo:
       self.step = int(self.step)
 
   def __str__(self) -> str:
-    return (
-        f'Checkpoint[step={self.step} | time={self.time} |'
-        f' is_locked={self.is_locked}]'
-    )
+    return f'Checkpoint[step={self.step} | time={self.time}]'
 
   def __eq__(self, other: CheckpointInfo) -> bool:
     return self.step == other.step and self.time == other.time
