@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -94,7 +94,11 @@ class InternalCheckpointMetadata:
       cls, metadata_dict: SerializedMetadata
   ) -> InternalCheckpointMetadata:
     """Deserializes `metadata_dict` to `InternalCheckpointMetadata`."""
-    assert isinstance(metadata_dict, dict)
+    if not isinstance(metadata_dict, dict):
+      raise ValueError(
+          'Metadata dict must be a dictionary, but got'
+          f' {type(metadata_dict).__name__}'
+      )
     fields = dataclasses.fields(InternalCheckpointMetadata)
     field_names = {field.name for field in fields}
 
@@ -113,7 +117,11 @@ class InternalCheckpointMetadata:
         **validated_metadata_dict['performance_metrics']
     )
 
-    assert isinstance(validated_metadata_dict['custom_metadata'], dict)
+    if not isinstance(validated_metadata_dict['custom_metadata'], dict):
+      raise ValueError(
+          'Custom metadata must be a dictionary, but got'
+          f' {type(validated_metadata_dict["custom_metadata"]).__name__}'
+      )
     for k in metadata_dict:
       if k not in validated_metadata_dict:
         validated_metadata_dict['custom_metadata'][k] = (
@@ -156,6 +164,26 @@ class InternalCheckpointMetadata:
         custom_metadata=step_metadata.custom_metadata,
     )
 
+  # TODO(b/407609701): Add `to_checkpoint_metadata` (like `to_step_metadata`)
+  # for conversion to V1 metadata after dependencies are separated.
+
+  @classmethod
+  def create(
+      cls,
+      *,
+      handler_typestrs: dict[str, str],
+      init_timestamp_nsecs: int,
+      commit_timestamp_nsecs: int,
+      custom_metadata: dict[str, Any] | None = None,
+  ) -> InternalCheckpointMetadata:
+    """Return internal representation, dropping fields handled separately."""
+    return cls(
+        item_handlers=handler_typestrs,
+        init_timestamp_nsecs=init_timestamp_nsecs,
+        commit_timestamp_nsecs=commit_timestamp_nsecs,
+        custom_metadata=custom_metadata,
+    )
+
 
 def get_step_metadata(path: epath.PathLike) -> StepMetadata:
   """Returns StepMetadata for a given checkpoint directory."""
