@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,6 +14,7 @@
 
 """Orbax utils related to multihost_utils functionality."""
 
+import functools
 import threading
 import time
 from typing import List, Optional, Protocol, Set
@@ -24,10 +25,7 @@ from jax.experimental import multihost_utils
 import numpy as np
 
 # Default timeout in seconds.
-_DEFAULT_BARRIER_TIMEOUT = 1200
-
-DIRECTORY_CREATION_TIMEOUT = 360
-DIRECTORY_DELETION_TIMEOUT = 360
+_DEFAULT_BARRIER_TIMEOUT = 300
 
 # Used in unit tests with multiple parallel test cases.
 _TEST_CASE_INDEX = None
@@ -45,8 +43,33 @@ EXPERIMENTAL_ORBAX_USE_DISTRIBUTED_PROCESS_ID = flags.DEFINE_bool(
     ' jax.process_index().',
 )
 
+EXPERIMENTAL_ORBAX_USE_DISTRIBUTED_BARRIER = flags.DEFINE_bool(
+    'experimental_orbax_use_distributed_barrier',
+    False,
+    'If True, uses the JAX distributed client for barriers, which supports'
+    ' timeouts. Otherwise, uses collectives-based barriers.',
+)
 
 
+def coordination_timeout() -> int:
+  """Returns the timeout for coordination operations."""
+  return _DEFAULT_BARRIER_TIMEOUT
+
+
+
+
+def is_pathways_backend() -> bool:
+  # Pathways is single-host.
+  return (
+      hasattr(jax.devices()[0].client, 'pathways')
+      or jax.devices()[0].client.runtime_type == 'pathways'
+      or jax.devices()[0].client.runtime_type == 'proxy/pathways'
+  )
+
+
+def is_pathways_controller() -> bool:
+  return jax.local_devices()[0].client.runtime_type == 'pathways'
+
 
 def is_runtime_to_distributed_ids_initialized() -> bool:
   return _RUNTIME_TO_DISTRIBUTED_ID is not None
@@ -313,8 +336,20 @@ def sync_global_processes(
     return
   timeout = timeout or _DEFAULT_BARRIER_TIMEOUT
   sync_start_time = time.time()
+  try:
+    use_distributed_barrier = EXPERIMENTAL_ORBAX_USE_DISTRIBUTED_BARRIER.value
+  except Exception:  # pylint: disable=broad-exception-caught
+    logging.log_first_n(
+        logging.INFO,
+        '[thread=%s] Failed to get flag value for'
+        ' EXPERIMENTAL_ORBAX_USE_DISTRIBUTED_BARRIER.',
+        1,
+        threading.current_thread().name,
+    )
+    use_distributed_barrier = False
+
   # Temporarily default to existing behavior to minimize risk of breakage.
-  if processes is None:
+  if processes is None and not use_distributed_barrier:
     key = _unique_barrier_key(name)
     logging.vlog(
         1,
@@ -380,12 +415,10 @@ def process_count() -> int:
   return jax.process_count()
 
 
-def process_index() -> int:
-  """Customized logic for obtaining JAX process index."""
+def use_experimental_distributed_process_id() -> bool:
+  """Returns True if the experimental distributed process id is enabled."""
   try:
-    experimental_orbax_use_distributed_process_id = (
-        EXPERIMENTAL_ORBAX_USE_DISTRIBUTED_PROCESS_ID.value
-    )
+    return EXPERIMENTAL_ORBAX_USE_DISTRIBUTED_PROCESS_ID.value
   except Exception:  # pylint: disable=broad-exception-caught
     logging.log_first_n(
         logging.INFO,
@@ -394,8 +427,12 @@ def process_index() -> int:
         1,
         threading.current_thread().name,
     )
-    experimental_orbax_use_distributed_process_id = False
-  if experimental_orbax_use_distributed_process_id:
+    return False
+
+
+def process_index() -> int:
+  """Customized logic for obtaining JAX process index."""
+  if use_experimental_distributed_process_id():
     logging.log_first_n(
         logging.INFO,
         '[thread=%s] Using distributed process id.',
@@ -407,8 +444,43 @@ def process_index() -> int:
     return jax.process_index()
 
 
+def process_index_from_device(device: jax.Device) -> int:
+  """Customized logic for obtaining JAX process index from a device."""
+  if use_experimental_distributed_process_id():
+    return runtime_to_distributed_process_id(device.process_index)
+  else:
+    return device.process_index
+
+
 def unique_processes_from_devices(device_array: np.ndarray) -> Set[int]:
-  get_pids_from_devices = np.vectorize(
-      lambda d: runtime_to_distributed_process_id(d.process_index)
-  )
+  get_pids_from_devices = np.vectorize(process_index_from_device)
   return set(get_pids_from_devices(device_array).flat)
+
+
+def global_max(values: list[int]) -> list[int]:
+  """Computes the global max of a list of integers across all hosts."""
+  host_mesh = jax.sharding.Mesh(
+      np.asarray(jax.devices()).reshape(
+          process_count(), jax.local_device_count()
+      ),
+      ['host', 'dev'],
+  )
+  sharding = jax.sharding.NamedSharding(
+      host_mesh, jax.sharding.PartitionSpec('host', None)
+  )
+  local_array = np.array([values], dtype=np.int32)
+  # Create the global array, which is sharded across hosts.
+  global_array = jax.make_array_from_process_local_data(sharding, local_array)
+
+  @jax.jit
+  @functools.partial(
+      jax.shard_map,
+      mesh=host_mesh,
+      in_specs=jax.sharding.PartitionSpec('host', None),
+      out_specs=jax.sharding.PartitionSpec(),
+  )
+  def reduce_max_fn(x):
+    return jax.lax.pmax(x, axis_name='host')
+
+  max_values_array = reduce_max_fn(global_array).squeeze(axis=0)
+  return list(np.asarray(max_values_array).astype(int))
