@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
 """Multislice utilities."""
 
 import functools
-from typing import Any, Optional, Set, Tuple, Union
+from typing import Any, Optional, Set, Union
 
 from absl import logging
 import jax
@@ -30,69 +30,68 @@ PyTree = Any
 MEMORY_FACTOR = 3
 
 
-def process_slice_id(
+def process_replica_id(
     process_index: int,
     global_mesh: jax.sharding.Mesh,
     *,
     replica_axis_index: int = 0,
 ) -> int:
   """Returns the slice id that the process_index belongs to."""
-  for slice_id in range(
-      slice_count(global_mesh, replica_axis_index=replica_axis_index)
+  for replica_id in range(
+      replica_count(global_mesh, replica_axis_index=replica_axis_index)
   ):
-    device_slice = slice_devices(
-        global_mesh, replica_id=slice_id, replica_axis_index=replica_axis_index
+    device_slice = replica_devices(
+        global_mesh,
+        replica_id=replica_id,
+        replica_axis_index=replica_axis_index,
     )
     if process_index in multihost.unique_processes_from_devices(device_slice):
-      return slice_id
+      return replica_id
   return -1
 
 
-def _process_in_device_slice(
+def _process_in_device_replica(
     process_index: int, device_slice: np.ndarray
 ) -> bool:
   return process_index in multihost.unique_processes_from_devices(device_slice)
 
 
-def slice_devices(
+def replica_devices(
     global_mesh: jax.sharding.Mesh,
     *,
     replica_id: int = 0,
     replica_axis_index: int = 0,
 ) -> np.ndarray:
-  devices = global_mesh.devices
-  if hasattr(jax.devices()[0], 'slice_index'):
-    get_slice_id = np.vectorize(lambda x: x.slice_index)
-    return devices[get_slice_id(devices) == replica_id]
-  else:
-    return np.take(
-        global_mesh.devices,
-        replica_id,
-        axis=replica_axis_index,
-    )
+  return np.take(
+      global_mesh.devices,
+      replica_id,
+      axis=replica_axis_index,
+  )
 
 
-def slice_count(
+def replica_count(
     global_mesh: jax.sharding.Mesh, *, replica_axis_index: int = 0
 ) -> int:
   """Number of slices implied by the mesh's replica dimension."""
+  if len(global_mesh.shape_tuple) == 1:
+    return 1
   return global_mesh.devices.shape[replica_axis_index]
 
 
-def local_slice_devices(
+def local_replica_devices(
     global_mesh: jax.sharding.Mesh, *, replica_axis_index: int = 0
 ) -> np.ndarray:
   """Get devices in the host-local slice."""
   for replica_id in range(
-      slice_count(global_mesh, replica_axis_index=replica_axis_index)
+      replica_count(global_mesh, replica_axis_index=replica_axis_index)
   ):
-    if in_slice(
+    if in_replica(
         multihost.process_index(),
         global_mesh,
         replica_id=replica_id,
         replica_axis_index=replica_axis_index,
     ):
-      return slice_devices(
+      return replica_devices(
           global_mesh,
           replica_id=replica_id,
           replica_axis_index=replica_axis_index,
@@ -103,23 +102,23 @@ def local_slice_devices(
   )
 
 
-def primary_process_in_slice(
+def primary_process_in_replica(
     global_mesh: jax.sharding.Mesh,
     *,
     replica_id: int = 0,
     replica_axis_index: int = 0,
 ) -> int:
   """Returns an arbitrary process in the requested slice to serve as primary."""
-  device_slice = slice_devices(
+  device_replica = replica_devices(
       global_mesh,
       replica_axis_index=replica_axis_index,
       replica_id=replica_id,
   )
-  processes = multihost.unique_processes_from_devices(device_slice)
+  processes = multihost.unique_processes_from_devices(device_replica)
   return next(iter(processes))
 
 
-def in_slice(
+def in_replica(
     process_index: int,
     global_mesh: jax.sharding.Mesh,
     *,
@@ -127,9 +126,9 @@ def in_slice(
     replica_axis_index: int = 0,
 ) -> bool:
   """Returns if the process belongs to the indicated slice ID."""
-  return _process_in_device_slice(
+  return _process_in_device_replica(
       process_index,
-      slice_devices(
+      replica_devices(
           global_mesh,
           replica_id=replica_id,
           replica_axis_index=replica_axis_index,
@@ -137,23 +136,22 @@ def in_slice(
   )
 
 
-@functools.partial(jax.jit, static_argnums=0)
-def fake_zero_data(sharding, x):
-  x = jnp.zeros_like(x)
-  return jax.lax.with_sharding_constraint(x, sharding)
-
-
 def get_device_memory() -> int:
   """Returns HBM capacity of the device on which the code is running(in bytes)."""
   device = jax.devices()[0]
-  if device.platform != 'tpu':
-    raise ValueError('Only TPU devices are supported.')
+  if device.platform not in ('tpu', 'gpu'):
+    raise ValueError('Only select TPU and GPU devices are supported.')
   hbm_memory = {
-      'TPU v3': int(16e9),  # two cores pre chip each with 16 GB HBM
+      'TPU v3': int(16e9),  # two cores per chip each with 16 GB HBM
       'TPU v4': int(32e9),  # one megacore per chip with 32 GB HBM
       'TPU v5 lite': int(16e9),  # one core per chip with 16 GB HBM
       'TPU v5': int(96e9),  # one megacore per chip with 96 GB HBM
       'TPU v6 lite': int(32e9),  # one core per chip with 32 GB HBM
+      'TPU 7x': int(96e9),  # two cores per chip each with 96 GB HBM
+      'NVIDIA H100 80GB HBM3': int(80e9),
+      'NVIDIA H200': int(144e9),
+      'NVIDIA B200': int(183e9),
+      'NVIDIA B300 SXM6 AC': int(275e9),
   }
   memory = hbm_memory.get(device.device_kind, None)
   if memory is None:
@@ -169,7 +167,7 @@ def get_leaf_memory_per_device(arr: jax.Array) -> int:
   return shard.data.size * shard.data.itemsize
 
 
-def tree_memory_per_device(tree: Tuple[PyTree, ...]) -> int:
+def tree_memory_per_device(tree: tuple[jax.Array, ...] | jax.Array) -> int:
   """Returns the memory usage of a PyTree on each device (in bytes)."""
   leaf_memory_per_device = jax.tree_util.tree_map(
       get_leaf_memory_per_device, tree
@@ -178,7 +176,7 @@ def tree_memory_per_device(tree: Tuple[PyTree, ...]) -> int:
 
 
 def get_available_memory(
-    in_tree: Tuple[PyTree, ...], scaling_factor: float
+    in_tree: tuple[jax.Array, ...], scaling_factor: float
 ) -> int:
   """Returns estimated available memory for broadcasting (in bytes).
 
@@ -199,14 +197,119 @@ def get_available_memory(
   return int(available_memory * scaling_factor / MEMORY_FACTOR)
 
 
+def slice_count() -> int:
+  """Returns the number of slices."""
+  return (
+      len(
+          set(d.slice_index for d in jax.devices() if hasattr(d, 'slice_index'))
+      )
+      or 1
+  )
+
+
+def _get_slice_shape(
+    index: tuple[slice, ...], global_shape: tuple[int, ...]
+) -> tuple[int, ...]:
+  """Calculates the shape of a slice from a global shape, assuming step is always 1."""
+  return tuple(
+      s.indices(global_shape[i])[1] - s.indices(global_shape[i])[0]
+      for i, s in enumerate(index)
+  )
+
+
+def _globalize_single_replica_arrays(
+    inp: jax.Array,
+    replica_axis_index: int,
+    global_mesh: jax.sharding.Mesh,
+    is_source: bool,
+) -> jax.Array:
+  """Globalizes a single replica array."""
+
+  num_replicas = global_mesh.devices.shape[replica_axis_index]
+  replica_axis_name = global_mesh.axis_names[replica_axis_index]
+  sharding = inp.sharding
+  if not isinstance(sharding, jax.sharding.NamedSharding):
+    raise ValueError(
+        'Must provide input arrays with NamedSharding. '
+        f'Got {type(sharding)} instead.'
+    )
+  local_replica_shape = inp.shape
+
+  assert replica_axis_name not in sharding.spec, (
+      f'Replica axis name {replica_axis_name} already exists in'
+      f' sharding.spec {sharding.spec}'
+  )
+  global_shape = (num_replicas,) + local_replica_shape
+  logging.vlog(
+      1,
+      'Globalizing array with local shape %s to Global shape: %s',
+      local_replica_shape,
+      global_shape,
+  )
+  global_spec = jax.sharding.PartitionSpec(
+      replica_axis_name,
+      *sharding.spec,
+  )
+  global_sharding = jax.sharding.NamedSharding(global_mesh, global_spec)
+
+  source_device_map = {}
+
+  if is_source:
+    for s in inp.addressable_shards:
+      source_device_map[s.device] = jnp.expand_dims(s.data, axis=0)
+
+  device_buffers = []
+  for d, index in global_sharding.addressable_devices_indices_map(
+      global_shape
+  ).items():
+    if d in source_device_map:
+      device_buffers.append(source_device_map[d])
+    else:
+      # Use jax.numpy.zeros to allocate directly on device
+      # to avoid Host RAM spike.
+      slice_shape = _get_slice_shape(index, global_shape)
+      zero_data = jnp.zeros(slice_shape, dtype=inp.dtype, device=d)
+      device_buffers.append(zero_data)
+
+  logging.vlog(
+      1,
+      'Device buffers: %r',
+      {d.device: d for d in device_buffers},
+  )
+  return jax.make_array_from_single_device_arrays(
+      global_shape,
+      global_sharding,
+      device_buffers,
+      dtype=inp.dtype,
+  )
+
+
+def _merge_globalized_replicas(
+    globalized_tree: tuple[jax.Array, ...],
+    global_mesh: jax.sharding.Mesh,
+):
+  """Merges globalized sharded replicas into a single replica."""
+  out_sharding = jax.tree.map(
+      lambda x: jax.sharding.NamedSharding(
+          global_mesh, jax.sharding.PartitionSpec(*x.sharding.spec[1:])
+      ),
+      globalized_tree,
+  )
+  out_subtree = jax.jit(
+      lambda tree: jax.tree.map(functools.partial(jnp.sum, axis=0), tree),
+      out_shardings=out_sharding,
+  )(globalized_tree)
+  return out_subtree
+
+
 def broadcast_one_replica_to_all(
-    in_tree: Tuple[PyTree, ...],
+    in_tree: tuple[jax.Array, ...],
     global_mesh: jax.sharding.Mesh,
     replica_axis_index: int,
     is_source: bool,
     memory_limit_bytes: Optional[Union[int, None]] = None,
     memory_scaling_factor: Optional[float] = 0.75,
-) -> Tuple[Tuple[PyTree, ...], int]:
+) -> tuple[tuple[jax.Array, ...], int]:
   """One replica reads the data and broadcasts to others.
 
   Args:
@@ -224,34 +327,10 @@ def broadcast_one_replica_to_all(
       - pytree with broadcasted data
       - number of broadcasts performed.
   """
-  num_replicas = global_mesh.devices.shape[replica_axis_index]
-  replica_axis_name = global_mesh.axis_names[replica_axis_index]
-
   if memory_limit_bytes is None:
     memory_limit_bytes = get_available_memory(in_tree, memory_scaling_factor)
     logging.info('Using available memory of %d bytes.', memory_limit_bytes)
 
-  # Set replica_axis to be 0, regardless of its actual value.
-  def globalize_single_replica_arrays(inp):
-    sharding = inp.sharding
-    if not isinstance(sharding, jax.sharding.NamedSharding):
-      raise ValueError(
-          'Must provide input arrays with NamedSharding. '
-          f'Got {type(sharding)} instead.'
-      )
-    if not is_source:
-      inp = fake_zero_data(sharding, inp)
-    inp = jnp.expand_dims(inp, axis=0)
-    in_spec = jax.sharding.PartitionSpec(
-        replica_axis_name,
-        *sharding.spec,
-    )
-    global_shape = (num_replicas,) + inp.shape[1:]
-    global_sharding = jax.sharding.NamedSharding(global_mesh, in_spec)
-    return jax.make_array_from_single_device_arrays(
-        global_shape, global_sharding, [s.data for s in inp.addressable_shards]
-    )
-
   tree_len = len(in_tree)
   start = 0
   out_tree = []
@@ -279,20 +358,20 @@ def broadcast_one_replica_to_all(
         end += 1
     subtree = tuple(subtree)
     num_broadcasts += 1
-    out_sharding = jax.tree.map(
-        lambda x: jax.sharding.NamedSharding(
-            global_mesh, jax.sharding.PartitionSpec(*x.sharding.spec)
+    globalized_sharded_subtree = jax.tree.map(
+        functools.partial(
+            _globalize_single_replica_arrays,
+            global_mesh=global_mesh,
+            replica_axis_index=replica_axis_index,
+            is_source=is_source,
         ),
         subtree,
     )
-    in_tree_sharded = jax.tree.map(globalize_single_replica_arrays, subtree)
     # Delete immediately to conserve memory.
     jax.tree.map(lambda x: x.delete(), subtree)
-
-    out_subtree = jax.jit(
-        lambda tree: jax.tree.map(functools.partial(jnp.sum, axis=0), tree),
-        out_shardings=out_sharding,
-    )(in_tree_sharded)
+    out_subtree = _merge_globalized_replicas(
+        globalized_sharded_subtree, global_mesh
+    )
     out_tree.extend(out_subtree)
     jax.block_until_ready(out_subtree)
     start = end
@@ -306,13 +385,59 @@ def get_primary_replica_ids_and_pids(
     replica_axis_idx: int,
     mesh: jax.sharding.Mesh,
     primary_replica_id: int,
-) -> Tuple[Set[int], Set[int]]:
+) -> tuple[Set[int], Set[int]]:
   """Returns the primary replica ids and process ids."""
-  replica_devices = slice_devices(
+  devices = replica_devices(
       mesh,
       replica_id=primary_replica_id,
       replica_axis_index=replica_axis_idx,
   ).flatten()
-  ids = set([d.id for d in replica_devices])
-  pids = set([d.process_index for d in replica_devices])
+  ids = set([d.id for d in devices])
+  pids = multihost.unique_processes_from_devices(devices)
   return ids, pids
+
+
+def process_spans_multiple_replicas(
+    global_mesh: jax.sharding.Mesh,
+    *,
+    replica_axis_index: int = 0,
+) -> bool:
+  """Checks if any JAX process controls devices across different replicas.
+
+  Replicas are defined by slicing the `global_mesh` along the
+  `replica_axis_index`. This function iterates through all unique JAX processes
+  and, for each process, checks if the devices it manages belong to more than
+  one replica group.
+
+  Args:
+    global_mesh: The global JAX mesh.
+    replica_axis_index: The index of the axis in the mesh shape that
+      differentiates the replicas.
+
+  Returns:
+    True if at least one process has devices in multiple replicas,
+    False otherwise.
+  """
+  num_replicas = replica_count(
+      global_mesh, replica_axis_index=replica_axis_index
+  )
+  all_processes = multihost.unique_processes_from_devices(
+      global_mesh.devices.flatten()
+  )
+
+  for process_idx in all_processes:
+    found_replica_ids = []
+    for replica_id in range(num_replicas):
+      devices_in_replica = replica_devices(
+          global_mesh,
+          replica_id=replica_id,
+          replica_axis_index=replica_axis_index,
+      )
+      if process_idx in multihost.unique_processes_from_devices(
+          devices_in_replica
+      ):
+        found_replica_ids.append(replica_id)
+
+    if len(found_replica_ids) > 1:
+      return True
+  return False
