@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,72 +20,106 @@ paths, which allow other implementations.
 
 from __future__ import annotations
 
-from typing import Optional, Protocol
-
+import abc
 from etils import epath
 from orbax.checkpoint import options as options_lib
 from orbax.checkpoint._src.metadata import checkpoint as checkpoint_metadata
 
 
 
-# TODO(b/326119183) Support configuration of temporary path detection
-# (currently handled by `tmp_checkpoints` util methods).
-class TemporaryPath(Protocol):
+TMP_DIR_SUFFIX = '.orbax-checkpoint-tmp'
+COMMIT_SUCCESS_FILE = 'commit_success.txt'
+
+
+class ValidationError(ValueError):
+  """Raised when a TemporaryPath or its final path is invalid."""
+
+
+class TemporaryPath(abc.ABC):
   """Class that represents a temporary path.
 
   Importantly, the temporary path always has a corresponding finalized path, and
   is primarily constructed from this path. The class contains logic to create
   the temporary path, and to finalize it into the final path.
 
-  NOTE: All methods are intended to be called across all active processes,
-  except for `finalize`, which is only called on the primary host.
+  `from_final` should be called from all hosts to construct the temporary path
+  instance from the given final path.
+  `create` and `finalize` must be called only on the primary host.
   """
 
   @classmethod
+  @abc.abstractmethod
+  async def validate(
+      cls,
+      temporary_path: epath.Path,
+  ):
+    """Validates the temporary path or raises a ValidationError."""
+    ...
+
+  @classmethod
+  @abc.abstractmethod
+  async def validate_final(
+      cls,
+      final_path: epath.Path,
+  ):
+    """Validates the final path or raises a ValidationError."""
+    ...
+
+  @classmethod
+  @abc.abstractmethod
+  def from_temporary(
+      cls,
+      temporary_path: epath.Path,
+      *,
+      file_options: options_lib.FileOptions | None = None,
+      use_snapshot: bool | None = None,
+  ) -> TemporaryPath:
+    """Creates a TemporaryPath from a temporary path."""
+    ...
+
+  @classmethod
+  @abc.abstractmethod
   def from_final(
       cls,
       final_path: epath.Path,
       *,
-      checkpoint_metadata_store: Optional[
-          checkpoint_metadata.MetadataStore
-      ] = None,
-      file_options: Optional[options_lib.FileOptions] = None,
-      multiprocessing_options: Optional[
-          options_lib.MultiprocessingOptions
-      ] = None,
+      checkpoint_metadata_store: (
+          checkpoint_metadata.MetadataStore | None
+      ) = None,
+      file_options: options_lib.FileOptions | None = None,
+      use_snapshot: bool | None = None,
   ) -> TemporaryPath:
     """Creates a TemporaryPath from a final path."""
     ...
 
-  @classmethod
-  def match(cls, temporary_path: epath.Path, final_path: epath.Path) -> bool:
-    """Determines if `temporary_path` could correspond to `final_path`."""
-    ...
-
+  @abc.abstractmethod
   def get(self) -> epath.Path:
     """Constructs the temporary path without actually creating it."""
     ...
 
+  @abc.abstractmethod
   def get_final(self) -> epath.Path:
     """Returns the final path without creating it."""
     ...
 
-  async def create(
-      self,
-      *,
-      file_options: options_lib.FileOptions = options_lib.FileOptions(),
-  ) -> epath.Path:
-    """Creates the temporary path on disk."""
+  @abc.abstractmethod
+  async def create(self) -> epath.Path:
+    """Creates the temporary path on disk.
+
+    NOTE: This method should be only called on the primary host.
+
+    Returns:
+      The created temporary path.
+    """
     ...
 
-  def finalize(
+  @abc.abstractmethod
+  async def finalize(
       self,
-  ):
+  ) -> None:
     """Finalizes the temporary path into the final path.
 
-    NOTE: This method is only called on the primary host. This is in contrast
-    with all other methods in this class, which are called across all active
-    processes.
+    NOTE: This method should be only called on the primary host.
 
     This function is called from a background thread.
 
