@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@ from orbax.checkpoint import test_utils
 from orbax.checkpoint._src.metadata import checkpoint
 from orbax.checkpoint._src.metadata import step_metadata_serialization
 from orbax.checkpoint._src.path import atomicity
+from orbax.checkpoint._src.path import gcs_utils
 from orbax.checkpoint._src.path import step as step_lib
 
 
@@ -209,7 +210,7 @@ class StandardNameFormatTest(parameterized.TestCase):
       )
       with self.subTest('uncommitted_gcs_step'):
         with mock.patch.object(
-            step_lib, 'is_gcs_path', autospec=True, return_value=True
+            gcs_utils, 'is_gcs_path', autospec=True, return_value=True
         ):
           with self.assertRaises(ValueError):
             _ = name_format.find_step(self.directory, step)
@@ -227,7 +228,7 @@ class StandardNameFormatTest(parameterized.TestCase):
           'find_uncommited_gcs_step_path_with_include_uncommitted=False'
       ):
         with mock.patch.object(
-            step_lib, 'is_gcs_path', autospec=True, return_value=True
+            gcs_utils, 'is_gcs_path', autospec=True, return_value=True
         ):
           with self.assertRaises(ValueError):
             _ = step_lib.find_step_path(
@@ -254,7 +255,7 @@ class StandardNameFormatTest(parameterized.TestCase):
         step_dir = self.directory / step_name
         step_dir.mkdir(parents=True, exist_ok=True)
         if step_name is not uncommitted_step_name:
-          commit_success_file = step_dir / step_lib._COMMIT_SUCCESS_FILE
+          commit_success_file = step_dir / atomicity.COMMIT_SUCCESS_FILE
           commit_success_file.write_text(
               f'Checkpoint commit was successful to {step_dir}'
           )
@@ -269,7 +270,7 @@ class StandardNameFormatTest(parameterized.TestCase):
 
     if gcs:
       with mock.patch.object(
-          step_lib, 'is_gcs_path', autospec=True, return_value=True
+          gcs_utils, 'is_gcs_path', autospec=True, return_value=True
       ):
         self.assertEqual(
             self.directory / uncommitted_step_name,
@@ -359,7 +360,7 @@ class StandardNameFormatTest(parameterized.TestCase):
     )
     with self.subTest('find_all_uncommitted_gcs_steps'):
       with mock.patch.object(
-          step_lib, 'is_gcs_path', autospec=True, return_value=True
+          gcs_utils, 'is_gcs_path', autospec=True, return_value=True
       ):
         self.assertEmpty([
             metadata.path.name
@@ -443,24 +444,24 @@ class UtilsTest(parameterized.TestCase):
     )
 
   @parameterized.parameters((None,), ('checkpoint_',), ('foobar_',))
-  def test_is_tmp_checkpoint(self, step_prefix):
+  def test_is_path_temporary(self, step_prefix):
     step_dir = step_lib.get_save_directory(
         5, self.directory, step_prefix=step_prefix
     )
     step_dir.mkdir(parents=True)
-    self.assertFalse(step_lib.is_tmp_checkpoint(step_dir))
+    self.assertFalse(step_lib.is_path_temporary(step_dir))
     tmp_step_dir = atomicity._get_tmp_directory(step_dir)
     test_utils.create_tmp_directory(tmp_step_dir, step_dir)
-    self.assertTrue(step_lib.is_tmp_checkpoint(tmp_step_dir))
+    self.assertTrue(step_lib.is_path_temporary(tmp_step_dir))
 
     item_dir = step_lib.get_save_directory(
         10, self.directory, name='params', step_prefix=step_prefix
     )
     item_dir.mkdir(parents=True)
-    self.assertFalse(step_lib.is_tmp_checkpoint(item_dir))
+    self.assertFalse(step_lib.is_path_temporary(item_dir))
     tmp_item_dir = atomicity._get_tmp_directory(item_dir)
     test_utils.create_tmp_directory(tmp_item_dir, item_dir)
-    self.assertTrue(step_lib.is_tmp_checkpoint(tmp_item_dir))
+    self.assertTrue(step_lib.is_path_temporary(tmp_item_dir))
 
   @parameterized.parameters(
       ('0', 0),
@@ -470,11 +471,13 @@ class UtilsTest(parameterized.TestCase):
       ('checkpoint_0000', 0),
       ('checkpoint_003400', 3400),
       ('foobar_1000', 1000),
+      ('foo_bar_1000', 1000),
       ('0.orbax-checkpoint-tmp-1010101', 0),
       ('0000.orbax-checkpoint-tmp-12323232', 0),
       ('foobar_1.orbax-checkpoint-tmp-12424424', 1),
       ('foobar_000505.orbax-checkpoint-tmp-13124', 505),
       ('checkpoint_16.orbax-checkpoint-tmp-123214324', 16),
+      ('foo_bar_1000.orbax-checkpoint-tmp-123123', 1000),
   )
   def test_step_from_checkpoint_name(self, name, step):
     self.assertEqual(step_lib.step_from_checkpoint_name(name), step)
@@ -485,16 +488,50 @@ class UtilsTest(parameterized.TestCase):
       ('checkpoint_1010_',),
       ('_191',),
       ('.orbax-checkpoint-tmp-191913',),
-      ('0.orbax-checkpoint-tmp-',),
+      ('0.orbax-checkpoint-',),
       ('checkpoint_.orbax-checkpoint-tmp-191913',),
   )
   def test_step_from_checkpoint_name_invalid(self, name):
     with self.assertRaises(ValueError):
       step_lib.step_from_checkpoint_name(name)
 
-  def test_checkpoint_steps_paths_nonexistent_directory_fails(self):
-    with self.assertRaisesRegex(ValueError, 'does not exist'):
-      step_lib.checkpoint_steps_paths('/non/existent/dir')
+  def test_is_standard_name_format_returns_true(self):
+    self.assertTrue(
+        step_lib.is_standard_name_format(step_lib.standard_name_format())
+    )
+
+  def test_is_standard_name_format_returns_false(self):
+    self.assertFalse(
+        step_lib.is_standard_name_format(
+            step_lib.composite_name_format(
+                write_name_format=step_lib.standard_name_format(),
+                read_name_formats=[step_lib.standard_name_format()],
+            )
+        )
+    )
+
+  def test_single_host_load_and_broadcast_name_format_standard(self):
+    standard_format = step_lib.standard_name_format()
+    new_format = step_lib.single_host_load_and_broadcast_name_format(
+        standard_format
+    )
+
+    self.assertFalse(standard_format.single_host_load_and_broadcast)  # pytype: disable=attribute-error
+    self.assertTrue(new_format.single_host_load_and_broadcast)  # pytype: disable=attribute-error
+
+  def test_single_host_load_and_broadcast_name_format_composite_raises_error(
+      self,
+  ):
+    composite_format = step_lib.composite_name_format(
+        write_name_format=step_lib.standard_name_format(),
+        read_name_formats=[step_lib.standard_name_format()],
+    )
+
+    with self.assertRaises(ValueError):
+      step_lib.single_host_load_and_broadcast_name_format(composite_format)
+
+  def test_checkpoint_steps_paths_nonexistent_directory_returns_empty(self):
+    self.assertEqual(step_lib.checkpoint_steps_paths('/non/existent/dir'), [])
 
   def test_checkpoint_steps_paths_returns_finalized_paths(self):
     digit_only_path = epath.Path(self.directory / '2')
@@ -580,6 +617,38 @@ class UtilsTest(parameterized.TestCase):
             self.directory / '1', name_format
         )
 
+  def test_string_representation(self):
+    self.assertEqual(
+        str(step_lib.standard_name_format(step_prefix='step')),
+        'StandardNameFormat("step_1234")',
+    )
+    self.assertEqual(
+        str(
+            step_lib.standard_name_format(
+                step_prefix='step', step_format_fixed_length=8
+            )
+        ),
+        'StandardNameFormat("step_00001234")',
+    )
+    self.assertEqual(
+        str(
+            step_lib.composite_name_format(
+                write_name_format=step_lib.standard_name_format(
+                    step_prefix='write', step_format_fixed_length=8
+                ),
+                read_name_formats=[
+                    step_lib.standard_name_format(
+                        step_prefix='write', step_format_fixed_length=8
+                    ),
+                    step_lib.standard_name_format(step_prefix='read'),
+                ],
+            )
+        ),
+        'Composite(['
+        'StandardNameFormat("write_00001234"),StandardNameFormat("read_1234")'
+        '])',
+    )
+
 
 
 if __name__ == '__main__':
