@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,7 +31,9 @@ from orbax.checkpoint._src.multihost import multihost
 
 Shape = types.Shape
 Index = types.Index
-OptionalAxisAndShape = tuple[int | None, Shape| None]
+OptionalAxisAndShapeAndReplicaCount = tuple[
+    int | None, Shape | None, int | None
+]
 
 HashableIndex = types.HashableIndex
 HashableSlice = types.HashableSlice
@@ -44,6 +46,7 @@ class SliceArgs:
   Intended to be passed to `ReplicaSlice` in order to select a slice of
   the `unsliced_data` array.
   """
+
   start_index: int
   limit_index: int
   axis: int
@@ -79,15 +82,28 @@ class ReplicaSlice:
     return isinstance(self.unsliced_data, np.ndarray)
 
   def data(self):
+    """Returns the sliced data for this replica.
+
+    If `slice_args` is None, the entire `unsliced_data` is returned. Otherwise,
+    a slice of `unsliced_data` is returned based on `slice_args`.
+    """
     if self.slice_args is None:
       return self.unsliced_data
     else:
-      return jax.lax.slice_in_dim(
-          self.unsliced_data,
-          start_index=self.slice_args.start_index,
-          limit_index=self.slice_args.limit_index,
-          axis=self.slice_args.axis,
+      # If a global mesh is set, slice_in_dim can fail with incompatible device
+      # errors. To avoid this, we temporarily set a mesh constructed from
+      # array's devices.
+      mesh = jax.sharding.Mesh(
+          np.array(list(self.unsliced_data.sharding.device_set)), ('data',)
       )
+      with jax.sharding.set_mesh(mesh):
+        sliced_data = jax.lax.slice_in_dim(
+            self.unsliced_data,
+            start_index=self.slice_args.start_index,
+            limit_index=self.slice_args.limit_index,
+            axis=self.slice_args.axis,
+        )
+      return sliced_data
 
 
 @dataclasses.dataclass(frozen=True)
@@ -113,17 +129,31 @@ class ReplicaSlices:
 
   @property
   def nbytes(self) -> int:
-    slice_nbytes = math.prod(self.local_shape) * self.dtype.itemsize
-    return slice_nbytes * len(self.replica_slices)
+    """Returns the total number of bytes for all replica slices."""
+    total_bytes = 0
+    for rslice in self.replica_slices:
+      if rslice.slice_args is None:
+        # No slicing, use the full local_shape
+        slice_nbytes = math.prod(self.local_shape) * self.dtype.itemsize
+      else:
+        # Replica-parallel, calculate bytes for the specific slice
+        slice_shape = list(self.local_shape)
+        axis = rslice.slice_args.axis
+        slice_shape[axis] = (
+            rslice.slice_args.limit_index - rslice.slice_args.start_index
+        )
+        slice_nbytes = math.prod(slice_shape) * self.dtype.itemsize
+      total_bytes += slice_nbytes
+    return total_bytes
 
-  def to_fragments(self) -> fragments.Fragments:
+  def to_fragments(self) -> fragments.ConcreteFragments:
     """Converts replica slices to fragments."""
     assert self.is_on_host
-    result = fragments.Fragments(
+    result = fragments.ConcreteFragments(
         shape=self.global_shape,
         dtype=self.dtype,
         fragments=[
-            fragments.Fragment(
+            fragments.ConcreteFragment(
                 index=numpy_utils.resolve_slice(
                     rslice.index, self.global_shape
                 ),
@@ -178,13 +208,15 @@ def _sharding_num_replicas(
 
 
 def calculate_replica_parallel_axis_and_local_shape(
-    arr: jax.Array,
-) -> OptionalAxisAndShape:
+    arr: jax.Array, max_replicas_for_replica_parallel: Optional[int]
+) -> OptionalAxisAndShapeAndReplicaCount:
   """Calculates a local shape for replica-parallel serialization."""
   shard0 = arr.addressable_shards[0]
   replica_count = _sharding_num_replicas(arr.sharding, arr.shape)
+  if max_replicas_for_replica_parallel is not None:
+    replica_count = min(replica_count, max_replicas_for_replica_parallel)
   if shard0.data.size == 0 or replica_count <= 1:
-    return None, None
+    return None, None, None
   try:
     axis = next(
         axis_index
@@ -192,18 +224,20 @@ def calculate_replica_parallel_axis_and_local_shape(
         if axis_size % replica_count == 0
     )
   except StopIteration:
-    return None, None
+    return None, None, None
   local_shape = tuple(
       axis_size // (replica_count if axis_index == axis else 1)
       for axis_index, axis_size in enumerate(shard0.data.shape)
   )
-  return axis, local_shape
+  return axis, local_shape, replica_count
 
 
 def get_replica_slices(
     arr: jax.Array,
     replica_id: Optional[int],
     use_replica_parallel: bool,
+    min_slice_bytes_for_replica_parallel: Optional[int] = None,
+    max_replicas_for_replica_parallel: Optional[int] = None,
 ) -> ReplicaSlices:
   """Returns the replica slices a given replica is responsible for.
 
@@ -218,6 +252,11 @@ def get_replica_slices(
     use_replica_parallel: Whether to use replica-parallel serialization to allow
       arrays with replicated shards to be written cooperatively by different
       hosts.
+    min_slice_bytes_for_replica_parallel: Minimum number of bytes per replica
+      slice. Only uses replica-parallel when the amount of data written per
+      replica is greater than or equal to this number.
+    max_replicas_for_replica_parallel: Maximum number of replicas over which
+      saving will be parallelized if use_replica_parallel is True.
 
   Returns:
     ReplicaSlices object.
@@ -254,8 +293,16 @@ def get_replica_slices(
     # Check whether replica-parallel applies: we are dealing with non-empty
     # shards, we have more than one replica, and some dimension of the shards
     # is evenly divisible across replicas.
-    axis, local_shape = calculate_replica_parallel_axis_and_local_shape(arr)
-    if axis is None or local_shape is None:
+    axis, local_shape, replica_count = (
+        calculate_replica_parallel_axis_and_local_shape(
+            arr, max_replicas_for_replica_parallel
+        )
+    )
+    if axis is None or local_shape is None or replica_count is None:
+      return None
+
+    min_slice_bytes = min_slice_bytes_for_replica_parallel or 0
+    if math.prod(local_shape) * arr.itemsize < min_slice_bytes:
       return None
 
     rslices: list[ReplicaSlice] = []
@@ -263,6 +310,10 @@ def get_replica_slices(
       # Sanity check that all shards have the same shape.
       assert shard.data.shape == shard0.data.shape
 
+      # Parallelize saving across only `replica_count` replicas.
+      if shard.replica_id >= replica_count:
+        continue
+
       size = local_shape[axis]
       slize = shard.index[axis]
       start = slize.start or 0
@@ -295,14 +346,38 @@ def get_replica_slices(
         ]),
     )
 
+  # Small pinned host arrays have layout requirements so slicing them might hang
+  # this is a temporary workaround to avoid this. TODO: b/417243451
+  is_safe_to_slice = (
+      arr.sharding.memory_kind != 'pinned_host'
+      or arr.ndim >= 2
+      and arr.addressable_shards[0].data.size % 1024 == 0
+      and arr.addressable_shards[0].data.shape[-1] % 128 == 0
+  )
   # In order for all processes to agree on the right serialization metadata
   # we want to compute the correct local shape regardless of whether there
   # are any replica slices to save locally.
-  rslices, local_shape = (  # pytype: disable=attribute-error
-      use_replica_parallel
-      and maybe_pick_replica_parallel()
-      or pick_single_replica()
-  )
+  candidate_slices = None
+  if use_replica_parallel and is_safe_to_slice:
+    candidate_slices = maybe_pick_replica_parallel()
+  if candidate_slices is None:
+    candidate_slices = pick_single_replica()  # pytype: disable=attribute-error
+  rslices, local_shape = candidate_slices
+
+  if multihost.process_index() == 0:
+    # record sharded and replicated arrays metrics
+    replica_count = _sharding_num_replicas(arr.sharding, arr.shape)
+    if replica_count > 1:
+      jax.monitoring.record_scalar(
+          '/jax/orbax/write/replicated_array_gb',
+          value=np.prod(arr.shape) * arr.dtype.itemsize / (1024**3),
+      )
+    else:
+      jax.monitoring.record_scalar(
+          '/jax/orbax/write/sharded_array_gb',
+          value=np.prod(arr.shape) * arr.dtype.itemsize / (1024**3),
+      )
+
   return ReplicaSlices(
       global_shape=arr.shape,
       local_shape=local_shape,
@@ -317,6 +392,8 @@ def transfer_arrays_to_host(
     arrays: Sequence[jax.Array],
     replica_id: Optional[int],
     use_replica_parallel: bool,
+    min_slice_bytes_for_replica_parallel: Optional[int] = None,
+    max_replicas_for_replica_parallel: Optional[int] = None,
     *,
     enable_pinned_host_transfer: bool = False,
 ) -> Sequence[ReplicaSlices]:
@@ -332,6 +409,12 @@ def transfer_arrays_to_host(
     use_replica_parallel: Whether to use replica-parallel serialization to allow
       arrays with replicated shards to be written cooperatively by different
       hosts.
+    min_slice_bytes_for_replica_parallel: Minimum number of bytes per replica
+      slice. Only uses replica-parallel when the amount of data written per
+      replica is greater than or equal to this number.
+    max_replicas_for_replica_parallel: Maximum number of replicas over which to
+      parallelize the writing of replicated shards if use_replica_parallel is
+      True.
     enable_pinned_host_transfer: Whether to allow transfer to pinned host
       memory. Pinned memory is closely associated with a TPU device and can
 
@@ -340,8 +423,11 @@ def transfer_arrays_to_host(
   """
   logging.info(
       'Transferring arrays to host memory with options:'
-      ' use_replica_parallel=%s, enable_pinned_host_transfer=%s',
+      ' use_replica_parallel=%s, min_slice_bytes_for_replica_parallel=%s,'
+      ' max_replicas_for_replica_parallel=%s, enable_pinned_host_transfer=%s',
       use_replica_parallel,
+      min_slice_bytes_for_replica_parallel,
+      max_replicas_for_replica_parallel,
       enable_pinned_host_transfer,
   )
 
@@ -349,10 +435,7 @@ def transfer_arrays_to_host(
     has_pinned_host = any(
         m.kind == 'pinned_host' for m in device.addressable_memories()
     )
-    return (
-        enable_pinned_host_transfer
-        and has_pinned_host
-    )
+    return enable_pinned_host_transfer and has_pinned_host
 
   def async_transfer_slice(
       rslice: ReplicaSlice,
@@ -374,7 +457,13 @@ def transfer_arrays_to_host(
 
   # Gather the replica slices to be saved for each array.
   rslices_per_array = [
-      get_replica_slices(arr, replica_id, use_replica_parallel)
+      get_replica_slices(
+          arr,
+          replica_id,
+          use_replica_parallel,
+          min_slice_bytes_for_replica_parallel,
+          max_replicas_for_replica_parallel,
+      )
       for arr in arrays
   ]
   # Kick off transfers for all replica slices to be saved.
