@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,13 +17,10 @@
 import asyncio
 import collections
 from collections.abc import Mapping
-import contextlib
 import os
 import re
-from typing import Any, AsyncIterator, Dict, Optional, Protocol, Sequence, Union
+from typing import Any, Dict, Optional, Sequence, Union
 
-from absl import logging
-import humanize
 import jax
 from jax.experimental import layout
 import jax.numpy as jnp
@@ -32,6 +29,7 @@ from orbax.checkpoint._src.arrays import fragments
 from orbax.checkpoint._src.arrays import numpy_utils as np_utils
 from orbax.checkpoint._src.arrays import types
 from orbax.checkpoint._src.multihost import multihost
+from orbax.checkpoint._src.serialization import limits
 from orbax.checkpoint._src.serialization import replica_slices
 from orbax.checkpoint._src.serialization import tensorstore_utils as ts_utils
 import tensorstore as ts
@@ -42,7 +40,10 @@ _REMOVED_VALUE = 'Value removed'
 _CHECKPOINT_SUCCESS = 'checkpoint_write_success'
 
 Index = types.Index
-Layout = layout.Layout
+if jax.__version_info__ >= (0, 6, 2):
+  Format = layout.Format
+else:
+  Format = layout.Layout
 Shape = types.Shape
 
 
@@ -63,6 +64,17 @@ def _spec_has_metadata(tree):
 
 
 def _get_kvstore_for_gcs(ckpt_path: str):
+  """Constructs a TensorStore kvstore spec for a GCS path.
+
+  Args:
+    ckpt_path: A GCS path of the form gs://<bucket>/<path>.
+
+  Returns:
+    A dictionary containing the TensorStore kvstore spec.
+
+  Raises:
+    ValueError: if ckpt_path is not a valid GCS path.
+  """
   m = re.fullmatch('^gs://([^/]*)/(.*)$', ckpt_path, re.DOTALL)
   if m is None:
     raise ValueError(
@@ -71,7 +83,14 @@ def _get_kvstore_for_gcs(ckpt_path: str):
     )
   gcs_bucket = m.group(1)
   path_without_bucket = m.group(2)
-  return {'driver': 'gcs', 'bucket': gcs_bucket, 'path': path_without_bucket}
+  # TODO(stoelinga): Switch to gcs_grpc by default.
+  # gcs_grpc performs roughly twice as fast as gcs backend.
+  gcs_backend = os.environ.get('TENSORSTORE_GCS_BACKEND', 'gcs')
+  return {
+      'driver': f'{gcs_backend}',
+      'bucket': gcs_bucket,
+      'path': path_without_bucket,
+  }
 
 
 def get_tensorstore_spec(ckpt_path: str, ocdbt: bool = False):
@@ -104,89 +123,6 @@ def get_tensorstore_spec(ckpt_path: str, ocdbt: bool = False):
   return spec
 
 
-class ByteLimiter(Protocol):
-
-  async def wait_for_bytes(self, requested_bytes: int):
-    ...
-
-  async def release_bytes(self, requested_bytes: int):
-    ...
-
-
-# Lifted from T5X.
-class LimitInFlightBytes(ByteLimiter):
-  """Limits in-flight bytes when reading/writing checkpoints per process."""
-
-  def __init__(self, num_bytes: int):
-    if num_bytes <= 0:
-      raise ValueError(f'Must provide positive `num_bytes`. Found: {num_bytes}')
-    self._max_bytes = num_bytes
-    self._available_bytes = num_bytes
-    self._cv = asyncio.Condition(lock=asyncio.Lock())
-
-  async def wait_for_bytes(self, requested_bytes: int):
-    """Reserve bytes."""
-    if requested_bytes >= self._max_bytes:
-      raise ValueError('Requested more bytes than we reserved space for: '
-                       f'{requested_bytes} > {self._max_bytes}')
-    async with self._cv:
-      await self._cv.wait_for(lambda: self._available_bytes > requested_bytes)
-      self._available_bytes -= requested_bytes
-      logging.vlog(
-          1,
-          'Reserved bytes: %s | Remaining bytes: %s',
-          humanize.naturalsize(requested_bytes, binary=True),
-          humanize.naturalsize(self._available_bytes, binary=True),
-      )
-      assert self._available_bytes >= 0
-
-  async def release_bytes(self, requested_bytes: int):
-    async with self._cv:
-      self._available_bytes += requested_bytes
-      logging.vlog(
-          1,
-          'Releasing bytes: %s | Available bytes: %s',
-          humanize.naturalsize(requested_bytes, binary=True),
-          humanize.naturalsize(self._available_bytes, binary=True),
-      )
-      assert self._available_bytes <= self._max_bytes
-      self._cv.notify_all()
-
-
-class UnlimitedInFlightBytes(ByteLimiter):
-
-  async def wait_for_bytes(self, requested_bytes: int):
-    del requested_bytes
-    return
-
-  async def release_bytes(self, requested_bytes: int):
-    del requested_bytes
-    return
-
-
-def get_byte_limiter(concurrent_bytes: Optional[int] = None) -> ByteLimiter:
-  if concurrent_bytes is None:
-    return UnlimitedInFlightBytes()
-  if concurrent_bytes <= 0:
-    raise ValueError(
-        f'Must provide positive `concurrent_bytes`. Found: {concurrent_bytes}'
-    )
-  return LimitInFlightBytes(concurrent_bytes)
-
-
-@contextlib.asynccontextmanager
-async def reserved_bytes(
-    byte_limiter: ByteLimiter,
-    nbytes: int,
-) -> AsyncIterator[None]:
-  """Reserves some bytes for the duration of the context."""
-  await byte_limiter.wait_for_bytes(nbytes)
-  try:
-    yield
-  finally:
-    await byte_limiter.release_bytes(nbytes)
-
-
 async def async_serialize(
     arr_inp: jax.Array,
     tensorstore_spec: Dict[str, Any],
@@ -195,7 +131,7 @@ async def async_serialize(
     replica_id: int = 0,
     use_replica_parallel: bool = True,
     transaction: Optional[ts.Transaction] = None,
-    byte_limiter: Optional[ByteLimiter] = None,
+    byte_limiter: Optional[limits.ByteLimiter] = None,
 ):
   """Serialize an array using TensorStore.
 
@@ -229,7 +165,7 @@ async def async_serialize(
       use_replica_parallel,
   )[0]
 
-  byte_limiter = byte_limiter or get_byte_limiter()
+  byte_limiter = byte_limiter or limits.get_byte_limiter()
   # 'metadata' may not be present at the top level (for example, if we are using
   # a 'cast' driver).
   if not _spec_has_metadata(tensorstore_spec):
@@ -254,7 +190,7 @@ async def async_serialize_from_host(
     context: Optional[ts.Context] = None,
     primary_host: Optional[int] = 0,
     transaction: Optional[ts.Transaction] = None,
-    byte_limiter: Optional[ByteLimiter] = None,
+    byte_limiter: Optional[limits.ByteLimiter] = None,
 ):
   """Serialize replica slices using TensorStore.
 
@@ -276,7 +212,7 @@ async def async_serialize_from_host(
   """
   if not rslices_on_host.is_on_host:
     raise ValueError('Replica slices have not been transferred to host.')
-  byte_limiter = byte_limiter or get_byte_limiter()
+  byte_limiter = byte_limiter or limits.get_byte_limiter()
   if not _spec_has_metadata(tensorstore_spec):
     raise KeyError('`metadata` not found in tensorstore spec.')
   # Set dtype if it's not in spec
@@ -309,11 +245,11 @@ async def async_serialize_from_host(
       transaction=transaction,
   )
 
-  async def write_fragment(fragment: fragments.Fragment):
+  async def write_fragment(fragment: fragments.ConcreteFragment):
     """Writes a single fragment using TensorStore. No copy is performed."""
     assert isinstance(fragment.value, np.ndarray)
     requested_bytes = estimate_write_memory_footprint(fragment.value)
-    async with reserved_bytes(byte_limiter, requested_bytes):
+    async with limits.reserved_bytes(byte_limiter, requested_bytes):
       await t[fragment.index].write(
           fragment.value,
           # Avoid additional copy of input array into the TensorStore chunk
@@ -333,8 +269,9 @@ def estimate_write_memory_footprint(arr: np.ndarray) -> int:
   return arr.size * arr.dtype.itemsize
 
 
-def estimate_read_memory_footprint(t: ts.TensorStore,
-                                   domain: ts.IndexDomain) -> int:
+def estimate_read_memory_footprint(
+    t: ts.TensorStore, domain: ts.IndexDomain
+) -> int:
   """Estimates memory required to read a given domain."""
   rank = t.rank
   num_bytes = t.dtype.numpy_dtype.itemsize
@@ -362,7 +299,7 @@ def estimate_read_memory_footprint(t: ts.TensorStore,
     upper = origin_value + shape[i] - chunk_origin_value
     lower_aligned = lower // chunk_size * chunk_size
     upper_aligned = -(-upper // chunk_size) * chunk_size
-    num_bytes *= (upper_aligned - lower_aligned)
+    num_bytes *= upper_aligned - lower_aligned
 
   return num_bytes
 
@@ -398,9 +335,9 @@ async def _read_array_index_and_device_put(
     global_shape: Shape,
     new_shard_shape: Shape,
     dtype: jnp.dtype,
-    byte_limiter: ByteLimiter,
+    byte_limiter: limits.ByteLimiter,
     strict: bool,
-    dll: Optional[layout.DeviceLocalLayout],
+    dll,
     memory_kind: Optional[str],
 ) -> list[jax.Array]:
   """Callback that reads an array index and places on the devices."""
@@ -419,10 +356,10 @@ async def _read_array_index_and_device_put(
     else:
       raise ValueError(
           f'Requested shape: {global_shape} is not compatible with the stored'
-          f' shape: {t.shape}. Truncating/padding is disabled by setting of'
-          ' `strict=True`. When using standard Orbax APIs, this behavior can'
-          ' be modified by specifying `strict=False` in `ArrayRestoreArgs` for'
-          ' any array in which padding/truncation is desired.'
+          f' shape: {t.shape}. Truncating/padding is disabled. To enable it,'
+          ' set `strict=False` in `ArrayRestoreArgs` for any array in v0 API'
+          ' or `enable_padding_and_truncation=True` in `ArrayOptions.Loading`'
+          ' in v1 API.'
       )
   else:
     requested_domain = ts.IndexTransform(input_shape=global_shape)[index].domain
@@ -436,7 +373,7 @@ async def _read_array_index_and_device_put(
   # TODO(b/381111280) This de-duplication of reads does not fully solve the
   # problem of read amplification, since we can still run into problems if
   # we are resharding. See b/381111280 for details.
-  async with reserved_bytes(byte_limiter, requested_bytes):
+  async with limits.reserved_bytes(byte_limiter, requested_bytes):
     try:
       shard = await _read_shard(
           t=t,
@@ -454,7 +391,7 @@ async def _read_array_index_and_device_put(
       sharding = jax.sharding.SingleDeviceSharding(
           device, memory_kind=memory_kind
       )
-      result.append(jax.device_put(shard, Layout(dll, sharding)))
+      result.append(jax.device_put(shard, Format(dll, sharding)))  # pytype: disable=wrong-arg-types
   return result
 
 
@@ -471,9 +408,9 @@ async def read_and_create_array(
     new_shard_shape: Shape,
     sharding: jax.sharding.Sharding,
     dtype: jnp.dtype,
-    byte_limiter: ByteLimiter,
+    byte_limiter: limits.ByteLimiter,
     strict: bool,
-    dll: Optional[layout.DeviceLocalLayout],
+    dll,
 ) -> jax.Array:
   """Read shards from TensorStore and create a jax.Array."""
   local_indices_devices_map: dict[types.HashableIndex, list[jax.Device]] = (
@@ -505,22 +442,22 @@ async def read_and_create_array(
 
 
 async def async_deserialize(
-    user_sharding: jax.sharding.Sharding | Layout,
+    user_sharding: jax.sharding.Sharding | Format,  # pytype: disable=wrong-arg-types  # pytype: disable=unsupported-operands
     tensorstore_spec: Union[ts.Spec, Dict[str, Any]],
     global_shape: Optional[Shape] = None,
     dtype: Optional[jnp.dtype] = None,
     *,
-    byte_limiter: Optional[ByteLimiter] = None,
+    byte_limiter: Optional[limits.ByteLimiter] = None,
     context: Optional[ts.Context] = None,
     assume_metadata: bool = False,
     strict: bool = True,
 ) -> jax.Array:
   """Reads an array using TensorStore."""
-  byte_limiter = byte_limiter or get_byte_limiter()
+  byte_limiter = byte_limiter or limits.get_byte_limiter()
   context = context or ts_utils.get_ts_context(use_ocdbt=False)
   sharding = (
       user_sharding.sharding
-      if isinstance(user_sharding, Layout)
+      if isinstance(user_sharding, Format)
       else user_sharding
   )
   if not isinstance(sharding, jax.sharding.Sharding):
@@ -528,11 +465,16 @@ async def async_deserialize(
         'sharding passed to deserialization should be specified, concrete and'
         f' an instance of `jax.sharding.Sharding`. Got {sharding}'
     )
-  dll = (
-      user_sharding.device_local_layout
-      if isinstance(user_sharding, Layout)
-      else None
-  )
+
+  if isinstance(user_sharding, Format):
+    dll = (
+        user_sharding.layout
+        if jax.__version_info__ >= (0, 6, 3)
+        else user_sharding.device_local_layout  # type: ignore
+    )
+  else:
+    dll = None
+
   t = await ts.open(
       tensorstore_spec,
       open=True,
@@ -541,6 +483,10 @@ async def async_deserialize(
   )
   global_shape = tuple(t.shape if global_shape is None else global_shape)
   new_shard_shape = sharding.shard_shape(global_shape)
+  if new_shard_shape != t.shape:
+    jax.monitoring.record_event(
+        '/jax/orbax/checkpoint/deserialize/shard_shape_changed'
+    )
   return await read_and_create_array(
       t,
       global_shape=global_shape,
