@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,12 +24,13 @@ from absl.testing import absltest
 from absl.testing import parameterized
 import jax
 from jax import dtypes as _dtypes
-from jax.experimental import layout
 import jax.numpy as jnp
 import numpy as np
 from orbax.checkpoint import test_utils
 from orbax.checkpoint._src import asyncio_utils
+from orbax.checkpoint._src.arrays import sharding as arrays_sharding_lib
 from orbax.checkpoint._src.futures import future
+from orbax.checkpoint._src.serialization import limits
 from orbax.checkpoint._src.serialization import serialization
 from orbax.checkpoint._src.serialization import tensorstore_utils as ts_utils
 import tensorstore as ts
@@ -37,9 +38,8 @@ import tensorstore as ts
 
 NamedSharding = jax.sharding.NamedSharding
 P = jax.sharding.PartitionSpec
-DLL = layout.DeviceLocalLayout
-Layout = layout.Layout
-
+DLL = arrays_sharding_lib.DLL
+Format = arrays_sharding_lib.Format
 jax.config.update('jax_enable_x64', True)
 
 
@@ -158,6 +158,15 @@ class CheckpointTest(parameterized.TestCase):
   def assertDtypesMatch(self, x, y):
     self.assertEqual(_dtype(x), _dtype(y))
 
+  def add_monitoring_listener(self) -> list[tuple[str, dict[str, Any]]]:
+    """Adds a listener to capture Jax monitoring, returns a list of events."""
+    jax_events = []
+    jax.monitoring.clear_event_listeners()
+    def monitoring_listener(event, **kwargs):
+      jax_events.append((event, kwargs))
+    jax.monitoring.register_event_listener(monitoring_listener)
+    return jax_events
+
   def test_memory_consumption(self):
     global_mesh = create_global_mesh((2, 4), ('x', 'y'))
     inp_shape = (2_048, 4_096)
@@ -180,7 +189,7 @@ class CheckpointTest(parameterized.TestCase):
           sharding,
           tspec,
           inp_shape,
-          byte_limiter=serialization.LimitInFlightBytes(4_200_000),
+          byte_limiter=limits.LimitInFlightBytes(4_200_000),
       )
       r.block_until_ready()
 
@@ -292,6 +301,8 @@ class CheckpointTest(parameterized.TestCase):
 
   @parameterized.product(input_dtype=[np.int32, jnp.bfloat16])
   def test_checkpointing_with_bigger_shape_jax_array(self, input_dtype):
+    monitoring_events = self.add_monitoring_listener()
+
     global_mesh = create_global_mesh((2, 2), ('x', 'y'))
     global_input_shape = (8, 2)
     num = math.prod(global_input_shape)
@@ -334,6 +345,12 @@ class CheckpointTest(parameterized.TestCase):
     for l in m2.addressable_shards:
       self.assertArraysEqual(l.data, global_input_data1.astype('float32'))
 
+    self.assertLen(monitoring_events, 1)
+    self.assertEqual(
+        ('/jax/orbax/checkpoint/deserialize/shard_shape_changed', {}),
+        monitoring_events[0],
+    )
+
   @parameterized.product(input_dtype=[jnp.int4, jnp.int8])
   def test_checkpointing_with_int4(self, input_dtype):
     global_mesh = create_global_mesh((2, 2), ('x', 'y'))
@@ -585,11 +602,11 @@ class CheckpointTest(parameterized.TestCase):
     np_inp = np.arange(32).reshape(8, 4)
     s = NamedSharding(mesh, P('x', 'y'))
     arr = jax.device_put(np_inp, s)
-
-    out_layout = Layout(
-        device_local_layout=DLL(
-            arr.layout.device_local_layout.major_to_minor[::-1],
-            arr.layout.device_local_layout._tiling,
+    arr_layout = arrays_sharding_lib.get_device_local_layout(arr)
+    out_layout = Format(
+        DLL(
+            arr_layout.major_to_minor[::-1],
+            arr_layout.tiling,
         ),
         sharding=arr.sharding,
     )
@@ -605,7 +622,7 @@ class CheckpointTest(parameterized.TestCase):
 
     (out,) = deserialize([out_layout], tspecs)
 
-    self.assertEqual(out.layout, out_layout)
+    self.assertEqual(out.format, out_layout)
     self.assertIsInstance(out, jax.Array)
     self.assertArraysEqual(out, np_inp)
     for s in out.addressable_shards:
