@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -161,8 +161,7 @@ def _main(argv):
         f"--jax_num_tasks={num_processes}",
         f"--jax_task_id={i}",
         f"--jax_controller_address=localhost:{jax_port}",
-        "--jax_heartbeat_interval=1s",
-        "--jax_max_missing_heartbeats=3",
+        "--jax_heartbeat_timeout=3s",
         f"--jax_distributed_shutdown_timeout={_WORKER_SHUTDOWN_TIMEOUT.value}s",
         "--vmodule=client=10,service=10",
     ]
@@ -300,18 +299,19 @@ class MultiProcessTest(absltest.TestCase):
   def setUp(self):
     """Start distributed service."""
     super().setUp()
-    assert jax.process_count() == NUM_PROCESSES.value, (
-        jax.process_count(),
-        NUM_PROCESSES.value,
-    )
-    # Make sure all processes are at the same test case.
-    client = multihost.get_jax_distributed_client()
-    # Note that the name of this barrier is long and complicated, to prevent
-    # any collisions with barriers in user test code.
-    client.wait_at_barrier(
-        f"multiprocess_test_ensure_all_processes_arrive_at_test_case_{self._testMethodName}",
-        10000,
-    )
+    if multihost.is_pathways_backend():
+      assert (
+          jax.process_count() == 1
+      ), "Expected 1 process for Pathways backend."
+    else:
+      # Make sure all processes are at the same test case.
+      client = multihost.get_jax_distributed_client()
+      # Note that the name of this barrier is long and complicated, to prevent
+      # any collisions with barriers in user test code.
+      client.wait_at_barrier(
+          f"multiprocess_test_ensure_all_processes_arrive_at_test_case_{self._testMethodName}",
+          10000,
+      )
 
   def multiprocess_create_tempdir(
       self, name: str | None = None
