@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,13 +14,14 @@
 
 """Test for utils module."""
 
-from typing import Any, Mapping, NamedTuple, Sequence
+from typing import Any, Mapping, Sequence
 
 from absl.testing import absltest
 from absl.testing import parameterized
 from etils import epath
 import flax
 import jax
+import jax.tree_util as jtu
 import numpy as np
 import optax
 from orbax.checkpoint import test_utils
@@ -28,19 +29,13 @@ from orbax.checkpoint._src.testing import test_tree_utils
 from orbax.checkpoint._src.tree import utils as tree_utils
 
 
-Diff = tree_utils.Diff
 PyTree = tree_utils.PyTree
 
 
-class Record(NamedTuple):
-  first: Any
-  second: Any
-
-
-def _as_string_equality(a, b) -> bool:
-  if a is None or b is None:
-    return a == b
-  return str(a) == str(b)
+@flax.struct.dataclass
+class FlaxRecord:
+  alpha: Any
+  beta: Any
 
 
 # TODO: b/365169723 - Add tests: PyTreeMetadataOptions.support_rich_types=True.
@@ -278,282 +273,219 @@ class UtilsTest(parameterized.TestCase):
     self.assertFalse(tree_utils.isinstance_of_namedtuple(nt))
     self.assertFalse(tree_utils.issubclass_of_namedtuple(type(nt)))
 
-  @parameterized.parameters(
-      (
-          {'a': 100, 'b': [200, {'c': 300}]},
-          {'a': 1, 'b': [2, {'c': 3}]},
-          {'a': 1, 'b': [2, {'c': 3}]},
-      ),
-      (
-          {'a': 100, 'b': [200, {'c': 300}]},
-          {'a': 1, 'b': [2, {'c': 3, 'd': 4}], 'e': 5},
-          {'a': 1, 'b': [2, {'c': 3}]},
-      ),
-      (
-          {'a': [100, 200], 'b': {'c': 300}},
-          {'a': (1, 2), 'b': {'c': 3, 'd': 4}},
-          {'a': [1, 2], 'b': {'c': 3}},
-      ),
-      (
-          {},
-          {'a': 1, 'b': 2},
-          {},
-      ),
-      (
-          {},
-          {},
-          {},
-      ),
-  )
-  def test_tree_trim_valid(
-      self, template: PyTree, structure: PyTree, expected: PyTree
-  ):
-    self.assertDictEqual(expected, tree_utils.tree_trim(template, structure))
 
-  @parameterized.parameters(
-      int,
-      float,
-      complex,
-      str,
-      bytes,
-      bool,
-      np.int32,
-      np.float32,
-  )
-  def test_tree_trim_leaves(self, val_type: Any):
-    val = val_type()
-    self.assertEqual(val, tree_utils.tree_trim(val, val))
+class LookUpPyTreeKeyTest(parameterized.TestCase):
 
-  def test_tree_trim_extra_path_in_structure(self):
-    structure = {'a': 1, 'b': [2, {'c': 3}]}
+  def test_retrieves_element_of_internal_node(self):
+    self.assertEqual(
+        tree_utils.look_up_pytree_key(
+            [[0], [10], [20]],
+            jtu.SequenceKey(1),
+        ),
+        [10],
+    )
+    self.assertEqual(
+        tree_utils.look_up_pytree_key(
+            {'a': [0], 'b': [100], 'c': [200]},
+            jtu.DictKey('b'),
+        ),
+        [100],
+    )
+    self.assertEqual(
+        tree_utils.look_up_pytree_key(
+            FlaxRecord(alpha=[0, 10, 20], beta={'a': 100, 'b': 200}),
+            jtu.FlattenedIndexKey(1),
+        ),
+        {'a': 100, 'b': 200},
+    )
+    self.assertEqual(
+        tree_utils.look_up_pytree_key(
+            FlaxRecord(alpha=[0, 10, 20], beta={'a': 100, 'b': 200}),
+            jtu.GetAttrKey('beta'),
+        ),
+        {'a': 100, 'b': 200},
+    )
 
-    template = {'a': 100, 'b': [200, {'c': 300, 'd': 400}], 'e': 500}
-    with self.assertRaisesRegex(ValueError, 'Path "b.1.d" exists in template'):
-      tree_utils.tree_trim(template, structure)
-
-    template = {'a': 100, 'b': [200, {'c': 300}], 'e': 500}
-    with self.assertRaisesRegex(ValueError, 'Path "e" exists in template'):
-      tree_utils.tree_trim(template, structure)
-
-  def test_tree_trim_empty_template(self):
-    structure = {}
-    template = {'a': 1, 'b': 2}
-    with self.assertRaisesRegex(ValueError, 'Path "a" exists in template'):
-      tree_utils.tree_trim(template, structure)
-
-  def test_tree_trim_mismatch_structure(self):
-    structure = 10
-    template = {'a': 20}
-    with self.assertRaisesRegex(ValueError, 'Path "a" exists in template'):
-      tree_utils.tree_trim(template, structure)
-
-  def test_tree_trim_mismatch_template(self):
-    structure = {'a': 10}
-    template = 20
-    with self.assertRaisesRegex(ValueError, 'Path "" exists in template'):
-      tree_utils.tree_trim(template, structure)
-
-  def test_tree_trim_v2_custom_node(self):
-    @flax.struct.dataclass
-    class SimpleData:
-      x: int
-      y: str
-
-    structure = {'data': SimpleData(x=1, y='hello'), 'other': 10}
-    template = {'data': SimpleData(x=100, y='world')}
-    expected = {'data': SimpleData(x=1, y='hello')}
-
-    result = tree_utils.tree_trim(template, structure)
-    test_utils.assert_tree_equal(self, expected, result)
+  def test_raises_key_error_if_key_is_not_present(self):
+    with self.subTest('sequence'):
+      with self.assertRaises(KeyError):
+        tree_utils.look_up_pytree_key(
+            [[0], [10], [20]],
+            jtu.SequenceKey(4),
+        )
+    with self.subTest('dict'):
+      with self.assertRaises(KeyError):
+        tree_utils.look_up_pytree_key(
+            {'a': [0], 'b': [100], 'c': [200]},
+            jtu.DictKey('d'),
+        )
+    with self.subTest('flattened_index'):
+      with self.assertRaises(KeyError):
+        tree_utils.look_up_pytree_key(
+            FlaxRecord(alpha=[0, 10, 20], beta={'a': 100, 'b': 200}),
+            jtu.FlattenedIndexKey(2),
+        )
+    with self.subTest('attr'):
+      with self.assertRaises(KeyError):
+        tree_utils.look_up_pytree_key(
+            FlaxRecord(alpha=[0, 10, 20], beta={'a': 100, 'b': 200}),
+            jtu.GetAttrKey('gamma'),
+        )
 
 
-class TreeDifferenceTest(parameterized.TestCase):
+class PyTreePathAsTreePathTest(parameterized.TestCase):
 
-  def test_doc_example(self):
-    first = {'x': [1, 2, 3], 'y': 'same', 'z': {'p': {}, 'q': 4, 'r': 6}}
-    second = {'x': [1, 3], 'y': 'same', 'z': {'p': {}, 'q': 5, 's': 6}}
-    actual = tree_utils.tree_difference(first, second)
-    expected = {
-        'x': Diff(lhs=(list, 3), rhs=(list, 2)),
-        'z': {
-            'r': Diff(lhs=6, rhs=None),
-            's': Diff(lhs=None, rhs=6),
-        },
+  def test_converts_pytree_path_to_tree_path(self):
+    structure = {
+        'a': [0, 10],
+        'b': FlaxRecord(alpha=[65], beta=(66,)),
     }
-    self.assertEqual(actual, expected)
+    paths_and_values, _ = jtu.tree_flatten_with_path(structure)
+    legacy_paths = [
+        tree_utils.pytree_path_as_tree_path(p) for p, _ in paths_and_values
+    ]
 
-  def test_uses_leaves_equal_predicate(self):
-    first = {'x': [1, 2, 3], 'y': 'same', 'z': {'p': {}, 'q': 4, 'r': 6}}
-    second = {'x': [1, 2, 3], 'y': 'same', 'z': {'p': {}, 'q': 4, 's': 6}}
-    leaf_equality_fn = lambda x, y: x == y
-
-    actual = tree_utils.tree_difference(
-        first, second, leaves_equal=leaf_equality_fn
-    )
-    expected = {'z': {'r': Diff(lhs=6, rhs=None), 's': Diff(lhs=None, rhs=6)}}
-    self.assertEqual(actual, expected)
-
-  @parameterized.parameters([
-      ((),),
-      ([],),
-      ({},),
-      ((1, 2, 3),),  # tuples of same elements.
-      (
-          (1, 2, 3),
-          (2, 3, 4),
-      ),  # tuples of same length using type equality.
-      ([1, 2, 3],),
-      (
-          [1, 2, 3],
-          [2, 3, 4],
-      ),  # lists of same length using type equality.
-      ({'one': 1, 'two': 2, 'three': 3},),
-      (Record(first=1, second=2),),
-  ])
-  def test_match_returns_none(self, lhs, rhs=None):
-    if rhs is None:
-      rhs = lhs
-    self.assertIsNone(tree_utils.tree_difference(lhs, rhs))
-
-  @parameterized.parameters([
-      ('abc', 'abc'),
-      (123, 123),
-      (123, '123'),
-      ('123', 123),
-  ])
-  def test_match_returns_none_custom_equality_fn(self, lhs, rhs):
-    self.assertIsNone(
-        tree_utils.tree_difference(lhs, rhs, leaves_equal=_as_string_equality)
+    self.assertSameStructure(
+        [
+            ('a', 0),
+            ('a', 1),
+            ('b', 'alpha', 0),
+            ('b', 'beta', 0),
+        ],
+        legacy_paths,
     )
 
-  @parameterized.parameters([('abc', 'bcd'), (123, 234), (123, '123')])
-  def test_match_returns_diff_custom_equality_fn(self, lhs, rhs):
+
+class SelectByPyTreePathTest(parameterized.TestCase):
+
+  def test_select_dict(self):
+    tree = {'a': 1, 'b': 2}
     self.assertEqual(
-        tree_utils.tree_difference(lhs, rhs, leaves_equal=lambda x, y: x == y),
-        Diff(lhs=lhs, rhs=rhs),
+        tree_utils.select_by_pytree_path(tree, (jtu.DictKey('a'),)), 1
+    )
+    self.assertEqual(
+        tree_utils.select_by_pytree_path(tree, (jtu.DictKey('b'),)), 2
     )
 
-  def test_named_tuples_are_compared_fieldwise(self):
+  def test_select_sequence(self):
+    tree = [10, 20]
     self.assertEqual(
-        tree_utils.tree_difference(
-            Record(first=3, second=4), Record(first=5, second='four')
+        tree_utils.select_by_pytree_path(tree, (jtu.SequenceKey(0),)), 10
+    )
+    self.assertEqual(
+        tree_utils.select_by_pytree_path(tree, (jtu.SequenceKey(1),)), 20
+    )
+
+  def test_select_named_tuple(self):
+    tree = test_tree_utils.IntegerNamedTuple(1, 2)
+    self.assertEqual(
+        tree_utils.select_by_pytree_path(tree, (jtu.GetAttrKey('x'),)), 1
+    )
+    self.assertEqual(
+        tree_utils.select_by_pytree_path(tree, (jtu.GetAttrKey('y'),)), 2
+    )
+
+  def test_select_object(self):
+    tree = FlaxRecord(alpha=1, beta=2)
+    self.assertEqual(
+        tree_utils.select_by_pytree_path(tree, (jtu.GetAttrKey('alpha'),)), 1
+    )
+    self.assertEqual(
+        tree_utils.select_by_pytree_path(tree, (jtu.GetAttrKey('beta'),)), 2
+    )
+
+  def test_select_nested(self):
+    tree = {'a': FlaxRecord(alpha={'x': 10}, beta=20)}
+    self.assertEqual(
+        tree_utils.select_by_pytree_path(
+            tree, (jtu.DictKey('a'), jtu.GetAttrKey('alpha'), jtu.DictKey('x'))
         ),
-        Record(first=None, second=Diff(lhs=4, rhs='four')),
+        10,
     )
-
-  def test_named_tuples_are_compared_fieldwise_custom_equality_fn(self):
     self.assertEqual(
-        tree_utils.tree_difference(
-            Record(first='3', second=4),
-            Record(first=3, second='four'),
-            leaves_equal=_as_string_equality,
+        tree_utils.select_by_pytree_path(
+            tree, (jtu.DictKey('a'), jtu.GetAttrKey('beta'))
         ),
-        Record(first=None, second=Diff(lhs=4, rhs='four')),
+        20,
     )
 
-  @parameterized.product(
-      sequence_type=[tuple, list], eq_fn=[None, lambda x, y: x == y]
-  )
-  def test_length_difference_is_reported_as_pair_of_type_and_length(
-      self, sequence_type, eq_fn
-  ):
-    s = sequence_type
+  def test_select_empty_path(self):
+    tree = {'a': 1}
+    self.assertEqual(tree_utils.select_by_pytree_path(tree, ()), tree)
+
+  def test_select_dict_raises_error(self):
+    tree = {'a': 1}
+    with self.assertRaisesRegex(ValueError, 'Path .* does not exist'):
+      tree_utils.select_by_pytree_path(tree, (jtu.DictKey('b'),))
+
+  def test_select_sequence_raises_error(self):
+    tree = [10, 20]
+    with self.assertRaisesRegex(ValueError, 'Path .* does not exist'):
+      tree_utils.select_by_pytree_path(tree, (jtu.SequenceKey(2),))
+
+  def test_select_named_tuple_raises_error(self):
+    tree = test_tree_utils.IntegerNamedTuple(1, 2)
+    with self.assertRaisesRegex(ValueError, 'Path .* does not exist'):
+      tree_utils.select_by_pytree_path(tree, (jtu.GetAttrKey('z'),))
+
+  def test_select_object_raises_error(self):
+    tree = FlaxRecord(alpha=1, beta=2)
+    with self.assertRaisesRegex(ValueError, 'Path .* does not exist'):
+      tree_utils.select_by_pytree_path(tree, (jtu.GetAttrKey('gamma'),))
+
+
+class SelectByTreePathTest(parameterized.TestCase):
+
+  def test_select_dict(self):
+    tree = {'a': 1, 'b': 2}
+    self.assertEqual(tree_utils.select_by_tree_path(tree, ('a',)), 1)
+    self.assertEqual(tree_utils.select_by_tree_path(tree, ('b',)), 2)
+
+  def test_select_sequence(self):
+    tree = [10, 20]
+    self.assertEqual(tree_utils.select_by_tree_path(tree, (0,)), 10)
+    self.assertEqual(tree_utils.select_by_tree_path(tree, (1,)), 20)
+
+  def test_select_named_tuple(self):
+    tree = test_tree_utils.IntegerNamedTuple(1, 2)
+    self.assertEqual(tree_utils.select_by_tree_path(tree, ('x',)), 1)
+    self.assertEqual(tree_utils.select_by_tree_path(tree, ('y',)), 2)
+
+  def test_select_object(self):
+    tree = FlaxRecord(alpha=1, beta=2)
+    self.assertEqual(tree_utils.select_by_tree_path(tree, ('alpha',)), 1)
+    self.assertEqual(tree_utils.select_by_tree_path(tree, ('beta',)), 2)
+
+  def test_select_nested(self):
+    tree = {'a': FlaxRecord(alpha={'x': 10}, beta=20)}
     self.assertEqual(
-        tree_utils.tree_difference(
-            s([1, 2, 3]), s(['one', 'two']), leaves_equal=eq_fn
-        ),
-        Diff(lhs=(s, 3), rhs=(s, 2)),
+        tree_utils.select_by_tree_path(tree, ('a', 'alpha', 'x')), 10
     )
+    self.assertEqual(tree_utils.select_by_tree_path(tree, ('a', 'beta')), 20)
 
-  @parameterized.parameters(tuple, list)
-  def test_element_difference_is_none_at_matching_indices(self, sequence_type):
-    s = sequence_type
-    self.assertEqual(
-        tree_utils.tree_difference(s([1, 2, 3]), s([1, 'two', 2])),
-        s([None, Diff(lhs=2, rhs='two'), None]),
-    )
+  def test_select_empty_path(self):
+    tree = {'a': 1}
+    self.assertEqual(tree_utils.select_by_tree_path(tree, ()), tree)
 
-  @parameterized.parameters(tuple, list)
-  def test_element_difference_is_none_at_matching_indices_custom_equality_fn(
-      self, sequence_type
-  ):
-    s = sequence_type
-    leaf_equality_fn = _as_string_equality
+  def test_select_dict_raises_error(self):
+    tree = {'a': 1}
+    with self.assertRaisesRegex(ValueError, 'Path .* does not exist'):
+      tree_utils.select_by_tree_path(tree, ('b',))
 
-    self.assertEqual(
-        tree_utils.tree_difference(
-            s([1, 2, 3]), s([1, 'two', '3']), leaves_equal=leaf_equality_fn
-        ),
-        s([None, Diff(lhs=2, rhs='two'), None]),
-    )
+  def test_select_sequence_raises_error(self):
+    tree = [10, 20]
+    with self.assertRaisesRegex(ValueError, 'Path .* does not exist'):
+      tree_utils.select_by_tree_path(tree, (2,))
 
-  def test_mapping_difference_only_contains_mismatched_elements(self):
-    self.assertEqual(
-        tree_utils.tree_difference(
-            {'one': 1, 'two': 2, 'three': 3},
-            {'one': 1, 'two': 'two', 'three': 3},
-        ),
-        {'two': Diff(lhs=2, rhs='two')},
-    )
+  def test_select_named_tuple_raises_error(self):
+    tree = test_tree_utils.IntegerNamedTuple(1, 2)
+    with self.assertRaisesRegex(ValueError, 'Path .* does not exist'):
+      tree_utils.select_by_tree_path(tree, ('z',))
 
-  def test_difference_with_custom_pytree_node(self):
-    @flax.struct.dataclass
-    class NewModel(flax.struct.PyTreeNode):
-      x: Any
-      y: Any
-
-    self.assertEqual(
-        tree_utils.tree_difference(
-            NewModel(
-                x=jax.ShapeDtypeStruct(shape=(16,), dtype=np.int64),
-                y=Ellipsis,
-            ),
-            NewModel(
-                x=jax.ShapeDtypeStruct(shape=(16,), dtype=np.int64),
-                y=jax.ShapeDtypeStruct(shape=(16,), dtype=np.float64),
-            ),
-        ),
-        NewModel(
-            x=None,
-            y=Diff(
-                lhs=Ellipsis,
-                rhs=jax.ShapeDtypeStruct(shape=(16,), dtype=np.float64),
-            ),
-        ),
-    )
-
-  def test_uses_is_leaf_to_recognize_leaves(self):
-
-    def custom_is_leaf(x: PyTree) -> bool:
-      return isinstance(x, int) or isinstance(x, list)
-
-    first = {
-        'x': 0,
-        'y': [0],
-        'z': {'p': 0, 'q': [0, 1]},
-        'w': [0, [1, 2], [3], 4],
-    }
-    second = {
-        'x': 1,
-        'y': [1, 2],
-        'z': {'p': 1, 'q': [2]},
-        'w': [5, [6], [7], 8],
-    }
-
-    self.assertEqual(
-        tree_utils.tree_difference(first, second),
-        {
-            'y': Diff(lhs=(list, 1), rhs=(list, 2)),
-            'z': {
-                'q': Diff(lhs=(list, 2), rhs=(list, 1)),
-            },
-            'w': [None, Diff(lhs=(list, 2), rhs=(list, 1)), None, None],
-        },
-    )
-
-    self.assertIsNone(
-        tree_utils.tree_difference(first, second, is_leaf=custom_is_leaf)
-    )
+  def test_select_object_raises_error(self):
+    tree = FlaxRecord(alpha=1, beta=2)
+    with self.assertRaisesRegex(ValueError, 'Path .* does not exist'):
+      tree_utils.select_by_tree_path(tree, ('gamma',))
 
 
 if __name__ == '__main__':
