@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,3 +22,7 @@ from orbax.checkpoint._src.arrays.types import (
     Index,
     Shape,
 )
+
+from orbax.checkpoint._src.arrays.sharding import (
+    construct_maximal_shardings,
+)
