@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,6 +14,7 @@
 
 """High-level checkpoint utils provided for user convenience."""
 
+import asyncio
 import contextlib
 import time
 from typing import Any, Callable, Iterator, Optional
@@ -24,6 +25,7 @@ import jax
 from jax.experimental import layout
 import numpy as np
 from orbax.checkpoint import utils
+from orbax.checkpoint._src.arrays import sharding as arrays_sharding_lib
 from orbax.checkpoint._src.metadata import tree as tree_metadata
 from orbax.checkpoint._src.metadata import value as value_metadata
 from orbax.checkpoint._src.multihost import multihost
@@ -32,10 +34,14 @@ from orbax.checkpoint._src.path.snapshot import snapshot as snapshot_lib
 from orbax.checkpoint._src.serialization import type_handlers
 
 
+
 PyTree = Any
 STANDARD_ARRAY_TYPES = (int, float, np.ndarray, jax.Array)
 _SNAPSHOTS = '_SNAPSHOTS'
-Layout = layout.Layout
+if jax.__version_info__ >= (0, 6, 2):
+  Format = layout.Format
+else:
+  Format = layout.Layout
 PLACEHOLDER = type_handlers.PLACEHOLDER
 
 
@@ -60,38 +66,63 @@ def get_snapshot_dir_from_step_dir(
   return new_path
 
 
+@contextlib.contextmanager
+def _manage_snapshot_file_not_found(
+    ignore_errors: bool, formatted_message: str
+) -> Iterator[None]:
+  """Context manager to optionally suppress FileNotFoundError."""
+  try:
+    yield
+  except FileNotFoundError:
+    if ignore_errors:
+      logging.warning(formatted_message, exc_info=True)
+    else:
+      raise
+
+
 def _snapshot_checkpoint(
     checkpoint_dir: epath.Path,
     step: int,
     step_name_format: step_lib.NameFormat[step_lib.Metadata],
     snapshot_dir: Optional[epath.Path] = None,
+    *,
+    set_immutable: bool | None = None,
+    ignore_file_not_found_error: bool | None = None,
 ):
   """Uses `Snapshot` class to create a cheap "copy" of the checkpoint."""
-  if multihost.process_index() == 0:
-    logging.info('Snpashotting step: %d.', step)
-    step_dir = step_name_format.find_step(checkpoint_dir, step).path
-    if not step_dir.exists():
-      raise ValueError(f'Step directory {step_dir} does not exist.')
+  if multihost.process_index() != 0:
+    return False
 
-    if snapshot_dir is None:
-      snapshot_dir = checkpoint_dir / _SNAPSHOTS
-    if not snapshot_dir.exists():
-      try:
-        snapshot_dir.mkdir(parents=True, exist_ok=True)
-      except OSError as e:
-        raise ValueError(
-            f'Failed to create snapshot directory {snapshot_dir}. Please'
-            ' provide a snapshot directory instead.'
-        ) from e
+  logging.info('Snapshotting step: %d.', step)
+  step_dir = step_name_format.find_step(checkpoint_dir, step).path
+  if not step_dir.exists():
+    raise ValueError(f'Step directory {step_dir} does not exist.')
 
-    snapshot_path = get_snapshot_dir_from_step_dir(step_dir, snapshot_dir)
-    if epath.Path(snapshot_path).exists():
-      return True
-    snapshot_impl = snapshot_lib.create_instance(str(snapshot_path))
-    dst_path = snapshot_impl.create_snapshot(str(step_dir), str(snapshot_path))
-    if str(snapshot_path) == dst_path:
-      return True
-  return False
+  if snapshot_dir is None:
+    snapshot_dir = checkpoint_dir / _SNAPSHOTS
+  if not snapshot_dir.exists():
+    try:
+      snapshot_dir.mkdir(parents=True, exist_ok=True)
+    except OSError as e:
+      raise ValueError(
+          f'Failed to create snapshot directory {snapshot_dir}. Please'
+          ' provide a snapshot directory instead.'
+      ) from e
+
+  snapshot_path = get_snapshot_dir_from_step_dir(step_dir, snapshot_dir)
+  if epath.Path(snapshot_path).exists():
+    return True
+  snapshot_impl = snapshot_lib.create_instance(
+      step_dir, snapshot_path, set_immutable=set_immutable
+  )
+  with _manage_snapshot_file_not_found(
+      ignore_errors=ignore_file_not_found_error,
+      formatted_message=(
+          f'Ignoring error when snapshotting checkpoint for step: {step}'
+      ),
+  ):
+    asyncio.run(snapshot_impl.create_snapshot())
+  return True
 
 
 def _release_snapshot(
@@ -99,6 +130,8 @@ def _release_snapshot(
     step: int,
     step_name_format: step_lib.NameFormat[step_lib.Metadata],
     snapshot_dir: Optional[epath.Path] = None,
+    *,
+    ignore_file_not_found_error: bool | None = None,
 ):
   """Releases snapshot by deleting the snapshot of the checkpoint."""
   if multihost.process_index() == 0:
@@ -106,8 +139,14 @@ def _release_snapshot(
     if snapshot_dir is None:
       snapshot_dir = checkpoint_dir / _SNAPSHOTS
     snapshot_path = snapshot_dir / step_name_format.build_name(step)
-    snapshot_impl = snapshot_lib.create_instance(str(snapshot_path))
-    snapshot_impl.release_snapshot(str(snapshot_path))
+    snapshot_impl = snapshot_lib.create_instance(checkpoint_dir, snapshot_path)
+    with _manage_snapshot_file_not_found(
+        ignore_errors=ignore_file_not_found_error,
+        formatted_message=(
+            f'Ignoring error when releasing snapshot for step: {step}'
+        ),
+    ):
+      asyncio.run(snapshot_impl.release_snapshot())
 
 
 def _reached_desired_step(step: int, until_step: Optional[int]) -> bool:
@@ -129,6 +168,8 @@ def _wait_for_new_checkpoint(
     timeout: Optional[int] = None,
     timeout_fn: Optional[Callable[[], bool]] = None,
     snapshot_dir: Optional[epath.Path] = None,
+    set_immutable: bool | None = None,
+    ignore_snapshot_errors: bool | None = None,
 ) -> int:
   """See documentation for wait_for_new_checkpoint."""
   start = time.time()
@@ -163,7 +204,12 @@ def _wait_for_new_checkpoint(
       checkpoint_step = max(steps) if steps else None
       if _reached_desired_step(checkpoint_step, until_step):
         if not _snapshot_checkpoint(
-            checkpoint_dir, checkpoint_step, step_name_format, snapshot_dir
+            checkpoint_dir,
+            checkpoint_step,
+            step_name_format,
+            snapshot_dir,
+            set_immutable=set_immutable,
+            ignore_file_not_found_error=ignore_snapshot_errors,
         ):
           continue
         result = checkpoint_step
@@ -196,6 +242,8 @@ def wait_for_new_checkpoint(
     step_format_fixed_length: Optional[int] = None,
     step_name_format: Optional[step_lib.NameFormat[step_lib.Metadata]] = None,
     snapshot_dir: Optional[epath.Path] = None,
+    set_immutable: bool | None = None,
+    ignore_snapshot_errors: bool | None = True,
 ):
   """Waits until a new checkpoint file is found.
 
@@ -222,6 +270,10 @@ def wait_for_new_checkpoint(
       ignored.
     snapshot_dir: The directory in which snapshots are saved. If not provided,
       the snapshot directory is created as `checkpoint_dir / _SNAPSHOTS`.
+    set_immutable: If True and applicable, sets the files in the snapshot to be
+      immutable.
+    ignore_snapshot_errors: If True, ignore errors when creating/releasing
+      snapshots.
 
   Yields:
     a new checkpoint step, or -1 if the timeout was reached.
@@ -239,6 +291,8 @@ def wait_for_new_checkpoint(
       timeout=timeout,
       timeout_fn=timeout_fn,
       snapshot_dir=snapshot_dir,
+      set_immutable=set_immutable,
+      ignore_snapshot_errors=ignore_snapshot_errors,
   )
   try:
     yield step
@@ -248,7 +302,13 @@ def wait_for_new_checkpoint(
       logging.info(
           'Releasing snapshot for step: %d after releasing control.', step
       )
-      _release_snapshot(checkpoint_dir, step, step_name_format, snapshot_dir)
+      _release_snapshot(
+          checkpoint_dir,
+          step,
+          step_name_format,
+          snapshot_dir,
+          ignore_file_not_found_error=ignore_snapshot_errors,
+      )
 
 
 def checkpoints_iterator(
@@ -262,6 +322,8 @@ def checkpoints_iterator(
     step_format_fixed_length: Optional[int] = None,
     step_name_format: Optional[step_lib.NameFormat[step_lib.Metadata]] = None,
     snapshot_dir: Optional[epath.Path] = None,
+    set_immutable: bool | None = None,
+    ignore_snapshot_errors: bool | None = True,
 ) -> Iterator[int]:
   """Continuously yield new checkpoint files as they appear.
 
@@ -321,6 +383,10 @@ def checkpoints_iterator(
       ignored.
     snapshot_dir: The directory in which snapshots are saved. If not provided,
       the snapshot directory is created as `checkpoint_dir / _SNAPSHOTS`.
+    set_immutable: If True and applicable, sets the files in the snapshot to be
+      immutable.
+    ignore_snapshot_errors: If True, ignore errors when creating/releasing
+      snapshots.
 
   Yields:
     Integer step numbers of the latest checkpoints as they arrive.
@@ -331,12 +397,19 @@ def checkpoints_iterator(
       step_prefix=step_prefix,
       step_format_fixed_length=step_format_fixed_length,
   )
-  snapshot_impl = snapshot_lib.create_instance(str(checkpoint_dir))
   if snapshot_dir is None:
     snapshot_dir = checkpoint_dir / _SNAPSHOTS
   if snapshot_dir.exists():
     for step_dir in snapshot_dir.iterdir():
-      snapshot_impl.release_snapshot(str(step_dir))
+      snapshot_impl = snapshot_lib.create_instance(checkpoint_dir, step_dir)
+      with _manage_snapshot_file_not_found(
+          ignore_errors=ignore_snapshot_errors,
+          formatted_message=(
+              'Ignoring error when cleaning up leftover snapshot:'
+              f' {step_dir.name}'
+          ),
+      ):
+        asyncio.run(snapshot_impl.release_snapshot())
   checkpoint_step = None
   while True:
     until_step = checkpoint_step + 1 if checkpoint_step is not None else None
@@ -348,6 +421,8 @@ def checkpoints_iterator(
         timeout_fn=timeout_fn,
         step_name_format=step_name_format,
         snapshot_dir=snapshot_dir,
+        set_immutable=set_immutable,
+        ignore_snapshot_errors=ignore_snapshot_errors,
     ) as new_checkpoint_step:
       if new_checkpoint_step == -1:
         if not timeout_fn:
@@ -428,13 +503,19 @@ def construct_restore_args(
 
   def _array_restore_args(
       value: Any,
-      sharding: Optional[jax.sharding.Sharding | Layout],
+      sharding: Optional[jax.sharding.Sharding | Format],  # pytype: disable=unsupported-operands
       dtype: Optional[np.dtype] = None,
   ) -> type_handlers.ArrayRestoreArgs:
+    global_shape = None
+    # For random keys, we only allow overriding the sharding.
+    if set_global_shape and not jax.dtypes.issubdtype(
+        value.dtype, jax.dtypes.prng_key
+    ):
+      global_shape = value.shape
     return type_handlers.ArrayRestoreArgs(
         restore_type=jax.Array,
         sharding=sharding,
-        global_shape=value.shape if set_global_shape else None,
+        global_shape=global_shape,
         dtype=dtype,
     )
 
@@ -480,18 +561,25 @@ def construct_restore_args(
       raise ValueError(f'Unsupported type: {type(value)}')
 
   def _get_sharding_or_layout(value):
-    if hasattr(value, 'sharding'):
-      if (
-          support_layout
-          and hasattr(value, 'layout')
-          and value.layout.device_local_layout
-      ):
-        # value is a jax.Array or a jax.ShapeDtypeStruct.
-        return value.layout
-      else:
-        return value.sharding
+    if support_layout:
+      return arrays_sharding_lib.get_sharding_or_format(value)
     else:
-      return None
+      if hasattr(value, 'sharding'):
+        return value.sharding
+      else:
+        return None
+
+  def _return_key_data(value):
+    # replace jax.random.key with underneath jax.Array
+    if isinstance(value, jax.Array) and jax.dtypes.issubdtype(
+        value.dtype, jax.dtypes.prng_key
+    ):
+      # For random keys, extract the dtype and shape as a regular Jax array.
+      # Stored metadata will help restoring the original random key.
+      return jax.random.key_data(value)
+    return value
+
+  target = jax.tree.map(_return_key_data, target)
 
   if sharding_tree is None:
     sharding_tree = jax.tree.map(_get_sharding_or_layout, target)
