@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,12 +14,12 @@
 
 import jax
 from orbax.checkpoint._src.testing import multiprocess_test
-from orbax.checkpoint.experimental.emergency import test_utils
+from orbax.checkpoint.experimental.emergency.test_utils import test_base
 
 
-@test_utils.barrier_compatible_test
+@test_base.barrier_compatible_test
 class LocalCheckpointManagerTest(
-    test_utils.LocalCheckpointManagerTestBase.Test,
+    test_base.LocalCheckpointManagerTestBase.Test,
     multiprocess_test.MultiProcessTest,
 ):
 
@@ -30,7 +30,7 @@ class LocalCheckpointManagerTest(
 
     # setup global mesh info for 2-slice tests
     slice_processes = [{0, 1}, {2, 3}]
-    return test_utils.get_fake_global_mesh_for_slices(slice_processes)
+    return test_base.get_fake_global_mesh_for_slices(slice_processes)
 
 
 if __name__ == '__main__':
