@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,15 +25,17 @@ import jax
 import numpy as np
 from orbax.checkpoint import args as args_lib
 from orbax.checkpoint import test_utils
+from orbax.checkpoint._src.checkpoint_managers import preservation_policy as preservation_policy_lib
 from orbax.checkpoint._src.handlers import pytree_checkpoint_handler
 from orbax.checkpoint._src.multihost import multihost
 from orbax.checkpoint._src.multihost import multislice
+from orbax.checkpoint._src.path import gcs_utils
 from orbax.checkpoint._src.serialization import type_handlers
 from orbax.checkpoint.experimental.emergency import mesh_consistency
 from orbax.checkpoint.experimental.emergency import replicator_checkpoint_manager
-from orbax.checkpoint.experimental.emergency import test_utils as emergency_test_utils
+from orbax.checkpoint.experimental.emergency.test_utils import dataset_iterator_checkpoint_handler
+from orbax.checkpoint.experimental.emergency.test_utils import test_base as emergency_test_utils
 from orbax.checkpoint.path import atomicity
-from orbax.checkpoint.path import step as step_lib
 from .learning.brain.research.jax.tests.multiprocess import multiprocess_test
 
 
@@ -48,10 +50,37 @@ ReplicatorCheckpointManager = (
 ReplicatorCheckpointManagerOptions = (
     replicator_checkpoint_manager.ReplicatorCheckpointManagerOptions
 )
+DatasetIteratorCheckpointSave = (
+    dataset_iterator_checkpoint_handler.DatasetIteratorCheckpointSave
+)
+DatasetIteratorCheckpointRestore = (
+    dataset_iterator_checkpoint_handler.DatasetIteratorCheckpointRestore
+)
+DatasetIteratorCheckpointHandler = (
+    dataset_iterator_checkpoint_handler.DatasetIteratorCheckpointHandler
+)
 barrier_compatible_test = test_utils.barrier_compatible_test
 assert_tree_equal = test_utils.assert_tree_equal
 get_fake_global_mesh_for_slices = test_utils.get_fake_global_mesh_for_slices
 swap_slices_in_mesh = emergency_test_utils.swap_slices_in_mesh
+STATE = 'state'
+
+
+def get_composite_save_args(
+    pytree: PyTree,
+    checkpoint_args: args_lib.CheckpointArgs = PyTreeSaveArgs,
+) -> args_lib.Composite:
+  return args_lib.Composite(state=checkpoint_args(pytree))
+
+
+def get_composite_restore_args(
+    pytree: PyTree | None = None,
+    restore_args: PyTree | None = None,
+    checkpoint_args: args_lib.CheckpointArgs = PyTreeRestoreArgs,
+) -> args_lib.Composite:
+  if pytree is not None:
+    return args_lib.Composite(state=checkpoint_args(pytree))
+  return args_lib.Composite(state=checkpoint_args(restore_args=restore_args))
 
 
 @barrier_compatible_test
@@ -107,7 +136,7 @@ class ReplicatorCheckpointManagerTest(
     )
     self.enter_context(
         mock.patch.object(
-            step_lib, 'is_gcs_path', autospec=True, return_value=True
+            gcs_utils, 'is_gcs_path', autospec=True, return_value=True
         )
     )
     if not multihost.is_runtime_to_distributed_ids_initialized():
@@ -131,6 +160,16 @@ class ReplicatorCheckpointManagerTest(
             name=f'checkpointing_test_pid{multihost.process_index()}'
         ).full_path
     )
+    self.non_replicated_directory = (
+        epath.Path(self._get_tempdir_path_test())
+        / 'non_replicated_checkpointing_test'
+    )
+    if multihost.is_primary_host(primary_host=0):
+      self.non_replicated_directory = epath.Path(
+          self.create_tempdir(
+              name='non_replicated_checkpointing_test'
+          ).full_path
+      )
     logging.info(
         'self.directory=%s',
         self.local_directory,
@@ -187,7 +226,10 @@ class ReplicatorCheckpointManagerTest(
     )
 
     for i in range(17):
-      manager.save(i, args=PyTreeSaveArgs(pytree))
+      manager.save(
+          i,
+          args=get_composite_save_args(pytree),
+      )
     manager.wait_until_finished()
 
     expected_steps = [0, 3, 6, 9, 12, 15]
@@ -209,17 +251,93 @@ class ReplicatorCheckpointManagerTest(
         global_mesh=global_mesh,
     )
 
-    manager.save(0, args=PyTreeSaveArgs(pytree))
+    manager.save(
+        0,
+        args=get_composite_save_args(pytree),
+    )
     manager.wait_until_finished()
     self.assert_process_metadata_files_exist(0, global_mesh)
 
     restored = manager.restore(
-        0, args=PyTreeRestoreArgs(restore_args=restore_args)
+        0, args=get_composite_restore_args(restore_args=restore_args)
     )
 
-    test_utils.assert_tree_equal(
-        self, pytree, restored
+    test_utils.assert_tree_equal(self, pytree, restored.state)
+
+  def test_preservation_policy(self):
+    global_mesh = self.make_global_mesh(replica_axis_index=0)
+    pytree, _ = self.setup_pytree(global_mesh)
+    options = ReplicatorCheckpointManagerOptions(
+        save_interval_steps=1,
+        preservation_policy=preservation_policy_lib.EveryNSteps(2),
     )
+    manager = ReplicatorCheckpointManager(
+        self.local_directory,
+        options=options,
+        global_mesh=global_mesh,
+    )
+
+    for i in range(3):
+      manager.save(
+          i,
+          args=get_composite_save_args(pytree),
+      )
+      manager.wait_until_finished()
+    self.assertEqual(manager.all_steps(), [0, 2])
+
+  @parameterized.parameters((0,), (1,))
+  def test_save_restore_dataset_iterator(self, replica_axis_index: int):
+    global_mesh = self.make_global_mesh(replica_axis_index=replica_axis_index)
+    pytree, restore_args = self.setup_pytree(global_mesh)
+    options = ReplicatorCheckpointManagerOptions(
+        save_interval_steps=1,
+        preservation_policy=preservation_policy_lib.LatestN(1),
+    )
+    manager = ReplicatorCheckpointManager(
+        self.local_directory,
+        persistent_directory=self.non_replicated_directory,
+        options=options,
+        global_mesh=global_mesh,
+    )
+
+    dummy_dataset = [
+        ('hello', 'hola'),
+        ('world', 'mundo'),
+        ('test', 'prueba'),
+    ]
+    dummy_iterator = DatasetIteratorCheckpointHandler.DummyIterator(
+        dummy_dataset
+    )
+
+    manager.save(
+        0,
+        args=args_lib.Composite(
+            state=PyTreeSaveArgs(pytree),
+            dataset=DatasetIteratorCheckpointSave(dummy_iterator),
+        ),
+    )
+    manager.wait_until_finished()
+    self.assert_process_metadata_files_exist(0, global_mesh)
+
+    restored = manager.restore(
+        0,
+        args=args_lib.Composite(
+            state=PyTreeRestoreArgs(restore_args=restore_args),
+            dataset=DatasetIteratorCheckpointRestore(dummy_iterator),
+        ),
+    )
+    test_utils.assert_tree_equal(self, pytree, restored.state)
+    test_utils.assert_tree_equal(self, dummy_iterator, restored.dataset)
+
+    manager.save(
+        1,
+        args=args_lib.Composite(
+            state=PyTreeSaveArgs(pytree),
+            dataset=DatasetIteratorCheckpointSave(dummy_iterator),
+        ),
+    )
+    manager.wait_until_finished()
+    self.assertEqual(manager.all_steps(), [1])
 
   def test_no_cleanup(self):
     options = ReplicatorCheckpointManagerOptions(
@@ -231,22 +349,30 @@ class ReplicatorCheckpointManagerTest(
         global_mesh=self.global_mesh,
     )
 
-    manager.save(0, args=PyTreeSaveArgs(self.pytree))
+    manager.save(
+        0,
+        args=get_composite_save_args(self.pytree),
+    )
     manager.wait_until_finished()
     self.assert_process_metadata_files_exist(0, self.global_mesh)
 
-    manager.save(1, args=PyTreeSaveArgs(self.doubled_pytree))
+    manager.save(
+        1,
+        args=get_composite_save_args(self.doubled_pytree),
+    )
     manager.wait_until_finished()
     self.assert_process_metadata_files_exist(1, self.global_mesh)
 
     restored = manager.restore(
-        0, args=PyTreeRestoreArgs(restore_args=self.restore_args)
+        0,
+        args=get_composite_restore_args(restore_args=self.restore_args),
     )
-    test_utils.assert_tree_equal(self, self.pytree, restored)
+    test_utils.assert_tree_equal(self, self.pytree, restored.state)
     restored = manager.restore(
-        1, args=PyTreeRestoreArgs(restore_args=self.restore_args)
+        1,
+        args=get_composite_restore_args(restore_args=self.restore_args),
     )
-    test_utils.assert_tree_equal(self, self.doubled_pytree, restored)
+    test_utils.assert_tree_equal(self, self.doubled_pytree, restored.state)
 
   def test_startup_cleanup(self):
     options = ReplicatorCheckpointManagerOptions(
@@ -258,9 +384,18 @@ class ReplicatorCheckpointManagerTest(
         global_mesh=self.global_mesh,
     )
 
-    manager.save(0, args=PyTreeSaveArgs(self.pytree))
-    manager.save(1, args=PyTreeSaveArgs(self.pytree))
-    manager.save(2, args=PyTreeSaveArgs(self.doubled_pytree))
+    manager.save(
+        0,
+        args=get_composite_save_args(self.pytree),
+    )
+    manager.save(
+        1,
+        args=get_composite_save_args(self.pytree),
+    )
+    manager.save(
+        2,
+        args=get_composite_save_args(self.doubled_pytree),
+    )
     manager.wait_until_finished()
 
     # 3 steps should exist
@@ -290,7 +425,10 @@ class ReplicatorCheckpointManagerTest(
     self.assertTrue((self.local_directory / '2').exists())
 
     # new step save should work
-    manager2.save(3, args=PyTreeSaveArgs(self.doubled_pytree))
+    manager2.save(
+        3,
+        args=get_composite_save_args(self.doubled_pytree),
+    )
     manager2.wait_until_finished()
     self.assertEqual(manager2.all_steps(), [1, 2, 3])
     self.assertTrue((self.local_directory / '2').exists())
@@ -324,7 +462,10 @@ class ReplicatorCheckpointManagerTest(
     )
 
     for i in range(step):
-      manager.save(i, args=PyTreeSaveArgs(self.pytree))
+      manager.save(
+          i,
+          args=get_composite_save_args(self.pytree),
+      )
       manager.wait_until_finished()
 
     self.assertEqual(manager.should_save(step), expectation)
@@ -352,7 +493,10 @@ class ReplicatorCheckpointManagerTest(
     )
 
     for i in range(total_steps):
-      manager.save(i, args=PyTreeSaveArgs(self.pytree))
+      manager.save(
+          i,
+          args=get_composite_save_args(self.pytree),
+      )
       manager.wait_until_finished()
 
     self.assertEqual(sorted(manager.all_steps()), expectation)
@@ -384,39 +528,47 @@ class ReplicatorCheckpointManagerTest(
         options=options,
         global_mesh=global_mesh,
     ) as manager:
-      manager.save(0, args=PyTreeSaveArgs(pytree))
-      manager.save(1, args=PyTreeSaveArgs(pytree))
-      manager.save(2, args=PyTreeSaveArgs(pytree_double))
+      manager.save(
+          0,
+          args=get_composite_save_args(pytree),
+      )
+      manager.save(
+          1,
+          args=get_composite_save_args(pytree),
+      )
+      manager.save(
+          2,
+          args=get_composite_save_args(self.doubled_pytree),
+      )
       test_utils.assert_tree_equal(
           self,
           pytree_double,
           manager.restore(
-              None,
-              args=PyTreeRestoreArgs(restore_args=restore_args),
-          ),
+              None, args=get_composite_restore_args(restore_args=restore_args)
+          ).state,
       )
 
     self.assertEqual(jax.process_count(), 4)
     self.assertEqual(
-        multislice.process_slice_id(
+        multislice.process_replica_id(
             0, global_mesh, replica_axis_index=replica_axis_index
         ),
         0,
     )
     self.assertEqual(
-        multislice.process_slice_id(
+        multislice.process_replica_id(
             1, global_mesh, replica_axis_index=replica_axis_index
         ),
         0,
     )
     self.assertEqual(
-        multislice.process_slice_id(
+        multislice.process_replica_id(
             2, global_mesh, replica_axis_index=replica_axis_index
         ),
         1,
     )
     self.assertEqual(
-        multislice.process_slice_id(
+        multislice.process_replica_id(
             3, global_mesh, replica_axis_index=replica_axis_index
         ),
         1,
@@ -425,25 +577,25 @@ class ReplicatorCheckpointManagerTest(
         global_mesh, replica_axis_index=replica_axis_index
     )
     self.assertEqual(
-        multislice.process_slice_id(
+        multislice.process_replica_id(
             0, new_global_mesh, replica_axis_index=replica_axis_index
         ),
         1,
     )
     self.assertEqual(
-        multislice.process_slice_id(
+        multislice.process_replica_id(
             1, new_global_mesh, replica_axis_index=replica_axis_index
         ),
         1,
     )
     self.assertEqual(
-        multislice.process_slice_id(
+        multislice.process_replica_id(
             2, new_global_mesh, replica_axis_index=replica_axis_index
         ),
         0,
     )
     self.assertEqual(
-        multislice.process_slice_id(
+        multislice.process_replica_id(
             3, new_global_mesh, replica_axis_index=replica_axis_index
         ),
         0,
@@ -457,9 +609,9 @@ class ReplicatorCheckpointManagerTest(
       self.assertEqual(manager.latest_step(), 2)
       restored = manager.restore(
           None,
-          args=PyTreeRestoreArgs(restore_args=restore_args),
+          args=get_composite_restore_args(restore_args=restore_args),
       )
-      test_utils.assert_tree_equal(self, pytree_double, restored)
+      test_utils.assert_tree_equal(self, pytree_double, restored.state)
 
   @parameterized.parameters(
       (0,),
@@ -477,9 +629,18 @@ class ReplicatorCheckpointManagerTest(
         options=options,
         global_mesh=global_mesh,
     ) as manager:
-      manager.save(0, args=PyTreeSaveArgs(pytree))
-      manager.save(1, args=PyTreeSaveArgs(pytree))
-      manager.save(2, args=PyTreeSaveArgs(pytree))
+      manager.save(
+          0,
+          args=get_composite_save_args(pytree),
+      )
+      manager.save(
+          1,
+          args=get_composite_save_args(pytree),
+      )
+      manager.save(
+          2,
+          args=get_composite_save_args(pytree),
+      )
       manager.wait_until_finished()
       self.assertSameElements([0, 1, 2], manager.all_steps())
       self.assertEqual(2, manager.latest_step())
@@ -488,8 +649,8 @@ class ReplicatorCheckpointManagerTest(
           pytree,
           manager.restore(
               manager.latest_step(),
-              args=PyTreeRestoreArgs(restore_args=restore_args),
-          ),
+              args=get_composite_restore_args(restore_args=restore_args),
+          ).state,
       )
 
     new_global_mesh = swap_slices_in_mesh(
@@ -506,9 +667,8 @@ class ReplicatorCheckpointManagerTest(
           self,
           pytree,
           manager.restore(
-              None,
-              args=PyTreeRestoreArgs(restore_args=restore_args),
-          ),
+              None, args=get_composite_restore_args(restore_args=restore_args)
+          ).state,
       )
 
   def test_process_index_metadata(self):
@@ -517,11 +677,14 @@ class ReplicatorCheckpointManagerTest(
         global_mesh=self.global_mesh,
     ) as mngr:
       metadata_folder = mesh_consistency.process_metadata_folder(
-          self.local_directory/str(0)/'process_metadata'
+          self.local_directory / str(0) / 'process_metadata'
       )
       self.assertFalse(metadata_folder.exists())
 
-      mngr.save(0, args=PyTreeSaveArgs(self.pytree))
+      mngr.save(
+          0,
+          args=get_composite_save_args(self.pytree),
+      )
       mngr.wait_until_finished()
 
       self.assert_process_metadata_files_exist(0, self.global_mesh)
@@ -530,9 +693,7 @@ class ReplicatorCheckpointManagerTest(
       with self.assertRaisesRegex(
           ValueError, 'Process metadata folder was not finalized'
       ):
-        mesh_consistency.read_process_metadata(
-            metadata_folder
-        )
+        mesh_consistency.read_process_metadata(metadata_folder)
 
   def test_should_save_fn(self):
     expectation = [1, 5, 7, 8]
@@ -549,7 +710,10 @@ class ReplicatorCheckpointManagerTest(
         global_mesh=self.global_mesh,
     ) as manager:
       for i in range(10):
-        manager.save(i, args=PyTreeSaveArgs(self.pytree))
+        manager.save(
+            i,
+            args=get_composite_save_args(self.pytree),
+        )
         manager.wait_until_finished()
 
       self.assertSameElements(manager.all_steps(), expectation)
@@ -594,10 +758,14 @@ class ReplicatorCheckpointManagerTest(
       with self.assertRaises(TypeError):
         manager.save(0)
       with self.assertRaises(ValueError):
-        manager.save(0, args=args_lib.StandardSave(self.pytree))
+        manager.save(
+            0,
+            args=get_composite_save_args(self.pytree, args_lib.StandardRestore),
+        )
       with self.assertRaises(ValueError):
         manager.save(
-            0, args=args_lib.Composite(state=args_lib.StandardSave(self.pytree))
+            0,
+            args=get_composite_save_args(self.pytree, args_lib.StandardRestore),
         )
 
   def test_invalid_args_restore(self):
@@ -605,35 +773,60 @@ class ReplicatorCheckpointManagerTest(
         self.local_directory,
         global_mesh=self.global_mesh,
     ) as manager:
-      manager.save(0, args=args_lib.PyTreeSave(self.pytree))
-      restored = manager.restore(
-          0, args=args_lib.PyTreeRestore(restore_args=self.restore_args)
+      manager.save(
+          0,
+          args=get_composite_save_args(self.pytree, args_lib.PyTreeSave),
       )
-      test_utils.assert_tree_equal(self, self.pytree, restored)
+      restored = manager.restore(
+          0,
+          args=get_composite_restore_args(
+              restore_args=self.restore_args,
+              checkpoint_args=args_lib.PyTreeRestore,
+          ),
+      )
+      test_utils.assert_tree_equal(self, self.pytree, restored.state)
 
-      with self.assertRaises(ValueError):
-        manager.restore(0, args=args_lib.PyTreeRestore(self.pytree))
-      with self.assertRaises(ValueError):
-        manager.restore(0, args=args_lib.StandardRestore(self.pytree))
       with self.assertRaises(ValueError):
         manager.restore(
             0,
-            args=args_lib.Composite(
-                state=args_lib.StandardRestore(self.pytree)
+            args=get_composite_restore_args(
+                self.pytree, args_lib.StandardRestore
             ),
         )
       with self.assertRaises(ValueError):
+        manager.restore(
+            0,
+            args=get_composite_restore_args(
+                self.pytree, args_lib.StandardRestore
+            ),
+        )
+      with self.assertRaises(ValueError):
+        manager.restore(
+            0,
+            args=get_composite_restore_args(
+                self.pytree, args_lib.StandardRestore
+            ),
+        )
+      with self.assertRaises(TypeError):
         manager.restore(0)
       with self.assertRaises(FileNotFoundError):
-        manager.restore(1, args=args_lib.PyTreeRestore(self.pytree))
-      with self.assertRaises(ValueError):
+        manager.restore(
+            1,
+            args=get_composite_restore_args(
+                self.pytree, args_lib.PyTreeRestore
+            ),
+        )
+      with self.assertRaises(TypeError):
         manager.restore(1)
 
     with ReplicatorCheckpointManager(
         self.local_directory,
         global_mesh=self.global_mesh,
     ) as manager:
-      manager.save(0, args=args_lib.PyTreeSave(self.pytree))
+      manager.save(
+          0,
+          args=get_composite_save_args(self.pytree, args_lib.PyTreeSave),
+      )
 
   @parameterized.parameters(
       ((8,), ('data',)),
@@ -657,13 +850,15 @@ class ReplicatorCheckpointManagerTest(
         global_mesh=global_mesh,
     )
 
-    manager.save(0, args=PyTreeSaveArgs(pytree))
+    manager.save(
+        0, args=get_composite_save_args(self.pytree, args_lib.PyTreeSave)
+    )
     manager.wait_until_finished()
 
     restored = manager.restore(
-        0, args=PyTreeRestoreArgs(restore_args=restore_args)
+        0, args=get_composite_restore_args(restore_args=restore_args)
     )
-    test_utils.assert_tree_equal(self, pytree, restored)
+    test_utils.assert_tree_equal(self, pytree, restored.state)
 
 
 if __name__ == '__main__':
