@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,8 +19,10 @@ from __future__ import annotations
 from collections.abc import Iterable
 import contextvars
 
+from absl import logging
 from etils import epy
 from orbax.checkpoint.experimental.v1._src.context import options as options_lib
+from orbax.checkpoint.experimental.v1._src.synchronization import multihost
 from orbax.checkpoint.experimental.v1._src.synchronization import synchronization
 
 
@@ -45,9 +47,9 @@ class Context(epy.ContextManager):
     with ocp.Context(...):
       ocp.save_pytree(...)
 
-  Creating a new `Context` within an existing `Context` sets all parameters from
-  scratch; it does not inherit properties from the parent `Context`. To achieve
-  this, use::
+  Creating a new :py:class:`.Context` within an existing :py:class:`.Context`
+  sets all parameters from scratch; it does not inherit properties from the
+  parent :py:class:`.Context`. To achieve this, use::
 
     with Context(**some_properties) as outer_ctx:
       with Context(outer_ctx, **other) as inner_ctx:
@@ -57,7 +59,7 @@ class Context(epy.ContextManager):
   properties modified in the `dataclasses.replace` call.
 
   NOTE: The context is not shared across threads. In other words, the whole
-  context block must be executed in the same thread. Following example will
+  context block must be executed in the same thread. The following example will
   not work as expected::
 
     executor = ThreadPoolExecutor()
@@ -73,6 +75,8 @@ class Context(epy.ContextManager):
     multiprocessing_options: Options for multiprocessing behavior.
     file_options: Options for working with the file system.
     checkpointables_options: Options for controlling checkpointables behavior.
+    pathways_options: Options for Pathways checkpointing.
+    checkpoint_layout: The layout of the checkpoint. Defaults to ORBAX.
   """
 
   def __init__(
@@ -85,6 +89,8 @@ class Context(epy.ContextManager):
       multiprocessing_options: options_lib.MultiprocessingOptions | None = None,
       file_options: options_lib.FileOptions | None = None,
       checkpointables_options: options_lib.CheckpointablesOptions | None = None,
+      pathways_options: options_lib.PathwaysOptions | None = None,
+      checkpoint_layout: options_lib.CheckpointLayout | None = None,
   ):
     self._pytree_options = pytree_options or (
         context.pytree_options if context else options_lib.PyTreeOptions()
@@ -108,6 +114,14 @@ class Context(epy.ContextManager):
         if context
         else options_lib.CheckpointablesOptions()
     )
+    self._pathways_options = pathways_options or (
+        context.pathways_options if context else options_lib.PathwaysOptions()
+    )
+    self._checkpoint_layout = checkpoint_layout or (
+        context.checkpoint_layout
+        if context
+        else options_lib.CheckpointLayout.ORBAX
+    )
 
   @property
   def pytree_options(self) -> options_lib.PyTreeOptions:
@@ -133,6 +147,14 @@ class Context(epy.ContextManager):
   def checkpointables_options(self) -> options_lib.CheckpointablesOptions:
     return self._checkpointables_options
 
+  @property
+  def pathways_options(self) -> options_lib.PathwaysOptions:
+    return self._pathways_options
+
+  @property
+  def checkpoint_layout(self) -> options_lib.CheckpointLayout:
+    return self._checkpoint_layout
+
   def operation_id(self) -> str:
     return synchronization.OperationIdGenerator.get_current_operation_id()
 
@@ -142,3 +164,23 @@ class Context(epy.ContextManager):
       yield self
     finally:
       _CONTEXT.reset(token)
+
+
+async def synchronize_next_operation_id():
+  """Obtains the next operation id and synchronizes processes."""
+  context = get_context()
+  operation_id = synchronization.OperationIdGenerator.next_operation_id()
+  await multihost.sync_global_processes(
+      multihost.unique_barrier_key(
+          'next_awaitable_signal_operation_id:sync',
+          prefix=context.multiprocessing_options.barrier_sync_key_prefix,
+      ),
+      operation_id=context.operation_id(),
+      processes=context.multiprocessing_options.active_processes,
+  )
+  logging.vlog(
+      1,
+      '[process=%s] Synchronized next awaitable signal operation id to %s',
+      multihost.process_index(),
+      operation_id,
+  )
