@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,14 +17,20 @@
 from __future__ import annotations
 
 import dataclasses
+import enum
 from typing import Any, Callable, Protocol, Type
 
+from etils import epath
 import numpy as np
 from orbax.checkpoint import options as v0_options_lib
+from orbax.checkpoint._src.metadata import array_metadata_store as array_metadata_store_lib
 from orbax.checkpoint._src.metadata import tree as tree_metadata
 from orbax.checkpoint._src.path import atomicity_types
+from orbax.checkpoint._src.serialization import pathways_types
 from orbax.checkpoint.experimental.v1._src.handlers import registration
 from orbax.checkpoint.experimental.v1._src.handlers import types as handler_types
+from orbax.checkpoint.experimental.v1._src.path import types as path_types
+from orbax.checkpoint.experimental.v1._src.serialization import types as serialization_types
 from orbax.checkpoint.experimental.v1._src.tree import types as tree_types
 
 
@@ -33,11 +39,12 @@ from orbax.checkpoint.experimental.v1._src.tree import types as tree_types
 class AsyncOptions:
   """Options used to configure async behavior.
 
-  timeout_secs: The timeout in seconds for the async save operation.
-  post_finalization_callback: A function that is called after the async save
-    operation is complete.
-  create_directories_asynchronously: If true, create directories asynchronously
-    in the background.
+  timeout_secs:
+    The timeout in seconds for the async save operation.
+  post_finalization_callback:
+    A function that is called after the async save operation is complete.
+  create_directories_asynchronously:
+    If True, creates directories asynchronously in the background.
   """
 
   timeout_secs: int = 600  # 10 minutes.
@@ -56,19 +63,21 @@ class AsyncOptions:
 class MultiprocessingOptions:
   """Options used to configure multiprocessing behavior.
 
-  primary_host: the host id of the primary host.  Default to 0.  If it's set
-    to None, then all hosts will be considered as primary.  It's useful in
-    the case that all hosts are only working with local storage.
-  active_processes: A set of process indices (corresponding to
-    `multihost.process_index()`) over which `CheckpointManager` is expected to
-    be called. This makes it possible to have a `CheckpointManager` instance
-    that runs over a subset of processes, rather than all processes as it is
-    normally expected to do. If specified, `primary_host` must belong to
+  primary_host:
+    The host id of the primary host.  Default to 0.  If it's set to None, then
+    all hosts will be considered as primary.  It's useful in the case that all
+    hosts are only working with local storage.
+  active_processes:
+    A set of process indices (corresponding to :py:func:`.process_index`) over
+    which :py:class:`~.v1.training.Checkpointer` is expected to be called.
+    This makes it possible to have a :py:class:`~.v1.training.Checkpointer`
+    instance that runs over a subset of processes, rather than all processes as
+    it is normally expected to do. If specified, `primary_host` must belong to
     `active_processes`.
-  barrier_sync_key_prefix: A string to be prepended to the barrier sync key
-    used to synchronize processes. This is useful to avoid collisions with
-    other barrier syncs if another CheckpointManager is being used
-    concurrently.
+  barrier_sync_key_prefix:
+    A string to be prepended to the barrier sync key used to synchronize
+    processes. This is useful to avoid collisions with other barrier syncs if
+    another :py:class:`~.v1.training.Checkpointer` is being used concurrently.
   """
 
   primary_host: int | None = 0
@@ -89,19 +98,26 @@ class FileOptions:
   """Options used to configure checkpoint directories and files.
 
   Attributes:
-    path_permission_mode: Path permission mode for step directories, user
-      metadata files. e.g. 0o750. Please check
+    path_permission_mode:
+      Path permission mode for step directories, user metadata files. e.g.
+      0o750. Please check
       https://github.com/google/etils/blob/main/etils/epath/backend.py if your
-        path is supported. default=None.
-    temporary_path_class: A class that is used to create and finallize temporary
-      paths, and to ensure atomicity.
+      path is supported. default=None.
+    temporary_path_class:
+      A class that is used to create and finalize temporary paths, and to ensure
+      atomicity.
+    path_class:
+      The implementation of :py:class:`~.v1.path.Path` to use.  Defaults to 
+      `etils.epath.Path`, but may be overridden to some other subclass of 
+      :py:class:`~.v1.path.Path`.
   """
 
   path_permission_mode: int | None = None
-  temporary_path_class: atomicity_types.TemporaryPath | None = None
-
+  temporary_path_class: type[atomicity_types.TemporaryPath] | None = None
+  path_class: type[path_types.Path] = epath.Path
 
   def v0(self) -> v0_options_lib.FileOptions:
+    """Converts this :py:class:`~.v1.options.FileOptions` to a v0 :py:class:`~orbax.checkpoint.options.FileOptions`."""
     return v0_options_lib.FileOptions(
         path_permission_mode=self.path_permission_mode,
     )
@@ -117,20 +133,23 @@ class PyTreeOptions:
   Attributes:
     saving: Options for saving PyTrees.
     loading: Options for loading PyTrees.
+    leaf_handler_registry: Optional Leaf Handler Registry. If provided, it will
+      override the default Leaf Handler Registry.
   """
 
   @dataclasses.dataclass(frozen=True, kw_only=True)
   class Saving:
     """Options for saving PyTrees.
 
-    create_array_storage_options_fn: A function that is called in order to
-      create `ArrayOptions.Saving.StorageOptions` for each leaf in a PyTree,
-      when it is being saved. It is called similar to:
+    create_array_storage_options_fn:
+      A function that is called in order to create
+      :py:class:`.ArrayOptions.Saving.StorageOptions` for each leaf in a PyTree,
+      when it is
+      being saved. It is called similar to:
       `jax.tree.map_with_path(create_array_storage_options_fn, pytree_to_save)`.
       If provided, it overrides any default settings in
-      `ArrayOptions.Saving.StorageOptions`.
+      :py:class:`.ArrayOptions.Saving.StorageOptions`.
     pytree_metadata_options: Options for managing PyTree metadata.
-    partial_update: NOT IMPLEMENTED.
     """
 
     class CreateArrayStorageOptionsFn(Protocol):
@@ -144,20 +163,20 @@ class PyTreeOptions:
     pytree_metadata_options: tree_metadata.PyTreeMetadataOptions = (
         dataclasses.field(default_factory=tree_metadata.PyTreeMetadataOptions)
     )
-    partial_update: bool = False
 
   @dataclasses.dataclass(frozen=True, kw_only=True)
   class Loading:
     """Options for loading PyTrees.
 
-    partial_load: NOT IMPLEMENTED.
+    partial_load: If True, only restore the parameters that are specified
+      in the abstract PyTree.
     """
 
     partial_load: bool = False
 
   saving: Saving = dataclasses.field(default_factory=Saving)
   loading: Loading = dataclasses.field(default_factory=Loading)
-  # TODO(dnlng): Add `LeafHandlerRegistry`.
+  leaf_handler_registry: serialization_types.LeafHandlerRegistry | None = None
 
 
 @dataclasses.dataclass(frozen=True, kw_only=True)
@@ -181,17 +200,31 @@ class ArrayOptions:
         individual leaves. See below.
       use_ocdbt: Enables OCDBT format.
       use_zarr3: If True, use Zarr3 format.
+      use_compression: If True, use ZSTD compression.
       ocdbt_target_data_file_size: Specifies the target size (in bytes) of each
-        OCDBT data file.  It only applies when OCDBT is enabled and Zarr3 must
-        be turned on.  If left unspecified, default size is 2GB.  A value of 0
+        OCDBT data file. It only applies when OCDBT is enabled and Zarr3 must be
+        turned on. If left unspecified, default size is 2GB.  A value of 0
         indicates no maximum file size limit.  For best results, ensure
         chunk_byte_size is smaller than this value.  For more details, refer to
         https://google.github.io/tensorstore/kvstore/ocdbt/index.html#json-kvstore/ocdbt.target_data_file_size
-      enable_pinned_host_transfer: If False, disables transfer to pinned host
-        when copying from device to host, regardless of the presence of pinned
-        host memory.
+      enable_pinned_host_transfer: Whether to use pinned_host memory for the
+        transfer from device to host memory. Passing None will enable
+        pinned_host memory depending on the platform used (currently only
+        enables it for the GPU backend).
       enable_post_merge_validation: If True, enables validation of the
         parameters after the finalize step.
+      use_replica_parallel: Whether to parallelize saving across replicas.
+      min_slice_bytes_for_replica_parallel: Minimum number of bytes per replica
+        slice. Only uses replica-parallel when the amount of data written per
+        replica is greater than or equal to this number.
+      max_replicas_for_replica_parallel: Maximum number of replicas over which
+        saving will be parallelized if use_replica_parallel is True.
+      enable_replica_parallel_separate_folder: Whether to save replica data in
+        separate folders.
+      enable_write_sharding_file: whether to write sharding file, defaults to
+        True.
+      array_metadata_store: Store to manage per host ArrayMetadata. To disable
+        ArrayMetadata persistence, set it to None.
     """
 
     @dataclasses.dataclass(frozen=True, kw_only=True)
@@ -200,21 +233,19 @@ class ArrayOptions:
 
       dtype:
         If provided, casts the parameter to the given dtype before saving.
-        Note that the parameter must be compatible with the given type (e.g.
-        jnp.bfloat16 is not compatible with np.ndarray).
+        Note that the parameter must be compatible with the given type (e.g.,
+        `jnp.bfloat16` is not compatible with `np.ndarray`).
       chunk_byte_size:
         This is an experimental feature that automatically chooses the largest
-        chunk
-        shape possible, while keeping the chunk byte size less than or equal
-        to the
-        specified chunk_byte_size. Both the write_chunk_shape and
-        read_chunk_shape
-        are automatically set to the chosen shape. This uses a greedy
-        algorithm that
-        prioritizes splitting the largest dimensions first.
-      shard_axes: An optional list of axes that should be prioritized when
-        sharding array for storage. If empty, storage sharding implementation
-        will prioritize axes which are already sharded.
+        possible chunk shape while keeping the chunk byte size less than or
+        equal to the specified `chunk_byte_size`. Both `write_chunk_shape` and
+        `read_chunk_shape` are automatically set to the chosen shape. This uses
+        a greedy algorithm that prioritizes splitting the largest dimensions
+        first.
+      shard_axes:
+        An optional list of axes that should be prioritized when sharding an
+        array for storage. If empty, the storage sharding implementation will
+        prioritize axes which are already sharded.
       """
 
       dtype: np.typing.DTypeLike | None = None
@@ -227,23 +258,35 @@ class ArrayOptions:
     )
     use_ocdbt: bool = True
     use_zarr3: bool = True
+    use_compression: bool = True
     ocdbt_target_data_file_size: int | None = None
-    enable_pinned_host_transfer: bool = False
+    enable_pinned_host_transfer: bool | None = None
     enable_post_merge_validation: bool = True
+    use_replica_parallel: bool = True
+    min_slice_bytes_for_replica_parallel: int | None = None
+    max_replicas_for_replica_parallel: int | None = None
+    enable_replica_parallel_separate_folder: bool = False
+    enable_write_sharding_file: bool = True
+    array_metadata_store: array_metadata_store_lib.Store | None = (
+        array_metadata_store_lib.Store()
+    )
 
   @dataclasses.dataclass(frozen=True, kw_only=True)
   class Loading:
     """Options for loading arrays.
 
-    concurrent_bytes: Max concurrent bytes that are allowed for reading. Can
-      help to reduce the possibility of OOM's when large checkpoints are
-      restored.
-    enable_padding_and_truncation: If True, restoration allows silent
-      truncating/padding of arrays if the stored array shape does not match the
-      target shape. Otherwise, raises an error.
+    concurrent_bytes:
+      Max concurrent bytes that are allowed for reading. Can help to reduce the
+      possibility of OOM's when large checkpoints are restored.
+    enable_padding_and_truncation:
+      If True, restoration allows silent truncating/padding of arrays if the
+      stored array shape does not match the target shape. Otherwise, raises an
+      error.
     """
+
     concurrent_bytes: int | None = None
     enable_padding_and_truncation: bool = False
+    raise_array_data_missing_error: bool = True
 
   saving: Saving = dataclasses.field(default_factory=Saving)
   loading: Loading = dataclasses.field(default_factory=Loading)
@@ -253,8 +296,9 @@ class ArrayOptions:
 class CheckpointablesOptions:
   """Options used to configure `checkpointables` save/load behavior.
 
-  Primarily intended for registering custom `CheckpointableHandler` classes. You
-  can specify a registry directly, or use `create_with_handlers`. For example::
+  Primarily intended for registering custom :py:class:`.CheckpointableHandler`
+  classes. You can specify a registry directly, or use `create_with_handlers`.
+  For example::
 
     checkpointables_options = (
       ocp.options.CheckpointablesOptions.create_with_handlers(
@@ -272,18 +316,18 @@ class CheckpointablesOptions:
   by `BarHandler` and has the name `bar` can handled by this `BarHandler`.
 
   Recall that a global registry also exists, containing core handlers like
-  `PyTreeHandler` and `JsonHandler`. Use `ocp.handlers.register_handler` to
-  register a handler globally.
+  :py:class:`.PyTreeHandler` and :py:class:`.JsonHandler`. Use
+  `ocp.handlers.register_handler` to register a handler globally.
 
   Note that registration order matters. For example, if saving a dict containing
-  only strings, both `JsonHandler` and `PyTreeHandler` are capable of handling
-  this object, but `JsonHandler` will be selected first because it is registered
-  first.
+  only strings, both :py:class:`.JsonHandler` and :py:class:`.PyTreeHandler` are
+  capable of handling this object, but :py:class:`.JsonHandler` will be selected
+  first because it is registered first.
 
   Attributes:
-    registry: A `CheckpointableHandlerRegistry` that is used to resolve
-      `CheckpointableHandler` classes for each provided `checkpointable` during
-      saving and loading.
+    registry: A :py:class:`.CheckpointableHandlerRegistry` that is used to
+      resolve :py:class:`.CheckpointableHandler` classes for each provided
+      `checkpointable` during saving and loading.
   """
 
   registry: registration.CheckpointableHandlerRegistry = dataclasses.field(
@@ -304,3 +348,30 @@ class CheckpointablesOptions:
     for name, handler in named_handlers.items():
       registry.add(handler, name)
     return cls(registry=registry)
+
+
+@dataclasses.dataclass(frozen=True, kw_only=True)
+class PathwaysOptions:
+  """Options used to configure Pathways saving and loading.
+
+  Attributes:
+    checkpointing_impl: The implementation to use for Pathways checkpointing.
+  """
+
+  checkpointing_impl: pathways_types.CheckpointingImpl | None = None
+
+
+class CheckpointLayout(enum.Enum):
+  """The layout of the checkpoint.
+
+  By default, Orbax saves and loads checkpoints with its own layout. However,
+  support for other layouts is available, as a means of supporting
+  interoperatibility with other checkpointing libraries.
+
+  Currently supported layouts are:
+    ORBAX: Orbax's own layout.
+    SAFETENSORS: https://huggingface.co/docs/safetensors/en/index
+  """
+
+  ORBAX = 'Orbax'
+  SAFETENSORS = 'SafeTensors'
