@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,13 +20,12 @@ from __future__ import annotations
 
 import asyncio
 import dataclasses
-import json
 import threading
+import time
 from typing import Awaitable
 from unittest import mock
 
 from absl.testing import parameterized
-import aiofiles
 from etils import epath
 import flax
 import jax
@@ -34,10 +33,14 @@ from jax import numpy as jnp
 import numpy as np
 import optax
 from orbax.checkpoint import test_utils
+from orbax.checkpoint._src.multihost import multihost as multihost_v0
 from orbax.checkpoint._src.path import atomicity
 from orbax.checkpoint._src.serialization import serialization
 from orbax.checkpoint._src.tree import utils as tree_utils
 import orbax.checkpoint.experimental.v1 as ocp
+from orbax.checkpoint.experimental.v1._src.handlers import registration
+from orbax.checkpoint.experimental.v1._src.layout import checkpoint_layout
+from orbax.checkpoint.experimental.v1._src.path import async_utils
 from orbax.checkpoint.experimental.v1._src.path import types as path_types
 from orbax.checkpoint.experimental.v1._src.synchronization import multihost
 from orbax.checkpoint.experimental.v1._src.testing import array_utils as array_test_utils
@@ -48,6 +51,7 @@ from orbax.checkpoint.experimental.v1._src.tree import types as tree_types
 
 PyTree = tree_types.PyTree
 Path = path_types.Path
+InvalidLayoutError = checkpoint_layout.InvalidLayoutError
 
 create_sharded_array = array_test_utils.create_sharded_array
 create_numpy_pytree = array_test_utils.create_numpy_pytree
@@ -64,7 +68,7 @@ AbstractBar = handler_utils.AbstractBar
 
 ocp.handlers.register_handler(handler_utils.BazHandler)
 
-_original_create_paths = atomicity._create_paths
+_original_create_paths = async_utils._create_paths
 
 
 async def _sleep_and_create_paths(*args, **kwargs):
@@ -258,6 +262,17 @@ class SaveLoadTestBase:
             self, [jax_arr], ocp.load_pytree(self.directory / subdir)
         )
 
+      with self.subTest('jax_to_numpy_by_value'):
+        subdir = 'jax_to_np_by_value'
+        ocp.save_pytree(self.directory / subdir, [jax_arr])
+        test_utils.assert_tree_equal(
+            self,
+            [numpy_arr],
+            ocp.load_pytree(
+                self.directory / subdir, [np.array([], dtype=np.int64)]
+            ),
+        )
+
     def test_empty_array(self):
       value = np.ones(shape=(0,))
       with self.assertRaisesRegex(ValueError, 'zero size'):
@@ -440,9 +455,30 @@ class SaveLoadTestBase:
         with self.assertRaisesRegex(ValueError, 'User-provided restore item'):
           ocp.load_pytree(self.directory, abstract_pytree)
 
-    def test_force_overwrites(self):
+    def test_overwrites(self):
       ocp.save_pytree(self.directory, self.pytree)
-      ocp.save_pytree(self.directory, self.numpy_pytree, force=True)
+      with self.assertLogs(level='INFO') as cm:
+        ocp.save_pytree(self.directory, self.numpy_pytree, overwrite=True)
+        found_log = any(
+            'Specified `overwrite`: removing existing path.' in log
+            for log in cm.output
+        )
+        self.assertEqual(found_log, multihost.is_primary_host(0))
+      test_utils.assert_tree_equal(
+          self, self.numpy_pytree, ocp.load_pytree(self.directory)
+      )
+
+    def test_auto_overwrite_tmp_checkpoint(self):
+      ocp.save_pytree(self.directory, self.pytree)
+      if multihost.is_primary_host(0):
+        self.directory.rename(
+            self.directory.parent
+            / (self.directory.name + atomicity.TMP_DIR_SUFFIX)
+        )
+      test_utils.sync_global_processes(
+          'test_auto_overwrite_tmp_checkpoint:rename'
+      )
+      ocp.save_pytree(self.directory, self.numpy_pytree)
       test_utils.assert_tree_equal(
           self, self.numpy_pytree, ocp.load_pytree(self.directory)
       )
@@ -499,9 +535,7 @@ class SaveLoadTestBase:
       ocp.save_checkpointables(self.directory, checkpointables)
 
       with self.subTest('load_pytree'):
-        with self.assertRaisesRegex(
-            FileNotFoundError, 'must contain a subdirectory named "pytree"'
-        ):
+        with self.assertRaises(InvalidLayoutError):
           ocp.load_pytree(self.directory)
 
       with self.subTest('load_checkpointables'):
@@ -688,7 +722,9 @@ class SaveLoadTestBase:
           ocp.Context(checkpointables_options=checkpointables_options)
       )
       self.enter_context(
-          mock.patch.object(atomicity, '_create_paths', _sleep_and_create_paths)
+          mock.patch.object(
+              async_utils, '_create_paths', _sleep_and_create_paths
+          )
       )
       result = ocp.save_checkpointables_async(
           self.directory, {'foo': Foo(123, 'hi')}
@@ -699,45 +735,6 @@ class SaveLoadTestBase:
       self.assertTrue(self.directory.exists())
       self.assertLen(list(self.directory.parent.iterdir()), 1)
 
-    def test_async_directory_creation_failure(self):
-      class IncorrectFooHandler(handler_utils.FooHandler):
-
-        async def background_save(
-            self,
-            directory: path_types.PathAwaitingCreation,
-            checkpointable: Foo,
-        ):
-          directory = directory.path
-          # Note that we do not await contracted signals.
-          async with aiofiles.open(directory / 'foo.txt', 'w') as f:
-            contents = json.dumps(dataclasses.asdict(checkpointable))
-            await f.write(contents)
-
-        async def save(
-            self,
-            directory: path_types.PathAwaitingCreation,
-            checkpointable: Foo,
-        ) -> Awaitable[None]:
-          return self.background_save(
-              directory, Foo(**dataclasses.asdict(checkpointable))
-          )
-
-      checkpointables_options = (
-          ocp.options.CheckpointablesOptions.create_with_handlers(
-              IncorrectFooHandler,
-          )
-      )
-      self.enter_context(
-          ocp.Context(checkpointables_options=checkpointables_options)
-      )
-      self.enter_context(
-          mock.patch.object(atomicity, '_create_paths', _sleep_and_create_paths)
-      )
-      with self.assertRaisesRegex(
-          FileNotFoundError, 'No such file or directory'
-      ):
-        ocp.save_checkpointables(self.directory, {'foo': Foo(123, 'hi')})
-
     def test_background_error(self):
 
       async def raise_background_error(*args, **kwargs):
@@ -802,10 +799,175 @@ class SaveLoadTestBase:
       checkpointables = {'point': point}
       ocp.save_checkpointables(self.directory, checkpointables)
       self.assertTrue((self.directory / 'point' / 'foo.txt').exists())
-      loaded = ocp.load_checkpointables(
-          self.directory, {'point': handler_utils.Point(0, 0)}
+      with self.subTest('success'):
+        loaded = ocp.load_checkpointables(
+            self.directory, {'point': handler_utils.Point(0, 0)}
+        )
+        self.assertEqual(1, loaded['point'].x)
+        self.assertEqual(2, loaded['point'].y)
+      with self.subTest('No abstract_checkpointable'):
+        with self.assertRaisesRegex(
+            ValueError,
+            'To restore a `StatefulCheckpointable`, you must pass an instance'
+            ' of the object.',
+        ):
+          ocp.load_checkpointables(self.directory)
+
+    def test_checkpointable_missing_load(self):
+
+      @dataclasses.dataclass
+      class PointMissingLoad:
+        """Implements Point that violates StatefulCheckpointable protocol with missing load method."""
+
+        x: int
+        y: int
+
+        async def save(
+            self, directory: path_types.PathAwaitingCreation
+        ) -> Awaitable[None]:
+          return handler_utils.DataclassHandler().background_save(
+              directory,
+              self,
+              primary_host=handler_utils.context_lib.get_context().multiprocessing_options.primary_host,
+          )
+
+      point = PointMissingLoad(1, 2)
+      checkpointables = {'point': point}
+      with self.assertRaisesRegex(
+          registration.NoEntryError,
+          'Could not identify a valid handler for the checkpointable: "point"',
+      ):
+        ocp.save_checkpointables(self.directory, checkpointables)
+
+    def test_load_does_not_exist(self):
+      with self.assertRaises(InvalidLayoutError):
+        ocp.load_checkpointables(self.directory / 'foobar')
+
+    def test_load_tmp_checkpoint(self):
+      tmp_checkpoint_dir = self.directory / 'foo.orbax-checkpoint-tmp-1234'
+      tmp_checkpoint_dir.mkdir(parents=True, exist_ok=True)
+      with self.assertRaises(InvalidLayoutError):
+        ocp.load_checkpointables(tmp_checkpoint_dir)
+
+    def test_async_save_completes_without_result(self):
+      self.enter_context(
+          mock.patch.object(
+              async_utils, '_create_paths', _sleep_and_create_paths
+          )
       )
-      self.assertEqual(1, loaded['point'].x)
-      self.assertEqual(2, loaded['point'].y)
-      # TODO(yaning) Add failure case where abstract_checkpointable is not
-      # provided.
+      ocp.save_checkpointables_async(
+          self.directory, dict(baz=handler_utils.Baz(123, 'hi'))
+      )
+      self.assertFalse(self.directory.exists())
+      time.sleep(3)
+      self.assertTrue(self.directory.exists())
+
+    def test_partial_restore_placeholder(self):
+      ocp.save_pytree(self.directory, self.pytree)
+
+      reference_pytree = jax.tree.map(lambda x: x, self.abstract_pytree)
+      reference_pytree['b'] = PLACEHOLDER
+      reference_pytree['c']['e'] = PLACEHOLDER
+      reference_pytree['x'] = PLACEHOLDER
+
+      expected = {
+          'a': self.pytree['a'],
+          'b': PLACEHOLDER,
+          'c': {
+              'a': self.pytree['c']['a'],
+              'e': PLACEHOLDER,
+          },
+          'x': PLACEHOLDER,
+          'y': self.pytree['y'],
+      }
+
+      loaded = ocp.load_pytree(self.directory, reference_pytree)
+      test_utils.assert_tree_equal(self, expected, loaded)
+
+    def test_partial_restore_omission(self):
+      ocp.save_pytree(self.directory, self.pytree)
+
+      reference_pytree = jax.tree.map(lambda x: x, self.abstract_pytree)
+      del reference_pytree['b']
+      del reference_pytree['c']['e']
+      del reference_pytree['x']
+
+      expected = {
+          'a': self.pytree['a'],
+          'c': {
+              'a': self.pytree['c']['a'],
+          },
+          'y': self.pytree['y'],
+      }
+
+      with ocp.Context(
+          pytree_options=ocp.options.PyTreeOptions(
+              loading=ocp.options.PyTreeOptions.Loading(
+                  partial_load=True,
+              )
+          )
+      ):
+        loaded = ocp.load_pytree(self.directory, reference_pytree)
+
+      test_utils.assert_tree_equal(self, expected, loaded)
+
+    @parameterized.parameters(True, False)
+    def test_save_with_global_mesh(self, use_same_mesh: bool):
+      if use_same_mesh:
+        devices = np.asarray(jax.devices())
+        axis_names = ('x',)
+      else:
+        devices = np.asarray(jax.devices()[:4])
+        axis_names = ('x',)
+      mesh = jax.sharding.Mesh(devices, axis_names)
+      jax.sharding.set_mesh(mesh)
+
+      ocp.save_pytree(self.directory, self.pytree)
+      loaded = ocp.load_pytree(self.directory, self.abstract_pytree)
+      test_utils.assert_tree_equal(self, self.pytree, loaded)
+
+    @parameterized.parameters((3,), (8,))
+    def test_primary_host_background_error(self, timeout):
+      def _assert_false(*args, **kwargs):
+        del args, kwargs
+        assert False
+
+      with (
+          mock.patch.object(
+              atomicity, '_create_tmp_directory', new=_assert_false
+          ),
+          mock.patch.object(
+              multihost,
+              'coordination_timeout',
+              return_value=timeout,
+          ),
+          mock.patch.object(
+              multihost_v0,
+              'coordination_timeout',
+              return_value=timeout,
+          ),
+      ):
+        r = ocp.save_pytree_async(self.directory, self.pytree)
+        start = time.time()
+        if multihost.is_primary_host(primary_host=0):
+          with self.assertRaises(AssertionError):
+            r.result()
+          # Error should be raised quickly on primary host.
+          self.assertLessEqual(time.time() - start, timeout - 1)
+        else:
+          with self.assertRaises(BaseException):
+            r.result()
+          self.assertGreaterEqual(time.time() - start, timeout)
+
+    def test_save_checkpointables_directory_consistency_failure(self):
+      if jax.process_count() <= 1:
+        self.skipTest('Skip test for single-process.')
+
+      # Generates a random directory path for each process.
+      # This will cause a directory mismatch error when running multi-process.
+      directory = epath.Path(self.create_tempdir().full_path)
+
+      with self.assertRaisesRegex(
+          ValueError, 'Directory path mismatch in multi-process save'
+      ):
+        ocp.save_pytree(directory, self.pytree)
