@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,16 +17,20 @@
 from typing import Any
 import jax
 import numpy as np
-import orbax.checkpoint as ocp
+from orbax.checkpoint._src.tree import types
 
-JsonType = ocp.tree.JsonType
+JsonType = types.JsonType
 
 PyTree = Any
-PyTreeOf = ocp.tree.PyTreeOf
+PyTreeOf = types.PyTreeOf
 
-PyTreeKey = ocp.tree.PyTreeKey
-PyTreeKeyPath = ocp.tree.PyTreePath
+PyTreeKey = types.PyTreeKey
+PyTreeKeyPath = types.PyTreePath
 
 ScalarType = int | float | bool
 LeafType = jax.Array | np.ndarray | str | ScalarType | Any
 AbstractLeafType = Any  # TODO(cpgaffney): Add a type for abstract leaves.
+
+JsonType = types.JsonType
+
+PLACEHOLDER = ...
