@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,7 +21,9 @@ import ml_dtypes
 import numpy as np
 from orbax.checkpoint import test_utils
 from orbax.checkpoint._src.handlers import pytree_checkpoint_handler
+from orbax.checkpoint._src.handlers import standard_checkpoint_handler
 from orbax.checkpoint._src.handlers import standard_checkpoint_handler_test_utils
+from orbax.checkpoint._src.serialization import type_handler_registry
 from orbax.checkpoint._src.serialization import type_handlers
 import tensorstore as ts
 
@@ -158,7 +160,7 @@ class SingleHostTest(parameterized.TestCase):
     ):
       handler.save(
           self.ckpt_dir,
-          args=standard_checkpoint_handler_test_utils.StandardSaveArgs(x),
+          args=standard_checkpoint_handler.StandardSaveArgs(x),
       )
 
   @parameterized.product(
@@ -167,6 +169,7 @@ class SingleHostTest(parameterized.TestCase):
           ml_dtypes.bfloat16,
           np.dtype('float32'),
           np.dtype('float64'),
+          ml_dtypes.float8_e3m4,
           ml_dtypes.float8_e5m2fnuz,
           ml_dtypes.float8_e5m2,
           ml_dtypes.float8_e4m3fnuz,
@@ -218,7 +221,7 @@ class SingleHostTest(parameterized.TestCase):
         use_replica_parallel=True,
     )
     handler = PyTreeCheckpointHandler(
-        type_handler_registry=type_handlers.create_type_handler_registry(
+        type_handler_registry=type_handler_registry.create_type_handler_registry(
             (jax.Array, array_handler),
         ),
     )
