@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,12 +26,13 @@ import copy
 import inspect
 import time
 import typing
-from typing import Any, List, Optional, Set
+from typing import Any, Dict, List, Optional, Set
 from unittest import mock
 
 from absl import logging
 from etils import epath
 import jax
+from jax.experimental import multihost_utils
 from jax.experimental import pjit
 import jax.numpy as jnp
 import numpy as np
@@ -43,19 +44,22 @@ from orbax.checkpoint._src.metadata import step_metadata_serialization
 from orbax.checkpoint._src.multihost import multihost
 from orbax.checkpoint._src.multihost import multislice
 from orbax.checkpoint._src.path import atomicity
+from orbax.checkpoint._src.path import gcs_utils
 from orbax.checkpoint._src.path import step as step_lib
+from orbax.checkpoint._src.serialization import limits
 from orbax.checkpoint._src.serialization import replica_slices
-from orbax.checkpoint._src.serialization import serialization
 from orbax.checkpoint._src.serialization import tensorstore_utils as ts_utils
+from orbax.checkpoint._src.serialization import type_handler_registry
 from orbax.checkpoint._src.serialization import type_handlers
 from orbax.checkpoint._src.tree import utils as tree_utils
+import tensorstore as ts
 
 
 PyTree = Any
 
 
 def sync_global_processes(name: str):
-  jax.experimental.multihost_utils.sync_global_devices(name)
+  multihost_utils.sync_global_devices(name)
 
 
 def save_fake_tmp_dir(
@@ -72,7 +76,7 @@ def save_fake_tmp_dir(
   step_final_directory = directory / (step_prefix + str(step))
   step_tmp_directory = (
       step_final_directory
-      if step_lib.is_gcs_path(step_final_directory)
+      if gcs_utils.is_gcs_path(step_final_directory)
       else atomicity._get_tmp_directory(step_final_directory)
   )
   create_tmp_directory(step_tmp_directory, step_final_directory)
@@ -80,7 +84,7 @@ def save_fake_tmp_dir(
   item_final_directory = step_tmp_directory / item
   item_tmp_directory = (
       item_final_directory
-      if step_lib.is_gcs_path(item_final_directory)
+      if gcs_utils.is_gcs_path(item_final_directory)
       else atomicity._get_tmp_directory(item_final_directory)
   )
   create_tmp_directory(item_tmp_directory, item_final_directory)
@@ -99,7 +103,7 @@ def create_tmp_directory(
     primary_host: Optional[int] = 0,
     active_processes: Optional[Set[int]] = None,
     barrier_sync_key_prefix: Optional[str] = None,
-    path_permission_mode: int = step_lib.WORLD_READABLE_MODE,
+    path_permission_mode: Optional[int] = None,
     metadata_store: Optional[checkpoint_metadata.MetadataStore] = None,
 ) -> epath.Path:
   """Creates a non-deterministic tmp directory for saving for given `final_dir`.
@@ -133,12 +137,12 @@ def create_tmp_directory(
           prefix=barrier_sync_key_prefix,
           suffix=f'{final_dir.name}',
       ),
-      timeout=multihost.DIRECTORY_CREATION_TIMEOUT,
+      timeout=multihost.coordination_timeout(),
       processes=active_processes,
   )
   if multihost.is_primary_host(primary_host):
     if tmp_dir.exists():
-      if step_lib.is_tmp_checkpoint(tmp_dir):
+      if step_lib.is_path_temporary(tmp_dir):
         logging.warning(
             'Attempted to create temporary directory %s which already exists.'
             ' Removing existing directory since it is not finalized.',
@@ -167,7 +171,7 @@ def create_tmp_directory(
           prefix=barrier_sync_key_prefix,
           suffix=f'{final_dir.name}',
       ),
-      timeout=multihost.DIRECTORY_CREATION_TIMEOUT,
+      timeout=multihost.coordination_timeout(),
       processes=active_processes,
   )
   return tmp_dir
@@ -228,9 +232,9 @@ def assert_array_equal(testclass, v_expected, v_actual):
       for shard_expected, shard_actual in zip(
           v_expected.addressable_shards, v_actual.addressable_shards
       ):
-        np.testing.assert_array_equal(shard_expected.data, shard_actual.data)
+        np.testing.assert_array_equal(shard_actual.data, shard_expected.data)
   elif isinstance(v_expected, (np.ndarray, jnp.ndarray)):
-    np.testing.assert_array_equal(v_expected, v_actual)
+    np.testing.assert_array_equal(v_actual, v_expected)
   else:
     testclass.assertEqual(v_expected, v_actual)
 
@@ -249,6 +253,13 @@ def assert_tree_equal(testclass, expected, actual):
   jax.tree.map(_eq, expected, actual, is_leaf=lambda x: x is None)
 
 
+def assert_tree_same_structure(testclass, expected, actual):
+  """Asserts that two PyTrees have the same structure."""
+  expected_structure = jax.tree.structure(expected)
+  actual_structure = jax.tree.structure(actual)
+  testclass.assertEqual(expected_structure, actual_structure)
+
+
 def setup_pytree(add: int = 0):
   """Creates a numpy PyTree for testing."""
   pytree = {
@@ -266,9 +277,11 @@ def setup_pytree(add: int = 0):
 def setup_sharded_pytree(
     pytree: Optional[pytree_checkpoint_handler.PyTree] = None,
     reverse_devices: bool = False,
+    devices: Optional[list[jax.Device]] = None,
 ):
   """Creates a PyTree of sharded arrays for testing."""
-  devices = jax.devices()
+  if devices is None:
+    devices = jax.devices()
   num_devices = len(devices)
   if reverse_devices:
     devices = np.asarray(list(reversed(devices)))
@@ -357,37 +370,39 @@ def get_fake_global_mesh_for_slices(
 def select_single_replica(
     arrays: List[jax.Array], global_mesh: jax.sharding.Mesh
 ) -> List[jax.Array]:
-  """Returns arrays sharded over single slice."""
-  slice_devices = multislice.local_slice_devices(
+  """Returns arrays sharded over single replica."""
+  replica_devices = multislice.local_replica_devices(
       global_mesh, replica_axis_index=0
   )
-  single_slice_mesh = jax.sharding.Mesh(
-      slice_devices, global_mesh.axis_names[1:]
+  single_replica_mesh = jax.sharding.Mesh(
+      replica_devices, global_mesh.axis_names[1:]
   )
 
-  def _get_single_slice_sharding(arr: jax.Array):
+  def _get_single_replica_sharding(arr: jax.Array):
     sharding = typing.cast(jax.sharding.NamedSharding, arr.sharding)
     return jax.sharding.NamedSharding(
-        single_slice_mesh,
+        single_replica_mesh,
         jax.sharding.PartitionSpec(*sharding.spec),  # exclude 'replica' axis
     )
 
-  single_slice_shardings = jax.tree.map(
-      _get_single_slice_sharding,
+  single_replica_shardings = jax.tree.map(
+      _get_single_replica_sharding,
       arrays,
   )
 
-  def _make_single_slice_array(
-      arr: jax.Array, single_slice_sharding: jax.sharding.NamedSharding
+  def _make_single_replica_array(
+      arr: jax.Array, single_replica_sharding: jax.sharding.NamedSharding
   ):
     data = [
         arr.addressable_data(idx) for idx in range(len(arr.addressable_shards))
     ]
     return jax.make_array_from_single_device_arrays(
-        arr.shape, single_slice_sharding, data
+        arr.shape, single_replica_sharding, data
     )
 
-  return jax.tree.map(_make_single_slice_array, arrays, single_slice_shardings)
+  return jax.tree.map(
+      _make_single_replica_array, arrays, single_replica_shardings
+  )
 
 
 def is_leaf(x):
@@ -413,7 +428,10 @@ def create_sharded_array(arr, mesh, mesh_axes):
 def print_directory(directory: epath.PathLike, level: int = 0):
   """Prints a directory tree for debugging purposes."""
   directory = epath.Path(directory)
-  assert directory.is_dir()
+  if not directory.exists():
+    raise ValueError(f'Directory {directory} does not exist.')
+  if not directory.is_dir():
+    raise ValueError(f'Directory {directory} is not a directory.')
   level_str = '..' * level
   if level == 0:
     logging.info('Printing directory tree: %s/', directory)
@@ -518,19 +536,21 @@ class ErrorSaveArgs(pytree_checkpoint_handler.PyTreeSaveArgs):
 
 
 @checkpoint_args.register_with_handler(ErrorCheckpointHandler, for_restore=True)
-class ErrorRestoreArgs(pytree_checkpoint_handler.PyTreeSaveArgs):
+class ErrorRestoreArgs(pytree_checkpoint_handler.PyTreeRestoreArgs):
   pass
 
 
 @contextlib.contextmanager
 def register_type_handler(ty, handler, func):
   """Registers new func for type, and restores original handler when done."""
-  original_handler = type_handlers.get_type_handler(ty)
-  type_handlers.register_type_handler(ty, handler, func=func, override=True)
+  original_handler = type_handler_registry.get_type_handler(ty)
+  type_handler_registry.register_type_handler(
+      ty, handler, func=func, override=True
+  )
   try:
     yield
   finally:
-    type_handlers.register_type_handler(
+    type_handler_registry.register_type_handler(
         ty, original_handler, func=func, override=True
     )
 
@@ -538,12 +558,14 @@ def register_type_handler(ty, handler, func):
 @contextlib.contextmanager
 def global_type_handler_registry_context():
   """Context manager for changing the GLOBAL_TYPE_HANDLER_REGISTRY."""
-  original_type_handlers = copy.deepcopy(type_handlers._DEFAULT_TYPE_HANDLERS)
+  original_type_handlers = copy.deepcopy(
+      type_handler_registry._DEFAULT_TYPE_HANDLERS
+  )
   try:
     yield
   finally:
     for original_type, original_handler in original_type_handlers:
-      type_handlers.GLOBAL_TYPE_HANDLER_REGISTRY.add(
+      type_handler_registry.GLOBAL_TYPE_HANDLER_REGISTRY.add(
           original_type, original_handler, override=True
       )
 
@@ -551,16 +573,18 @@ def global_type_handler_registry_context():
 @contextlib.contextmanager
 def ocdbt_checkpoint_context(use_ocdbt: bool, ts_context: Any):
   """Use OCDBT driver within context."""
-  original_type_handlers = copy.deepcopy(type_handlers._DEFAULT_TYPE_HANDLERS)
+  original_type_handlers = copy.deepcopy(
+      type_handler_registry._DEFAULT_TYPE_HANDLERS
+  )
   if use_ocdbt:
-    type_handlers.register_standard_handlers_with_options(
+    type_handler_registry.register_standard_handlers_with_options(
         use_ocdbt=use_ocdbt, ts_context=ts_context
     )
   try:
     yield
   finally:
     for original_type, original_handler in original_type_handlers:
-      type_handlers.GLOBAL_TYPE_HANDLER_REGISTRY.add(
+      type_handler_registry.GLOBAL_TYPE_HANDLER_REGISTRY.add(
           original_type, original_handler, override=True
       )
 
@@ -621,18 +645,22 @@ def ensure_atomic_save(
 ):
   """Wrapper around TemporaryPath.finalize for testing."""
   if temp_ckpt_dir == final_ckpt_dir:
-    atomicity.CommitFileTemporaryPath(
-        temp_ckpt_dir,
-        final_ckpt_dir,
-        checkpoint_metadata_store=metadata_store,
-    ).finalize(
+    asyncio.run(
+        atomicity.CommitFileTemporaryPath(
+            temp_ckpt_dir,
+            final_ckpt_dir,
+            checkpoint_metadata_store=metadata_store,
+        ).finalize(
+        )
     )
   else:
-    atomicity.AtomicRenameTemporaryPath(
-        temp_ckpt_dir,
-        final_ckpt_dir,
-        checkpoint_metadata_store=metadata_store,
-    ).finalize(
+    asyncio.run(
+        atomicity.AtomicRenameTemporaryPath(
+            temp_ckpt_dir,
+            final_ckpt_dir,
+            checkpoint_metadata_store=metadata_store,
+        ).finalize(
+        )
     )
 
 
@@ -680,7 +708,7 @@ def _replica_devices(device_array: np.ndarray, replica_axis_idx: int):
   return np.expand_dims(replica_result, axis=replica_axis_idx)
 
 
-class TestLimitInFlightBytes(serialization.LimitInFlightBytes):
+class TestLimitInFlightBytes(limits.LimitInFlightBytes):
   """Limits in-flight bytes when reading/writing checkpoints per process."""
 
   def __init__(self, limit_bytes: int, sleep_time: float):
@@ -752,13 +780,19 @@ def assert_every_n_is_x_apart(testclass, values, n, x):
 
 
 def get_expected_chunk_shape(
-    arr: jax.Array, *, use_replica_parallel: bool = True
+    arr: jax.Array,
+    *,
+    use_replica_parallel: bool = True,
+    max_replicas_for_replica_parallel: Optional[bool] = None,
+    pathways_support_replica_parallel: bool = False,
 ) -> tuple[int, ...]:
   """Expected chunk shape for an array, accounting for replica-parallel."""
   if not use_replica_parallel:
     return arr.sharding.shard_shape(arr.shape)
-  _, local_shape = (
-      replica_slices.calculate_replica_parallel_axis_and_local_shape(arr)
+  _, local_shape, _ = (
+      replica_slices.calculate_replica_parallel_axis_and_local_shape(
+          arr, max_replicas_for_replica_parallel
+      )
   )
   if local_shape is None:
     local_shape = arr.sharding.shard_shape(arr.shape)
@@ -778,3 +812,54 @@ def filter_metadata_fields(
     return result
 
   return jax.tree.map(_include, pytree)
+
+
+def _get_simple_tensorstore_read_spec(
+    checkpoint_directory: epath.Path,
+    param_name: str,
+    use_zarr3: bool,
+    use_ocdbt: bool,
+) -> Dict[str, Any]:
+  """Returns a simple TensorStore read spec for testing."""
+  if use_ocdbt:
+    ts_spec = {
+        'driver': 'zarr3' if use_zarr3 else 'zarr',
+        'kvstore': {
+            'driver': 'ocdbt',
+            'base': f'file://{checkpoint_directory}',
+            'path': param_name,
+        },
+    }
+  else:
+    ts_spec = {
+        'driver': 'zarr3' if use_zarr3 else 'zarr',
+        'kvstore': {
+            'driver': 'file',
+            'path': f'{checkpoint_directory/ param_name}',
+        },
+    }
+
+  return ts_spec
+
+
+def is_compression_used(
+    checkpoint_directory: epath.Path,
+    param_name: str,
+    use_zarr3: bool,
+    use_ocdbt: bool,
+):
+  """Returns True if compression is used for a given paramin in Tensorstore."""
+  ts_spec = _get_simple_tensorstore_read_spec(
+      checkpoint_directory, param_name, use_zarr3, use_ocdbt
+  )
+  read_spec = ts.open(ts_spec).result().spec().to_json()
+
+  if use_zarr3:
+    # check if zstd is in the codecs
+    for codec in read_spec['metadata']['codecs'][0]['configuration']['codecs']:
+      if codec['name'] == 'zstd':
+        return True
+    return False
+
+  else:
+    return read_spec['metadata']['compressor'] is not None
