@@ -1,4 +1,4 @@
-# Copyright 2024 The Orbax Authors.
+# Copyright 2025 The Orbax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,8 +17,6 @@
 NOTE: Functions in this file are deprecated in favor of corresponding functions
 in sub-modules. Please use those functions instead, and do not add new
 functions here.
-
-TODO(b/266449081) Increase unit test coverage.
 """
 
 from typing import Any
@@ -27,7 +25,8 @@ from etils import epath
 import jax
 import numpy as np
 from orbax.checkpoint._src.multihost import multihost
-from orbax.checkpoint._src.path import async_utils
+from orbax.checkpoint._src.path import atomicity
+from orbax.checkpoint._src.path import gcs_utils
 from orbax.checkpoint._src.path import step as step_lib
 from orbax.checkpoint._src.tree import utils as tree_utils
 
@@ -49,20 +48,14 @@ broadcast_one_to_all = multihost.broadcast_one_to_all
 reached_preemption = multihost.reached_preemption
 is_primary_host = multihost.is_primary_host
 
-
-async_makedirs = async_utils.async_makedirs
-async_write_bytes = async_utils.async_write_bytes
-async_exists = async_utils.async_exists
-
-is_gcs_path = step_lib.is_gcs_path
+is_gcs_path = gcs_utils.is_gcs_path
 checkpoint_steps = step_lib.checkpoint_steps
 any_checkpoint_step = step_lib.any_checkpoint_step
-is_checkpoint_finalized = step_lib.is_checkpoint_finalized
-is_tmp_checkpoint = step_lib.is_tmp_checkpoint
-tmp_checkpoints = step_lib.tmp_checkpoints
-cleanup_tmp_directories = step_lib.cleanup_tmp_directories
+is_checkpoint_finalized = step_lib.is_path_finalized
+is_tmp_checkpoint = step_lib.is_path_temporary
+tmp_checkpoints = step_lib.all_temporary_paths
 get_save_directory = step_lib.get_save_directory
-record_saved_duration = step_lib.record_saved_duration
+record_saved_duration = atomicity.record_saved_duration
 step_from_checkpoint_name = step_lib.step_from_checkpoint_name
 checkpoint_steps_paths = step_lib.checkpoint_steps_paths
 
@@ -146,6 +139,10 @@ def pytree_structure(directory: epath.PathLike) -> PyTree:
       continue
     if k.name == '_METADATA':
       continue
+    # array_metadatas is not a checkpoint param. Only used when ocdbt is used.
+    # ocdbt is still disabled in some projects like paxml.
+    if k.name == 'array_metadatas':
+      continue
     tree = add_nested_key(tree, k.name.split('.'), k.name)
   return tree
 
@@ -182,7 +179,7 @@ def fully_replicated_host_local_array_to_global_array(
       [shard.data for shard in arr.addressable_shards],
       key=lambda x: list(x.devices())[0].id,
   )
-  if jax.config.jax_pmap_no_rank_reduction and global_shape != arr.shape:
+  if global_shape != arr.shape:
     dbs = [s[0] for s in dbs]
     global_shape = global_shape[1:]
   result = jax.make_array_from_single_device_arrays(
